@@ -9,7 +9,7 @@
 
 #include "adm_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/admin/kdb_destroy.c,v 1.4 1995/10/27 10:55:26 bg Exp $");
+RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/admin/kdb_destroy.c,v 1.5 1996/02/07 10:47:48 d91-jda Exp $");
 
 int
 main(void)
@@ -39,15 +39,18 @@ main(void)
 
     if (answer[0] == 'y' || answer[0] == 'Y') {
 #ifdef HAVE_NEW_DB
-	if (unlink(file) == 0)
+      if (unlink(file) == 0)
 #else
 	if (unlink(file1) == 0 && unlink(file2) == 0)
 #endif
+	  {
 	    fprintf(stderr, "Database deleted at %s\n", DBM_FILE);
+	    return 0;
+	  }
 	else
 	    fprintf(stderr, "Database cannot be deleted at %s\n",
 		    DBM_FILE);
     } else
 	fprintf(stderr, "Database not deleted.\n");
-    exit(1);
+    return 1;
 }
