@@ -10,12 +10,12 @@
 
 #include "adm_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/admin/kdb_init.c,v 1.9 1995/10/27 10:55:28 bg Exp $");
+RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/admin/kdb_init.c,v 1.11 1996/02/10 12:59:15 assar Exp $");
 
 enum ap_op {
     NULL_KEY,			/* setup null keys */
     MASTER_KEY,                 /* use master key as new key */
-    RANDOM_KEY,			/* choose a random key */
+    RANDOM_KEY			/* choose a random key */
 };
 
 char   *progname;
@@ -137,7 +137,8 @@ main(int argc, char **argv)
     printf("It is important that you NOT FORGET this password.\n");
     fflush(stdout);
 
-    if (kdb_get_master_key (TRUE, &master_key, master_key_schedule) != 0) {
+    if (kdb_get_master_key (KDB_GET_TWICE, &master_key,
+			    master_key_schedule) != 0) {
       fprintf (stderr, "Couldn't read master key.\n");
       exit (-1);
     }
