@@ -1,30 +1,67 @@
 #include "bsd_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/appl/bsd/forkpty.c,v 1.10 1995/10/24 14:02:56 bg Exp $");
+#ifndef HAVE_FORKPTY
+
+RCSID("$Id: forkpty.c,v 1.29 1996/02/11 17:21:40 bg Exp $");
+
+/* Systems where rlogind has been tested */
+#if 1 || (defined(sun) && !defined(__svr4__)) || defined(__hpux) || defined(__linux) || defined(__sgi__) || defined(__NetBSD__) || defined(_AIX)
+static int forkpty_ok = 1;
+#else
+static int forkpty_ok = 0;
+#endif
 
 #include <termios.h>
 
+#if defined(HAVE_SYS_IOCTL_H) && SunOS != 4
+#include <sys/ioctl.h>
+#endif
+
 #ifdef HAVE_STROPTS_H
 #include <stropts.h>
 #endif
 
-#if (defined(sun) && !defined(__svr4__)) || defined(__hpux)
-#define FORKPTY_OK
+#ifndef HAVE_PTSNAME
+static char *ptsname(int fd)
+{
+#ifdef HAVE_TTYNAME
+  return ttyname(fd);
+#else
+  return NULL;
+#endif
+}
 #endif
 
-#ifndef FORKPTY_OK
-int
-forkpty(int *amaster, char *name, struct termios *termp, struct winsize *winp)
-{
-    fatal(0, "Protocol not yet supported, use telnet", 0);
-    return -1;
-}
-#else
+#ifndef HAVE_GRANTPT
+#define grantpt(fdm) (0)
+#endif
 
-#ifdef HAVE_PTSNAME
-extern char *ptsname(int fd);
-#else
-static char *ptsname(int fd) { return NULL; }
+#ifndef HAVE_UNLOCKPT
+#define unlockpt(fdm) (0)
+#endif
+
+#ifndef HAVE_VHANGUP
+void vhangup() { return; }
+#endif
+
+#ifndef HAVE_REVOKE
+static
+void
+revoke(char *line)
+{
+    int slave;
+    RETSIGTYPE (*ofun)();
+
+    if ( (slave = open(line, O_RDWR)) < 0)
+	return;
+    
+    ofun = signal(SIGHUP, SIG_IGN);
+    vhangup();
+    signal(SIGHUP, ofun);
+#if 0
+    close(slave);		/* This seems not to work on SGI */
+#endif
+}
 #endif
 
 static int
@@ -34,39 +71,35 @@ ptym_open(char *pts_name, int *streams_pty)
     char *ptr1, *ptr2;
 
     /* Try clone device master ptys */
-    *streams_pty = 1;
-    fdm = open("/dev/ptmx", O_RDWR);
-    if (fdm < 0)
-	fdm = open("/dev/ptc", O_RDWR);
-    if (fdm < 0)
-	fdm = open("/dev/ptm", O_RDWR);
-    if (fdm < 0)
-	fdm = open("/dev/ptym/clone", O_RDWR);
+    char *clone[] = { "/dev/ptc", "/dev/ptmx", "/dev/ptm", 
+		      "/dev/ptym/clone", 0 };
 
+    char **q;
+    *streams_pty = 1;
+    for(q=clone; *q; q++){
+      fdm=open(*q, O_RDWR);
+      if(fdm >= 0)
+	break;
+    }
     if (fdm >= 0) {
-	if ( (ptr1 = ptsname(fdm)) != NULL) /* Get slave's name */
+        if ( (ptr1 = ptsname(fdm)) != NULL) /* Get slave's name */
 	    strcpy(pts_name, ptr1); /* Return name of slave */  
 	else {
 	    close(fdm);
 	    return(-4);
 	}
-#ifdef HAVE_GRANTPT
 	if (grantpt(fdm) < 0) {	/* Grant access to slave */
 	    close(fdm);
 	    return(-2);
 	}
-#endif
-#ifdef HAVE_UNLOCKPT
 	if (unlockpt(fdm) < 0) {	/* Clear slave's lock flag */
 	    close(fdm);
 	    return(-3);
 	}
-#endif
 	return(fdm);			/* return fd of master */
     }
     
-    /* Try BSD style master ptys */
-    *streams_pty = 0;
+    /* HP-UX has a different naming of their pty's */
 #ifdef __hpux
     strcpy(pts_name, "/dev/ptym/ptyXY");
 #define pts_name_X pts_name[13]
@@ -76,6 +109,9 @@ ptym_open(char *pts_name, int *streams_pty)
 #define pts_name_X pts_name[8]
 #define pts_name_Y pts_name[9]
 #endif
+
+    /* Try BSD style master ptys */
+    *streams_pty = 0;
     /* array index: 0123456789 (for references in following code) */
     for (ptr1 = "pqrstuvwxyzPQRST"; *ptr1 != 0; ptr1++)
 	{
@@ -89,7 +125,7 @@ ptym_open(char *pts_name, int *streams_pty)
 		    if ( (fdm = open(pts_name, O_RDWR)) < 0)
 			continue; /* Try next pty device */
 			    
-#if defined(sun) && !defined(__svr4__)
+#if SunOS == 4
 		    /* Avoid a bug in SunOS4 ttydriver */
 		    if (fdm > 0) {
 			int pgrp;
@@ -116,6 +152,33 @@ ptym_open(char *pts_name, int *streams_pty)
     return -1;
 }
 
+static int
+maybe_push_modules(int fd, char **modules)
+{
+#ifdef I_PUSH
+  char **p;
+  int err;
+
+  for(p=modules; *p; p++){
+    err=ioctl(fd, I_FIND, *p);
+    if(err == 1)
+      break;
+    if(err < 0 && errno != EINVAL)
+      return -17;
+    /* module not pushed or does not exist */
+  }
+  /* p points to null or to an already pushed module, now push all
+     modules before this one */
+
+  for(p--; p >= modules; p--){
+    err = ioctl(fd, I_PUSH, *p);
+    if(err < 0 && errno != EINVAL)
+      return -17;
+  }
+#endif
+  return 0;
+}
+
 static int
 ptys_open(int fdm, char *pts_name, int streams_pty)
 {
@@ -127,34 +190,24 @@ ptys_open(int fdm, char *pts_name, int streams_pty)
 	    close(fdm);
 	    return(-5);
 	}
-#ifdef I_PUSH
-	if (ioctl(fds, I_PUSH, "ptem") < 0) {
+
+	{
+	  char *ttymodules[] = { "ttcompat", "ldterm", "ptem", NULL };
+	  char *ptymodules[] = { "pckt", NULL };
+	  
+	  if(maybe_push_modules(fds, ttymodules)<0){
 	    close(fdm);
 	    close(fds);
-	    return(-6);
-	}
-	if (ioctl(fds, I_PUSH, "ldterm") < 0) {
+	    return -6;
+	  }
+	  if(maybe_push_modules(fdm, ptymodules)<0){
 	    close(fdm);
 	    close(fds);
-	    return(-7);
+	    return -7;
+	  }
 	}
-	/*
-	 * Not all systems have (or need) modules ttcompat and pckt so
-	 * don't flag it as a fatal error if they don't exist.
-	 */
-	if (ioctl(fds, I_PUSH, "ttcompat") < 0 && errno != EINVAL) {
-	    close(fdm);
-	    close(fds);
-	    return(-8);
-	}
-	if (ioctl(fds, I_PUSH, "pckt") < 0 && errno != EINVAL) {
-	    close(fdm);
-	    close(fds);
-	    return(-9);
-	}
-#endif /* I_PUSH */
     } else {
-	/* BSD style slave ptys */
+        /* BSD style slave ptys */
 	struct group *grptr;
 	int gid;
 	if ( (grptr = getgrnam("tty")) != NULL)
@@ -181,18 +234,31 @@ forkpty(int *ptrfdm, char *slave_name, struct termios *slave_termios, struct win
     pid_t	pid;
     char	pts_name[20];
 
+    if (!forkpty_ok)
+        fatal(0, "Protocol not yet supported, use telnet", 0);
+
     if ( (fdm = ptym_open(pts_name, &streams_pty)) < 0)
 	return -1;
 
     if (slave_name != NULL)
 	strcpy(slave_name, pts_name);	/* Return name of slave */
 
+    revoke(slave_name);
     if ( (pid = fork()) < 0)
 	return(-1);
     else if (pid == 0) {		/* Child */
 	if (setsid() < 0)
 	    return -1;
 
+#if defined(NeXT)
+	/* The NeXT is severely broken, this makes things slightly
+	 * better but we still doesn't get a working pty. If there
+	 * where a TIOCSCTTY we could perhaps fix things but... The
+	 * same problem also exists in xterm! */
+	if (setpgrp(0, 0) < 0)
+	    fatal(0, "NeXT kludge failed setpgrp", errno);
+#endif
+
 	/* SVR4 acquires controlling terminal on open() */
 	if ( (fds = ptys_open(fdm, pts_name, streams_pty)) < 0)
 	    return -1;
@@ -229,5 +295,4 @@ forkpty(int *ptrfdm, char *slave_name, struct termios *slave_termios, struct win
 	return(pid);	/* Parent returns pid of child */
     }
 }
-
-#endif
+#endif /* HAVE_FORKPTY */
