@@ -39,15 +39,13 @@
 
 #include "bsd_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/appl/bsd/login.c,v 1.23 1995/10/24 14:03:01 bg Exp $");
+RCSID("$Id: login.c,v 1.41 1996/02/10 17:36:15 assar Exp $");
 
-#ifdef	SKEY
+#ifdef	xSKEY
 #include <skey.h>
 #endif
 
-#ifdef SYSV_LOGINDEFS
 #include "sysv_default.h"
-#endif
 #ifdef SYSV_SHADOW
 #include "sysv_shadow.h"
 #endif
@@ -85,9 +83,7 @@ char	*krbtkfile_env;
 int	authok;
 #endif
 
-#ifdef SYSV4
 char    *ttyprompt;
-#endif
 
 char *__progname;
 struct	passwd *pwd;
@@ -96,28 +92,6 @@ char	term[64], *envinit[1], *hostname, *username, *tty;
 
 static  char rusername[100], lusername[100];
 
-#ifndef HAVE_SETENV
-int
-setenv(const char *var, const char *val, int rewrite)
-{
-  char *t;
-
-  if (!rewrite && getenv(var) != 0)
-    return 0;
-  
-  if ((t = malloc(strlen(var) + strlen(val) + 2)) == 0)
-    return -1;
-
-  strcpy(t, var);
-  strcat(t, "=");
-  strcat(t, val);
-  if (putenv(t) == 0)
-    return 0;
-  else
-    return -1;
-}
-#endif
-
 #ifndef HAVE_GETDTABLESIZE
 int
 getdtablesize()
@@ -140,6 +114,27 @@ setpriority(int which, int who, int niceval)
 }
 #endif
 
+static int
+change_passwd(struct passwd  *who)
+{
+        int             status;
+        int             pid;
+        int             wpid;
+ 
+        switch (pid = fork()) {
+	case -1:
+	    perror("Cannot execute /bin/passwd");
+	    sleepexit(1);
+	case 0:
+	    execlp("/bin/passwd", "passwd", who->pw_name, (char *) 0);
+	    _exit(1);
+	default:
+	    while ((wpid = wait(&status)) != -1 && wpid != pid)
+		/* void */ ;
+	    return (status);
+	}
+}
+
 int
 main(argc, argv)
 	int argc;
@@ -161,16 +156,13 @@ main(argc, argv)
 #ifdef	SKEY
 	int permit_passwd = 0;
 #endif
-#ifdef SYSV_LOGINDEFS
-	int mask;
-	int maxtrys;
-#endif /* SYSV_LOGINDEFS */
+	int mask = 022;		/* Default umask (set below) */
+	int maxtrys = 5;	/* Default number of allowed failed logins */
 
 	__progname = argv[0];
 
 	openlog("login", LOG_ODELAY, LOG_AUTH);
 
-#ifdef SYSV_LOGINDEFS
         /* Read defaults file and set the login timeout period. */
         sysv_defaults();
         timeout = atoi(default_timeout);
@@ -179,7 +171,6 @@ main(argc, argv)
                 syslog(LOG_WARNING, "bad umask default: %s", default_umask);
         else
                 umask(mask);
-#endif /* SYSV_LOGINDEFS */
 
 	(void)signal(SIGALRM, timedout);
 	(void)alarm(timeout);
@@ -205,10 +196,8 @@ main(argc, argv)
 	uid = getuid();
 	while ((ch = getopt(argc, argv, "d:fh:pr:")) != EOF)
 	        switch (ch) {
-#ifdef SYSV4 /* Allow but always ignore the -d option. */
 		case 'd':
                                 break;
-#endif /* SYSV4 */
 		case 'f':
 			fflag = 1;
 			break;
@@ -227,7 +216,11 @@ main(argc, argv)
 			hostname = optarg;
 			break;
 		case 'p':
-                        /* Ignored by SYSV */
+			if (getuid()) {
+			    fprintf(stderr,
+                                    "login: -p for super-user only.\n");
+			    exit(1);
+                        }
 			pflag = 1;
 			break;
 	        case 'r':
@@ -273,13 +266,11 @@ main(argc, argv)
                 ask = 1;
 	      } else
 #endif /* SYSV_ENV */
-#ifdef SYSV4 /* Solaris gets a '-' as login name. */
         if (*argv && strcmp(*argv, "-") == 0) {
                 argc--;
                 argv++;
                 ask = 1;
 	      } else
-#endif /* SYSV4 */
 	if (*argv) {
 		username = *argv;
 		ask = 0;
@@ -287,14 +278,15 @@ main(argc, argv)
                 argc--;
                 argv++;
 #endif /* SYSV_ENV */
-#ifdef SYSV4 /* Perhaps the prompt was already printed. */
         } else if ((ttyprompt = getenv("TTYPROMPT")) && *ttyprompt) {
                 getloginname(0);
                 ask = 0;
-#endif /* SYSV4 */
 	} else
 		ask = 1;
 
+	/* Default tty settings. */
+	stty_default();
+
 	for (cnt = getdtablesize(); cnt > 2; cnt--)
 		(void)close(cnt);
 
@@ -375,8 +367,11 @@ main(argc, argv)
 		permit_passwd = skeyaccess(username, tty,
 					   hostname ? full_hostname : NULL,
 					   NULL);
-		p = skey_getpass("Password:", pwd, permit_passwd);
-		ep = skey_crypt(p, salt, pwd, permit_passwd);
+		p = (char *) skey_getpass("Password:", pwd, permit_passwd);
+		strncpy(ep,
+			(char *)skey_crypt(p, salt, pwd, permit_passwd),
+			sizeof(ep));
+		ep[sizeof(ep) - 1] = 0;
 #else
 		p = getpass("Password:");
 		strncpy(ep, crypt(p, salt), sizeof(ep));
@@ -397,6 +392,12 @@ main(argc, argv)
 			} else
 #endif
 			rval = klogin(pwd, instance, localhost, p);
+			/* getpwnam() is not reentrant and klogin used it! */
+			pwd = getpwnam(username);
+			if (pwd == NULL){
+			      fprintf(stderr, "unknown login %s\n", username);
+			      exit(1);
+			}
 			if (rval != 0 && rootlogin && pwd->pw_uid != 0)
 				rootlogin = 0;
 			if (rval == 0)
@@ -439,23 +440,16 @@ main(argc, argv)
 
 		(void)printf("Login incorrect\n");
 		failures++;
-#ifdef SYSV_LOGINDEFS
+
                 /* max number of attemps and delays taken from defaults file */
-                if (++cnt >= maxtrys) {
-                        badlogin(username);
-                        sleepexit(1);
-                }
-                sleep(atoi(default_sleep));
-#else /* SYSV_LOGINDEFS */
-		/* we allow 10 tries, but after 3 we start backing off */
-		if (++cnt > 3) {
-			if (cnt >= 10) {
+		/* we allow maxtrys tries, but after 2 we start backing off */
+		if (++cnt > 2) {
+			if (cnt >= maxtrys) {
 				badlogin(username);
 				sleepexit(1);
 			}
-			sleep((u_int)((cnt - 3) * 5));
+			sleep((u_int)((cnt - 2) * atoi(default_sleep)));
 		}
-#endif /* SYSV_LOGINDEFS */
 	}
 
 	/* committed to login -- turn off timeout */
@@ -531,7 +525,8 @@ main(argc, argv)
 # ifdef HAVE_UT_ID
 	strncpy(utmp.ut_id, ttyn + sizeof("/dev/tty") - 1, sizeof(utmp.ut_id));
 # endif
-        if (utmpx_login(tty, username, hostname ? hostname : "") != 0) {
+        if (utmpx_login(tty, username, hostname ? hostname : "") != 0
+	    && !fflag) {
                 printf("No utmpx entry.  You must exec \"login\" from the lowest level \"sh\".\n");
                 sleepexit(0);
         }
@@ -539,11 +534,6 @@ main(argc, argv)
 
 	dolastlog(quietlog);
 
-#ifdef SYSV_LOGINDEFS
-        if (default_ulimit && ulimit(2, atol(default_ulimit)) < 0)
-                printf("Could not set ULIMIT to %ld\n", default_ulimit);
-#endif /* SYSV_LOGINDEFS */
-
 	/*
 	 * Set device protections, depending on what terminal the
 	 * user is logged in. This feature is used on Suns to give
@@ -568,8 +558,10 @@ main(argc, argv)
          * that LD_* and IFS are never preserved.
          */
 #ifdef SYSV_ENV
+	if (term[0] == '\0')
+		(void)strncpy(term, stypeof(tty), sizeof(term));
         /* set up a somewhat censored environment. */
-        sysv_newenv(argc, argv, pwd, term);
+        sysv_newenv(argc, argv, pwd, term, pflag);
 #else /* SYSV_ENV */
 	/* Destroy environment unless user has requested its preservation. */
 	if (!pflag)
@@ -622,10 +614,12 @@ main(argc, argv)
          * motd and mail stuff up to the shell startup file.
          */
 	if (!quietlog) {
+#if 0
 		(void)printf("%s\n\t%s  %s\n\n",
 	    "Copyright (c) 1980, 1983, 1986, 1988, 1990, 1991, 1993, 1994",
 		    "The Regents of the University of California. ",
 		    "All rights reserved.");
+#endif
 		motd();
 		(void)sprintf(tbuf,
 		    "%s/%s", _PATH_MAILDIR, pwd->pw_name);
@@ -662,6 +656,11 @@ main(argc, argv)
                 syslog(LOG_ERR, "setlogin() failure: %m");
 #endif
 
+#ifdef HAVE_SETPCRED
+	if (setpcred (pwd->pw_name, NULL) == -1)
+		syslog(LOG_ERR, "setpcred() failure: %m");
+#endif /* HAVE_SETPCRED */
+
 	/* Discard permissions last so can't get killed and drop core. */
 	if (rootlogin)
 		(void) setuid(0);
@@ -673,14 +672,13 @@ main(argc, argv)
          * optionally run, as the user, /bin/passwd.
          */
  
-#ifdef SYSV_LOGINDEFS
         if (pwd->pw_passwd[0] == 0 && strcasecmp(default_passreq, "YES") == 0) {
                 printf("You don't have a password.  Choose one.\n");
                 if (change_passwd(pwd))
                         sleepexit(0);
 		changepass = 0;
         }
-#endif /* SYSV_LOGINDEFS */
+
 #ifdef SYSV_SHADOW
         if (sysv_expire(spwd)) {
                 if (change_passwd(pwd))
@@ -725,11 +723,9 @@ getloginname(prompt)
 
 	for (;;) {
                 if (prompt)
-#ifdef SYSV4
                     if (ttyprompt && *ttyprompt)
                         (void)printf("%s", ttyprompt);
                     else
-#endif /* SYSV4 */
 		        (void)printf("login: ");
 		prompt = 1;
 		for (p = nbuf; (ch = getchar()) != '\n'; ) {
@@ -757,11 +753,7 @@ rootterm(ttyn)
 	char *ttyn;
 {
 #ifndef HAVE_TTYENT_H
-# ifdef SYSV_LOGINDEFS
         return (default_console == 0 || strcmp(default_console, ttyname(0)) == 0);
-# else
-	return 1;
-# endif
 #else
 	struct ttyent *t;
 
@@ -839,8 +831,7 @@ dolastlog(quiet)
                     ll.ll_time != 0) {
                         if (pwd->pw_uid && spwd->sp_inact > 0
                             && ll.ll_time / DAY + spwd->sp_inact < DAY_NOW) {
-                                printf("Your account has been inactive too long.
-\n");
+                                printf("Your account has been inactive too long.\n");
                                 sleepexit(1);
                         }
                         if (!quiet) {
@@ -911,15 +902,22 @@ badlogin(name)
 
 char *
 stypeof(ttyid)
-	char *ttyid;
+        char *ttyid;
 {
+    /* TERM is probably a better guess than anything else. */
+    char *term = getenv("TERM");
+
+    if (term != 0 && term[0] != 0)
+	return term;
+
+    {
 #ifndef HAVE_TTYENT_H
 	return UNKNOWN;
 #else
 	struct ttyent *t;
-
 	return (ttyid && (t = getttynam(ttyid)) ? t->ty_type : UNKNOWN);
 #endif
+    }
 }
 
 doremotelogin(host)
@@ -966,6 +964,12 @@ login(ut)
 {
         int fd;
 
+#ifdef HAVE_SETUTENT
+	utmpname(_PATH_UTMP);
+	setutent();
+	pututline(ut);
+	endutent();
+#else
 #ifdef HAVE_TTYSLOT
         int tty;
         tty = ttyslot();
@@ -974,40 +978,11 @@ login(ut)
                 (void)write(fd, (char *)ut, sizeof(struct utmp));
                 (void)close(fd);
         }
-#else  /* !HAVE_TTYSLOT */
-	utmpname(_PATH_UTMP);
-	setutent();
-	pututline(ut);
-	endutent();
 #endif /* HAVE_TTYSLOT */
+#endif /* HAVE_SETUTENT */
 
         if ((fd = open(_PATH_WTMP, O_WRONLY|O_APPEND, 0)) >= 0) {
                 (void)write(fd, (char *)ut, sizeof(struct utmp));
                 (void)close(fd);
         }
 }
-
-#if defined(SYSV_SHADOW) || defined(SYSV_LOGINDEFS)
- 
-change_passwd(who)
-        struct passwd  *who;
-{
-        int             status;
-        int             pid;
-        int             wpid;
- 
-        switch (pid = fork()) {
-	  case -1:
-                perror("Cannot execute /bin/passwd");
-                sleepexit(1);
-	      case 0:
-                execlp("/bin/passwd", "passwd", who->pw_name, (char *) 0);
-                _exit(1);
-	      default:
-                while ((wpid = wait(&status)) != -1 && wpid != pid)
-                         /* void */ ;
-                return (status);
-	    }
-    }
- 
-#endif /* SYSV_SHADOW || SYSV_LOGINDEFS */
