@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  *
  *	from: @(#)pathnames.h	5.2 (Berkeley) 4/9/90
- *	$Id: pathnames.h,v 1.13 1995/09/28 11:44:20 bg Exp $
+ *	$Id: pathnames.h,v 1.15 1996/01/31 20:33:55 bg Exp $
  */
 
 /******* First fix default path, we stick to _PATH_DEFPATH everywhere */
@@ -53,6 +53,10 @@
 #define _DEF_PATH _PATH_DEFPATH
 #endif /* !_PATH_DEFPATH */
 
+#ifndef _PATH_DEFSUPATH
+#define _PATH_DEFSUPATH __CONCAT("/usr/sbin:", _DEF_PATH)
+#endif /* _PATH_DEFSUPATH */
+
 /******* Default PATH fixed! */
 
 #undef  _PATH_RLOGIN		/* Redifine rlogin */
@@ -141,3 +145,22 @@
 #ifndef _PATH_WTMP
 #define _PATH_WTMP   "/usr/adm/wtmp"
 #endif
+
+#ifndef _PATH_ETC_DEFAULT_LOGIN
+#define _PATH_ETC_DEFAULT_LOGIN	"/etc/default/login"
+#endif
+
+/*
+ * NeXT KLUDGE ALERT!!!!!!!!!!!!!!!!!!
+ * Some sort of bug in the NEXTSTEP cpp.
+ */
+#ifdef NeXT
+#undef  _PATH_DEFSUPATH
+#define _PATH_DEFSUPATH "/usr/sbin:/usr/ucb:/usr/bin:/bin"
+#undef  _PATH_RLOGIN
+#define	_PATH_RLOGIN	"/usr/athena/bin/rlogin"
+#undef  _PATH_RSH
+#define _PATH_RSH	"/usr/athena/bin/rsh"
+#undef  _PATH_LOGIN
+#define _PATH_LOGIN	"/usr/athena/bin/login"
+#endif
