@@ -35,7 +35,7 @@
 
 #include "bsd_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/appl/bsd/rcp.c,v 1.10 1995/10/26 11:48:58 bg Exp $");
+RCSID("$Id: rcp.c,v 1.15 1996/02/04 15:05:51 assar Exp $");
 
 /* Globals */
 char *__progname;
@@ -58,19 +58,108 @@ static char cmd[CMDNEEDS];		/* must hold "rcp -r -p -d\0" */
 
 void rsource(char *name, struct stat *statp);
 
+#define	SERVICE_NAME	"rcmd"
+
+CREDENTIALS cred;
+MSG_DAT msg_data;
+struct sockaddr_in foreign, local;
+Key_schedule schedule;
+
+KTEXT_ST ticket;
+AUTH_DAT kdata;
+
+static void
+send_auth(char *h, char *r)
+{
+    int lslen, fslen, status;
+    long opts;
+
+    lslen = sizeof(struct sockaddr_in);
+    if (getsockname(rem, (struct sockaddr *)&local, &lslen) < 0)
+	{
+	    perror("getsockname");
+	    exit(1);
+	}
+    fslen = sizeof(struct sockaddr_in);
+    if (getpeername(rem, (struct sockaddr *)&foreign, &fslen) < 0)
+	{
+	    perror("getpeername");
+	    exit(1);
+	}
+    if ((r == NULL) || (*r == '\0'))
+	r = krb_realmofhost(h);
+    opts = KOPT_DO_MUTUAL;
+    if ((status = krb_sendauth(opts, rem, &ticket, SERVICE_NAME, h, r, 
+			    (unsigned long)getpid(), &msg_data, &cred, schedule, &local, 
+			    &foreign, "KCMDV0.1")) != KSUCCESS)
+	{
+	    fprintf(stderr, "krb_sendauth failure: %s\n", 
+		    krb_err_txt[status]);
+	    exit(1);
+	}
+}
+
+static void
+answer_auth()
+{
+    int lslen, fslen, status;
+    long opts;
+    char inst[INST_SZ], v[9];
+
+    lslen = sizeof(struct sockaddr_in);
+    if (getsockname(rem, (struct sockaddr *)&local, &lslen) < 0)
+	{
+	    perror("getsockname");
+	    exit(1);
+	}
+    fslen = sizeof(struct sockaddr_in);
+    if(getpeername(rem, (struct sockaddr *)&foreign, &fslen) < 0)
+	{
+	    perror("getperrname");
+	    exit(1);
+	}
+    strcpy(inst, "*");
+    opts = KOPT_DO_MUTUAL;
+    if ((status = krb_recvauth(opts, rem, &ticket, SERVICE_NAME, inst, &foreign, &local, 
+			       &kdata, "", schedule, v)) != KSUCCESS)
+	{
+	    fprintf(stderr, "krb_recvauth failure: %s\n", 
+		    krb_err_txt[status]);
+	    exit(1);
+	}
+}
+
+static int
+des_read(int fd, char *buf, int len)
+{
+    if (doencrypt)
+	return(des_enc_read(fd, buf, len, schedule, 
+			    (iamremote? &kdata.session : &cred.session)));
+    else
+	return(read(fd, buf, len));
+}
+
+static int
+des_write(int fd, char *buf, int len)
+{
+    if (doencrypt)
+	return(des_enc_write(fd, buf, len, schedule, 
+			     (iamremote? &kdata.session : &cred.session)));
+    else
+	return(write(fd, buf, len));
+}
+
 static void
 run_err(const char *fmt, const char *arg1, const char *arg2)
 {
-	static FILE *fp;
+        char errbuf[1024];
 
 	++errs;
-	if (fp == NULL && !(fp = fdopen(rem, "w")))
-		return;
-	(void)fprintf(fp, "%c", 0x01);
-	(void)fprintf(fp, "rcp: ");
-	(void)fprintf(fp, fmt, arg1, arg2);
-	(void)fprintf(fp, "\n");
-	(void)fflush(fp);
+#define RCPERR "\001rcp: "
+	strcpy(errbuf, RCPERR);
+	sprintf(errbuf+strlen(RCPERR), fmt, arg1, arg2);
+	strcat(errbuf, "\n");
+	des_write(rem, errbuf, strlen(errbuf));
 
 	if (!iamremote)
 		warnx(fmt, arg1, arg2);
@@ -155,7 +244,7 @@ response(void)
 {
 	char ch, *cp, resp, rbuf[BUFSIZ];
 
-	if (read(rem, &resp, sizeof(resp)) != sizeof(resp))
+	if (des_read(rem, &resp, sizeof(resp)) != sizeof(resp))
 		lostconn(0);
 
 	cp = rbuf;
@@ -168,7 +257,7 @@ response(void)
 	case 1:				/* error, followed by error msg */
 	case 2:				/* fatal error, "" */
 		do {
-			if (read(rem, &ch, sizeof(ch)) != sizeof(ch))
+			if (des_read(rem, &ch, sizeof(ch)) != sizeof(ch))
 				lostconn(0);
 			*cp++ = ch;
 		} while (cp < &rbuf[BUFSIZ] && ch != '\n');
@@ -225,13 +314,13 @@ syserr:			run_err("%s: %s", name, k_strerror(errno));
 			 */
 			(void)sprintf(buf, "T%ld 0 %ld 0\n",
 			    stb.st_mtime, stb.st_atime);
-			(void)write(rem, buf, strlen(buf));
+			(void)des_write(rem, buf, strlen(buf));
 			if (response() < 0)
 				goto next;
 		}
 		(void)sprintf(buf, "C%04o %ld %s\n",
 		    (int)stb.st_mode & MODEMASK, (long) stb.st_size, last);
-		(void)write(rem, buf, strlen(buf));
+		(void)des_write(rem, buf, strlen(buf));
 		if (response() < 0)
 			goto next;
 		if ((bp = allocbuf(&buffer, fd, BUFSIZ)) == NULL) {
@@ -250,9 +339,9 @@ next:			(void)close(fd);
 					haderr = result >= 0 ? EIO : errno;
 			}
 			if (haderr)
-				(void)write(rem, bp->buf, amt);
+				(void)des_write(rem, bp->buf, amt);
 			else {
-				result = write(rem, bp->buf, amt);
+				result = des_write(rem, bp->buf, amt);
 				if (result != amt)
 					haderr = result >= 0 ? EIO : errno;
 			}
@@ -260,7 +349,7 @@ next:			(void)close(fd);
 		if (close(fd) && !haderr)
 			haderr = errno;
 		if (!haderr)
-			(void)write(rem, "", 1);
+			(void)des_write(rem, "", 1);
 		else
 			run_err("%s: %s", name, k_strerror(haderr));
 		(void)response();
@@ -286,7 +375,7 @@ rsource(char *name, struct stat *statp)
 	if (pflag) {
 		(void)sprintf(path, "T%ld 0 %ld 0\n",
 		    statp->st_mtime, statp->st_atime);
-		(void)write(rem, path, strlen(path));
+		(void)des_write(rem, path, strlen(path));
 		if (response() < 0) {
 			closedir(dirp);
 			return;
@@ -294,7 +383,7 @@ rsource(char *name, struct stat *statp)
 	}
 	(void)sprintf(path,
 	    "D%04o %d %s\n", (int)statp->st_mode & MODEMASK, 0, last);
-	(void)write(rem, path, strlen(path));
+	(void)des_write(rem, path, strlen(path));
 	if (response() < 0) {
 		closedir(dirp);
 		return;
@@ -313,7 +402,7 @@ rsource(char *name, struct stat *statp)
 		source(1, vect);
 	}
 	(void)closedir(dirp);
-	(void)write(rem, "E\n", 2);
+	(void)des_write(rem, "E\n", 2);
 	(void)response();
 }
 
@@ -321,17 +410,17 @@ static int get_shell_port(int kerberos, int encryption)
 {
   struct servent *sp;
   char *service="shell";
-  int port=514;
+  int port=htons(514);
 
   if(kerberos){
     service="kshell";
-    port=544;
+    port=htons(544);
   }
   
   sp=getservbyname(service, "tcp");
   if(sp == NULL){
     fprintf(stderr, "rcp: can't get entry for %s service, using default port %d.\n",
-	    service, port);
+	    service, ntohs(port));
   }else{
     port=sp->s_port;
   }
@@ -341,6 +430,7 @@ static int get_shell_port(int kerberos, int encryption)
 static int
 kerberos(char **host, char *bp, char *locuser, char *user)
 {
+        int sock = -1, err;
 again:
 	if (use_kerberos) {
 		rem = KSUCCESS;
@@ -348,7 +438,34 @@ again:
 		if (dest_realm == NULL)
 			dest_realm = krb_realmofhost(*host);
 
+#if 0
 		rem = krcmd(host, port, user, bp, 0, dest_realm);
+#else
+		err = kcmd(
+		    &sock,
+		    host,
+		    port,
+		    NULL,	/* locuser not used */
+		    user,
+		    bp,
+		    0,
+		    &ticket,
+		    SERVICE_NAME,
+		    dest_realm,
+		    (CREDENTIALS *) NULL, /* credentials not used */
+		    (bit_64 *) NULL, /* key schedule not used */
+		    (MSG_DAT *) NULL, /* MSG_DAT not used */
+		    (struct sockaddr_in *) NULL, /* local addr not used */
+		    (struct sockaddr_in *) NULL, /* foreign addr not used */
+		    0L);	/* authopts */
+		if (err > KSUCCESS && err < MAX_KRB_ERRORS) {
+			fprintf(stderr, "kcmd: %s\n", krb_err_txt[err]);
+			rem = -1;
+		} else if (err < 0)
+		    rem = -1;
+		else
+		    rem = sock;
+#endif
 		if (rem < 0) {
 			use_kerberos = 0;
 			port=get_shell_port(use_kerberos, doencrypt);
@@ -370,7 +487,10 @@ again:
 static void
 toremote(char *targ, int argc, char **argv)
 {
-	int i, len, tos;
+	int i, len;
+#ifdef IP_TOS
+	int tos;
+#endif
 	char *bp, *host, *src, *suser, *thost, *tuser;
 
 	*targ++ = 0;
@@ -445,6 +565,8 @@ toremote(char *targ, int argc, char **argv)
 				    &tos, sizeof(int)) < 0)
 					warn("TOS (ignored)");
 #endif /* IP_TOS */
+				if (doencrypt)
+				    send_auth(host, dest_realm);
 				if (response() < 0)
 					exit(1);
 				(void)free(bp);
@@ -465,7 +587,7 @@ sink(int argc, char **argv)
 	BUF *bp;
 	off_t i, j;
 	int amt, count, exists, first, mask, mode, ofd, omode;
-	int setimes, size, targisdir, wrerrno;
+	int setimes, size, targisdir, wrerrno=0;
 	char ch, *cp, *np, *targ, *why, *vect[1], buf[BUFSIZ];
 
 #define	atime	tv[0]
@@ -483,17 +605,17 @@ sink(int argc, char **argv)
 	targ = *argv;
 	if (targetshouldbedirectory)
 		verifydir(targ);
-	(void)write(rem, "", 1);
+	(void)des_write(rem, "", 1);
 	if (stat(targ, &stb) == 0 && S_ISDIR(stb.st_mode))
 		targisdir = 1;
 	for (first = 1;; first = 0) {
 		cp = buf;
-		if (read(rem, cp, 1) <= 0)
+		if (des_read(rem, cp, 1) <= 0)
 			return;
 		if (*cp++ == '\n')
 			SCREWUP("unexpected <newline>");
 		do {
-			if (read(rem, &ch, sizeof(ch)) != sizeof(ch))
+			if (des_read(rem, &ch, sizeof(ch)) != sizeof(ch))
 				SCREWUP("lost connection");
 			*cp++ = ch;
 		} while (cp < &buf[BUFSIZ - 1] && ch != '\n');
@@ -509,7 +631,7 @@ sink(int argc, char **argv)
 			continue;
 		}
 		if (buf[0] == 'E') {
-			(void)write(rem, "", 1);
+			(void)des_write(rem, "", 1);
 			return;
 		}
 
@@ -533,7 +655,7 @@ sink(int argc, char **argv)
 			getnum(atime.tv_usec);
 			if (*cp++ != '\0')
 				SCREWUP("atime.usec not delimited");
-			(void)write(rem, "", 1);
+			(void)des_write(rem, "", 1);
 			continue;
 		}
 		if (*cp != 'C' && *cp != 'D') {
@@ -615,7 +737,7 @@ sink(int argc, char **argv)
 bad:			run_err("%s: %s", np, k_strerror(errno));
 			continue;
 		}
-		(void)write(rem, "", 1);
+		(void)des_write(rem, "", 1);
 		if ((bp = allocbuf(&buffer, ofd, BUFSIZ)) == NULL) {
 			(void)close(ofd);
 			continue;
@@ -628,7 +750,7 @@ bad:			run_err("%s: %s", np, k_strerror(errno));
 				amt = size - i;
 			count += amt;
 			do {
-				j = read(rem, cp, amt);
+				j = des_read(rem, cp, amt);
 				if (j <= 0) {
 					run_err("%s", j ? k_strerror(errno) :
 					    "dropped connection", "");
@@ -688,7 +810,7 @@ bad:			run_err("%s: %s", np, k_strerror(errno));
 			run_err("%s: %s", np, k_strerror(wrerrno));
 			break;
 		case NO:
-			(void)write(rem, "", 1);
+			(void)des_write(rem, "", 1);
 			break;
 		case DISPLAYED:
 			break;
@@ -702,7 +824,10 @@ screwup:
 static void
 tolocal(int argc, char **argv)
 {
-	int i, len, tos;
+	int i, len;
+#ifdef IP_TOS
+	int tos;
+#endif
 	char *bp, *host, *src, *suser;
 
 	for (i = 0; i < argc - 1; i++) {
@@ -752,6 +877,8 @@ tolocal(int argc, char **argv)
 		if (setsockopt(rem, IPPROTO_IP, IP_TOS, &tos, sizeof(int)) < 0)
 			warn("TOS (ignored)");
 #endif /* IP_TOS */
+		if (doencrypt)
+		    send_auth(host, dest_realm);
 		sink(1, argv + argc - 1);
 		(void)seteuid(0);
 		(void)close(rem);
@@ -778,10 +905,8 @@ main(int argc, char **argv)
 			(void)strncpy(dst_realm_buf, optarg, REALM_SZ);
 			break;
 		case 'x':
-			fprintf(stderr, "rcp -x option not yet supported.\n");
-			fprintf(stderr, "Use rsh -x and tar instead.\n");
-			exit(1);
 			doencrypt = 1;
+			{ extern int LEFT_JUSTIFIED; LEFT_JUSTIFIED = 1; };
 			break;
 		case 'p':
 			pflag = 1;
@@ -816,6 +941,8 @@ main(int argc, char **argv)
 	rem = STDIN_FILENO;		/* XXX */
 
 	if (fflag) {			/* Follow "protocol", send data. */
+		if (doencrypt)
+		    answer_auth();
 		(void)response();
 		(void)setuid(userid);
 		source(argc, argv);
@@ -823,6 +950,8 @@ main(int argc, char **argv)
 	}
 
 	if (tflag) {			/* Receive data. */
+	    if (doencrypt)
+		    answer_auth();
 		(void)setuid(userid);
 		sink(argc, argv);
 		exit(errs);
