@@ -38,10 +38,14 @@
  */
 #include "bsd_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/appl/bsd/rlogin.c,v 1.19 1995/10/26 11:48:59 bg Exp $");
+RCSID("$Id: rlogin.c,v 1.26 1996/02/07 10:47:57 d91-jda Exp $");
 
 #include <termios.h>
 
+#if defined(NeXT)               /* NEXTSTEP 3.3 */
+#include <sys/ioctl.h>
+#endif
+
 #if defined(__osf__)		/* OSF/1 */
 #include <sys/ioctl.h>
 #endif
@@ -55,29 +59,18 @@ RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/appl
 #include <sys/ioctl.h>
 #endif
 
-#if defined(sun) && !defined(__svr4__) /* SunOS 4 */
+#if SunOS == 4
 #ifndef _POSIX_VDISABLE
 #define _POSIX_VDISABLE 0
 #endif /* _POSIX_VDISABLE */
 #include <sys/ttold.h>
 #endif
 
-#if defined(sun) && defined(__svr4__) /* SunOS 5 */
+#if SunOS == 5
 #include <sys/tty.h>
 #include <sys/ptyvar.h>
 #endif
 
-#if defined(__linux)		/* Linux version ?? */
-/* KENT-950824
-* rlogin has problems with urgent data when logging into suns which
-* results in the connection being closed with an IO error. LINUX_KLUDGE
-* is a work around - the actual bug is probably in tcp.c in the kernel, but
-* I haven't managed to find it yet.
-* Andrew.Tridgell@anu.edu.au (12th March 1993)
-*/
-#define LINUX_KLUDGE
-#endif
-
 CREDENTIALS cred;
 Key_schedule schedule;
 int use_kerberos = 1, doencrypt;
@@ -100,10 +93,9 @@ char dst_realm_buf[REALM_SZ], *dest_realm = NULL;
 
 int eight, rem;
 struct termios deftty;
-int dosigwinch;
 
 int noescape;
-u_char escapechar = '~';
+char escapechar = '~';
 
 #ifdef OLDSUN
 struct winsize {
@@ -144,7 +136,7 @@ struct	winsize winsize;
 #define	WRITING	2
 
 jmp_buf rcvtop;
-int ppid, rcvcnt, rcvstate;
+int parent, rcvcnt, rcvstate;
 char rcvbuf[8 * 1024];
 
 static void
@@ -283,33 +275,12 @@ oob_real(int signo)
 
 	out = O_RDWR;
 	rcvd = 0;
-	while (recv(rem, &mark, 1, MSG_OOB) < 0) {
-		switch (errno) {
-		case EWOULDBLOCK:
-			/*
-			 * Urgent data not here yet.  It may not be possible
-			 * to send it yet if we are blocked for output and
-			 * our input buffer is full.
-			 */
-			if (rcvcnt < sizeof(rcvbuf)) {
-				n = read(rem, rcvbuf + rcvcnt,
-				    sizeof(rcvbuf) - rcvcnt);
-				if (n <= 0)
-					return;
-				rcvd += n;
-			} else {
-				n = read(rem, waste, sizeof(waste));
-				if (n <= 0)
-					return;
-			}
-			continue;
-		default:
-			return;
-		}
+	if (recv(rem, &mark, 1, MSG_OOB) < 0) {
+		return;
 	}
 	if (mark & TIOCPKT_WINDOW) {
 		/* Let server know about window size changes */
-		(void)kill(ppid, SIGUSR1);
+		(void)kill(parent, SIGUSR1);
 	}
 	if (!eight && (mark & TIOCPKT_NOSTOP)) {
 		(void)tcgetattr(0, &tty);
@@ -360,41 +331,20 @@ oob_real(int signo)
 		longjmp(rcvtop, 1);
 }
 
-static void
-oob(int signo)
-{
-	oob_real(signo);
-#ifdef LINUX_KLUDGE
-	signal(SIGURG, oob);
-#endif
-}
-
 /* reader: read from remote: line -> 1 */
 static int
 reader(sigset_t omask)
 {
-	int pid, n, remaining;
+	int n, remaining;
 	char *bufp;
 
-	pid = getpid();		/* modern systems use positives for pid */
 	(void)signal(SIGTTOU, SIG_IGN);
-	(void)signal(SIGURG, oob);
-	ppid = getppid();
-#ifdef F_SETOWN
-	(void)fcntl(rem, F_SETOWN, pid);
-#else
-	{
-		/* hpux invention */
-		int fioflag = 1;
-		ioctl(rem, FIOSSAIOSTAT, &fioflag);
-		ioctl(rem, FIOSSAIOOWN, &pid);
-	}
-#endif
 	(void)setjmp(rcvtop);
 	if (sigprocmask(SIG_SETMASK, &omask, 0) != 0)
 	    perror("sigprocmask");
 	bufp = rcvbuf;
 	for (;;) {
+	        fd_set readfds, exceptfds;
 		while ((remaining = rcvcnt - (bufp - rcvbuf)) > 0) {
 			rcvstate = WRITING;
 			n = write(STDOUT_FILENO, bufp, remaining);
@@ -409,6 +359,31 @@ reader(sigset_t omask)
 		rcvcnt = 0;
 		rcvstate = READING;
 
+		FD_ZERO (&readfds);
+		FD_SET (rem, &readfds);
+		FD_ZERO (&exceptfds);
+		FD_SET (rem, &exceptfds);
+		if (select(rem+1, &readfds, 0, &exceptfds, 0) == -1)
+		    {
+			if (errno == EINTR)
+			    continue; /* Got signal */
+			else
+			    {
+				msg("select failed mysteriously\n");
+				exit(1);
+			    }
+		    }
+
+		if (FD_ISSET(rem, &readfds))
+		    /* do it below */;
+		else if (FD_ISSET(rem, &exceptfds))
+		    {
+			oob_real(0);
+			continue;
+		    }
+		else
+		    fprintf(stderr, "nothing to read?\n");
+
 		if (doencrypt)
 #ifdef NOENCRYPTION
 			rcvcnt = read(rem, rcvbuf, sizeof (rcvbuf));
@@ -463,15 +438,12 @@ sigwinch(int signo)
 {
 	struct winsize ws;
 
-	if (dosigwinch && get_window_size(0, &ws) == 0 &&
+	if (get_window_size(0, &ws) == 0 &&
 	    bcmp(&ws, &winsize, sizeof(ws))) {
 		winsize = ws;
 		sendwindow();
 	}
-
-#ifdef LINUX_KLUDGE
        signal(SIGWINCH, sigwinch);
-#endif
 }
 
 static void
@@ -482,7 +454,7 @@ stop(int all)
 	(void)kill(all ? 0 : getpid(), SIGTSTP);
 	(void)signal(SIGCHLD, catch_child);
 	mode(1);
-	sigwinch(0);			/* check for size changes */
+	kill(SIGWINCH, getpid()); /* check for size changes, if caught */
 }
 
 /*
@@ -572,10 +544,6 @@ writer(void)
 		    CCEQ(deftty.c_cc[VSUSP], c) ||
 		    c == '\r' || c == '\n';
 	}
-
-#ifdef LINUX_KLUDGE
-       signal(SIGWINCH, sigwinch);
-#endif
 }
 
 static void
@@ -586,6 +554,7 @@ doit(sigset_t omask)
 	setsignal(SIGHUP);
 	setsignal(SIGQUIT);
 	mode(1);
+	parent = getpid();
 	child = fork();
 	if (child == -1) {
 		(void)fprintf(stderr, "rlogin: fork: %s.\n", k_strerror(errno));
@@ -602,10 +571,10 @@ doit(sigset_t omask)
 	}
 
 	/*
-	 * We may still own the socket, and may have a pending SIGURG (or might
+	 * We may still own the socket, and may have a pending OOB (or might
 	 * receive one soon) that we really want to send to the reader.  When
 	 * one of these comes in, the trap copytochild simply copies such
-	 * signals to the child. We can now unblock SIGURG and SIGUSR1
+	 * signals to the child. We can now unblock OOB and SIGUSR1
 	 * that were set above.
 	 */
 	if (sigprocmask(SIG_SETMASK, &omask, 0) != 0)
@@ -617,17 +586,18 @@ doit(sigset_t omask)
 }
 
 /*
- * This is called when the reader process gets the out-of-band (urgent)
- * request to turn on the window-changing protocol.
+ * This is called when the reader process gets the out-of-band
+ * (urgent) request to turn on the window-changing protocol. It is
+ * signalled from the child(reader).
  */
 static void
-writeroob(int signo)
+sigusr1(int signo)
 {
-	if (dosigwinch == 0) {
-		sendwindow();
-		(void)signal(SIGWINCH, sigwinch);
-	}
-	dosigwinch = 1;
+        /*
+	 * Now we now daemon supports winsize hack,
+	 */
+	sendwindow();
+	(void)signal(SIGWINCH, sigwinch); /* so we start to support it */
 }
 
 static void
@@ -638,16 +608,6 @@ lostpeer(int signo)
 	done(1);
 }
 
-/* copy SIGURGs to the child process. */
-static void
-copytochild(int signo)
-{
-	(void)kill(child, SIGURG);
-#ifdef LINUX_KLUDGE
-	signal(SIGCHLD, copytochild);
-#endif
-}
-
 static void
 usage(void)
 {
@@ -677,29 +637,29 @@ getescape(register char *p)
 	}
 	msg("illegal option value -- e");
 	usage();
-	/* NOTREACHED */
+	return 0;
 }
 
 static int get_login_port(int kerberos, int encryption)
 {
   struct servent *sp;
   char *service="login";
-  int port=513;
+  int port=htons(513);
 
   if(kerberos && encryption){
     service="eklogin";
-    port=2105;
+    port=htons(2105);
   }
   
   if(kerberos && !encryption){
     service="klogin";
-    port=543;
+    port=htons(543);
   }
   
   sp=getservbyname(service, "tcp");
   if(sp == NULL){
     fprintf(stderr, "rlogin: can't get entry for %s service, using default port %d.\n",
-	    service, port);
+	    service, ntohs(port));
   }else{
     port=sp->s_port;
   }
@@ -799,24 +759,11 @@ main(int argc, char **argv)
 	(void)get_window_size(0, &winsize);
 
 	(void)signal(SIGPIPE, lostpeer);
-	/* will use SIGUSR1 for window size hack, so hold it off */
-	{
-	    sigset_t sigmsk;
 
-	    sigemptyset(&sigmsk);
-	    sigaddset(&sigmsk, SIGURG);
-	    sigaddset(&sigmsk, SIGUSR1);
-	    if (sigprocmask(SIG_BLOCK, &sigmsk, &omask) != 0)
-	      perror("sigprocmask");
-	}
 	/*
-	 * We set SIGURG and SIGUSR1 below so that an
-	 * incoming signal will be held pending rather than being
-	 * discarded. Note that these routines will be ready to get
-	 * a signal by the time that they are unblocked below.
+	 * Child sends parent this signal for window size hack.
 	 */
-	(void)signal(SIGURG, copytochild);
-	(void)signal(SIGUSR1, writeroob);
+	(void)signal(SIGUSR1, sigusr1);
 
 try_connect:
 	if (use_kerberos) {
@@ -863,7 +810,8 @@ try_connect:
 		exit(1);
 
 	if (dflag &&
-	    setsockopt(rem, SOL_SOCKET, SO_DEBUG, &one, sizeof(one)) < 0)
+	    setsockopt(rem, SOL_SOCKET, SO_DEBUG, (void *)&one,
+		       sizeof(one)) < 0)
 		(void)fprintf(stderr, "rlogin: setsockopt: %s.\n",
 		    k_strerror(errno));
 #ifdef IP_TOS
@@ -874,5 +822,5 @@ try_connect:
 
 	(void)setuid(uid);
 	doit(omask);
-	/*NOTREACHED*/
+	return 0;
 }
