@@ -42,11 +42,11 @@
 
 #include "bsd_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/appl/bsd/rlogind.c,v 1.22 1995/10/26 15:33:36 bg Exp $");
+RCSID("$Id: rlogind.c,v 1.40 1996/02/11 17:32:54 bg Exp $");
 
 #include <termios.h>
 
-#if defined(__osf__)		/* OSF/1 */
+#if defined(HAVE_SYS_IOCTL_H) && SunOS != 4
 #include <sys/ioctl.h>
 #endif
 
@@ -56,22 +56,22 @@ RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/appl
 #include <sys/ptyio.h>
 #endif
 
-#if defined(_AIX)		/* AIX */
-#include <sys/ioctl.h>
-#endif
-
-#if defined(sun) && !defined(__svr4__) /* SunOS 4 */
+#if SunOS == 4
 #ifndef _POSIX_VDISABLE
 #define _POSIX_VDISABLE 0
 #endif /* _POSIX_VDISABLE */
 #include <sys/ttold.h>
 #endif
 
-#if defined(sun) && defined(__svr4__) /* SunOS 5 */
+#if SunOS == 5
 #include <sys/tty.h>
 #include <sys/ptyvar.h>
 #endif
 
+#if defined(_AIX)
+#include <sys/termio.h>
+#endif
+
 #ifndef CEOF
 #define CEOF 04
 #endif
@@ -80,7 +80,16 @@ RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/appl
 #define TIOCPKT_WINDOW 0x80
 #endif
 
-#define	SECURE_MESSAGE "This rlogin session is using DES encryption for all transmissions.\r\n"
+char *INSECURE_MESSAGE =
+"\r\n*** Connection not encrypted! Communication may be eavesdropped. ***"
+"\r\n*** Use telnet or rlogin -x instead! ***\r\n";
+
+#ifndef NOENCRYPTION
+char *SECURE_MESSAGE =
+"This rlogin session is using DES encryption for all transmissions.\r\n";
+#else
+#define	SECURE_MESSAGE INSECURE_MESSAGE
+#endif
 
 AUTH_DAT	*kdata;
 KTEXT		ticket;
@@ -114,8 +123,102 @@ void	usage __P((void));
 int	local_domain __P((char *));
 char	*topdomain __P((char *));
 
-static int
-logout(const char *line)
+static
+int
+readstream(int p, char *ibuf, int bufsize)
+{
+#ifndef HAVE_GETMSG
+        return read(p, ibuf, bufsize);
+#else
+        static int flowison = -1;  /* current state of flow: -1 is unknown */
+        static struct strbuf strbufc, strbufd;
+	static unsigned char ctlbuf[BUFSIZ];
+	static int use_read = 1;
+
+	int flags = 0;
+	int ret;
+	struct termios tsp;
+
+	struct iocblk ip;
+	char vstop, vstart;
+	int ixon;
+	int newflow;
+
+	if (use_read)
+	  {
+	    ret = read(p, ibuf, bufsize);
+	    if (ret < 0 && errno == EBADMSG)
+	      use_read = 0;
+	    else
+	      return ret;
+	  }
+
+	strbufc.maxlen = BUFSIZ;
+	strbufc.buf = (char *)ctlbuf;
+	strbufd.maxlen = bufsize-1;
+	strbufd.len = 0;
+	strbufd.buf = ibuf+1;
+	ibuf[0] = 0;
+
+	ret = getmsg(p, &strbufc, &strbufd, &flags);
+	if (ret < 0)  /* error of some sort -- probably EAGAIN */
+		return(-1);
+
+	if (strbufc.len <= 0 || ctlbuf[0] == M_DATA) {
+		/* data message */
+		if (strbufd.len > 0) {			/* real data */
+			return(strbufd.len + 1);	/* count header char */
+		} else {
+			/* nothing there */
+			errno = EAGAIN;
+			return(-1);
+		}
+	}
+
+	/*
+	 * It's a control message.  Return 1, to look at the flag we set
+	 */
+
+	switch (ctlbuf[0]) {
+	case M_FLUSH:
+		if (ibuf[1] & FLUSHW)
+			ibuf[0] = TIOCPKT_FLUSHWRITE;
+		return(1);
+
+	case M_IOCTL:
+		memcpy(&ip, (ibuf+1), sizeof(ip));
+
+		switch (ip.ioc_cmd) {
+		case TCSETS:
+		case TCSETSW:
+		case TCSETSF:
+		        memcpy(&tsp,
+			       (ibuf+1 + sizeof(struct iocblk)),
+			       sizeof(tsp));
+			vstop = tsp.c_cc[VSTOP];
+			vstart = tsp.c_cc[VSTART];
+			ixon = tsp.c_iflag & IXON;
+			break;
+		default:
+			errno = EAGAIN;
+			return(-1);
+		}
+
+		newflow =  (ixon && (vstart == 021) && (vstop == 023)) ? 1 : 0;
+		if (newflow != flowison) {  /* it's a change */
+			flowison = newflow;
+			ibuf[0] = newflow ? TIOCPKT_DOSTOP : TIOCPKT_NOSTOP;
+			return(1);
+		}
+	}
+
+	/* nothing worth doing anything about */
+	errno = EAGAIN;
+	return(-1);
+#endif
+}
+
+static int logout(const char *line)
 {
 	register FILE *fp;
 	struct utmp ut;
@@ -150,7 +253,6 @@ logwtmp(const char *line, const char *name, const char *host)
 	struct stat buf;
 	int fd;
 	time_t time();
-	char *strncpy();
 
 	if ((fd = open(_PATH_WTMP, O_WRONLY|O_APPEND, 0)) < 0)
 		return;
@@ -168,10 +270,7 @@ logwtmp(const char *line, const char *name, const char *host)
 	(void)close(fd);
 }
 
-int
-main(argc, argv)
-	int argc;
-	char *argv[];
+int main(int argc, char **argv)
 {
 	extern int __check_rhosts_file;
 	struct sockaddr_in from;
@@ -237,10 +336,7 @@ int	confirmed;
 struct winsize win = { 0, 0, 0, 0 };
 
 
-void
-doit(f, fromp)
-	int f;
-	struct sockaddr_in *fromp;
+void doit(int f, struct sockaddr_in *fromp)
 {
 	int master, pid, on = 1;
 	int authenticated = 0;
@@ -319,13 +415,15 @@ doit(f, fromp)
 	}
 	if (doencrypt)
 #ifdef NOENCRYPTION
-	    (void) write(f, SECURE_MESSAGE, sizeof(SECURE_MESSAGE) - 1);
+	    (void) write(f, INSECURE_MESSAGE, strlen(INSECURE_MESSAGE));
 #else
-	    (void) des_enc_write(f, SECURE_MESSAGE, sizeof(SECURE_MESSAGE) - 1, schedule, &kdata->session);
+	    (void) des_enc_write(f, SECURE_MESSAGE, strlen(SECURE_MESSAGE), schedule, &kdata->session);
 #endif
+	else
+	    (void) write(f, INSECURE_MESSAGE, strlen(INSECURE_MESSAGE));
 	netf = f;
 
-	pid = forkpty(&master, line, NULL, &win);
+	pid = forkpty(&master, line, NULL, NULL);
 	if (pid < 0) {
 		if (errno == ENOENT)
 			fatal(f, "Out of ptys", 0);
@@ -336,7 +434,7 @@ doit(f, fromp)
 		if (f > 2)	/* f should always be 0, but... */
 			(void) close(f);
 		setup_term(0);
-		if (strchr(lusername, '-')) {
+		if (lusername[0] == '-'){
 			syslog(LOG_ERR, "tried to pass user \"%s\" to login",
 			       lusername);
 			fatal(STDERR_FILENO, "invalid user", 0);
@@ -381,11 +479,7 @@ char	oobdata[] = {TIOCPKT_WINDOW};
  * in the data stream.  For now, we are only willing to handle
  * window size changes.
  */
-int
-control(pty, cp, n)
-	int pty;
-	char *cp;
-	int n;
+int control(int master, char *cp, int n)
 {
 	struct winsize w;
 
@@ -397,16 +491,14 @@ control(pty, cp, n)
 	w.ws_col = ntohs(w.ws_col);
 	w.ws_xpixel = ntohs(w.ws_xpixel);
 	w.ws_ypixel = ntohs(w.ws_ypixel);
-	(void)ioctl(pty, TIOCSWINSZ, &w);
+	(void)ioctl(master, TIOCSWINSZ, &w);
 	return (4+sizeof (w));
 }
 
 /*
  * rlogin "protocol" machine.
  */
-void
-protocol(f, p)
-	register int f, p;
+void protocol(int f, int master)
 {
 	char pibuf[1024+1], fibuf[1024], *pbp, *fbp;
 	register pcc = 0, fcc = 0;
@@ -420,10 +512,10 @@ protocol(f, p)
 	 */
 	(void) signal(SIGTTOU, SIG_IGN);
 	send(f, oobdata, 1, MSG_OOB);	/* indicate new rlogin */
-	if (f > p)
+	if (f > master)
 		nfd = f + 1;
 	else
-		nfd = p + 1;
+		nfd = master + 1;
 	if (nfd > FD_SETSIZE) {
 		syslog(LOG_ERR, "select mask too small, increase FD_SETSIZE");
 		fatal(f, "internal error (select mask too small)", 0);
@@ -436,7 +528,7 @@ protocol(f, p)
 		FD_ZERO(&obits);
 		omask = (fd_set *)NULL;
 		if (fcc) {
-			FD_SET(p, &obits);
+			FD_SET(master, &obits);
 			omask = &obits;
 		} else
 			FD_SET(f, &ibits);
@@ -445,8 +537,8 @@ protocol(f, p)
 				FD_SET(f, &obits);
 				omask = &obits;
 			} else
-				FD_SET(p, &ibits);
-		FD_SET(p, &ebits);
+				FD_SET(master, &ibits);
+		FD_SET(master, &ebits);
 		if ((n = select(nfd, &ibits, omask, &ebits, 0)) < 0) {
 			if (errno == EINTR)
 				continue;
@@ -458,14 +550,14 @@ protocol(f, p)
 			continue;
 		}
 #define	pkcontrol(c)	((c)&(TIOCPKT_FLUSHWRITE|TIOCPKT_NOSTOP|TIOCPKT_DOSTOP))
-		if (FD_ISSET(p, &ebits)) {
-			cc = read(p, &cntl, 1);
+		if (FD_ISSET(master, &ebits)) {
+			cc = readstream(master, &cntl, 1);
 			if (cc == 1 && pkcontrol(cntl)) {
 				cntl |= oobdata[0];
 				send(f, &cntl, 1, MSG_OOB);
 				if (cntl & TIOCPKT_FLUSHWRITE) {
 					pcc = 0;
-					FD_CLR(p, &ibits);
+					FD_CLR(master, &ibits);
 				}
 			}
 		}
@@ -493,7 +585,7 @@ protocol(f, p)
 					if (cp[0] == magic[0] &&
 					    cp[1] == magic[1]) {
 						left = fcc - (cp-fibuf);
-						n = control(p, cp, left);
+						n = control(master, cp, left);
 						if (n) {
 							left -= n;
 							if (left > 0)
@@ -502,20 +594,20 @@ protocol(f, p)
 							goto top; /* n^2 */
 						}
 					}
-				FD_SET(p, &obits);		/* try write */
+				FD_SET(master, &obits);		/* try write */
 			}
 		}
 
-		if (FD_ISSET(p, &obits) && fcc > 0) {
-			cc = write(p, fbp, fcc);
+		if (FD_ISSET(master, &obits) && fcc > 0) {
+			cc = write(master, fbp, fcc);
 			if (cc > 0) {
 				fcc -= cc;
 				fbp += cc;
 			}
 		}
 
-		if (FD_ISSET(p, &ibits)) {
-			pcc = read(p, pibuf, sizeof (pibuf));
+		if (FD_ISSET(master, &ibits)) {
+			pcc = readstream(master, pibuf, sizeof (pibuf));
 			pbp = pibuf;
 			if (pcc < 0 && errno == EWOULDBLOCK)
 				pcc = 0;
@@ -548,7 +640,7 @@ protocol(f, p)
 				 * from p, but some old kernels balk at large
 				 * writes even when select returns true.
 				 */
-				if (!FD_ISSET(p, &ibits))
+				if (!FD_ISSET(master, &ibits))
 					sleep(5);
 				continue;
 			}
@@ -560,9 +652,7 @@ protocol(f, p)
 	}
 }
 
-void
-cleanup(signo)
-	int signo;
+void cleanup(int signo)
 {
 	char *p;
 
@@ -575,11 +665,14 @@ cleanup(signo)
 	(void)chmod(line, 0666);
 	(void)chown(line, 0, 0);
 	shutdown(netf, 2);
+	signal(SIGHUP, SIG_IGN);
+#ifdef HAVE_VHANGUP
+	vhangup();
+#endif /* HAVE_VHANGUP */
 	exit(1);
 }
 
-void
-fatal(int f, const char *msg, int syserr)
+void fatal(int f, const char *msg, int syserr)
 {
 	int len;
 	char buf[BUFSIZ], *bp = buf;
@@ -607,9 +700,7 @@ fatal(int f, const char *msg, int syserr)
 	exit(1);
 }
 
-int
-do_rlogin(dest)
-	struct sockaddr_in *dest;
+int do_rlogin(struct sockaddr_in *dest)
 {
 	getstr(rusername, sizeof(rusername), "remuser too long");
 	getstr(lusername, sizeof(lusername), "locuser too long");
@@ -624,11 +715,7 @@ do_rlogin(dest)
 	return (iruserok(dest->sin_addr.s_addr, 0, rusername, lusername));
 }
 
-void
-getstr(buf, cnt, errmsg)
-	char *buf;
-	int cnt;
-	char *errmsg;
+void getstr(char *buf, int cnt, char *errmsg)
 {
 	char c;
 
@@ -643,9 +730,7 @@ getstr(buf, cnt, errmsg)
 
 extern	char **environ;
 
-void
-setup_term(fd)
-	int fd;
+void setup_term(int fd)
 {
 	register char *cp = strchr(term+ENVSIZE, '/');
 	char *speed;
@@ -692,9 +777,7 @@ setup_term(fd)
  * Return -1 on valid authentication, no authorization
  * Return >0 for error conditions
  */
-int
-do_krb_login(dest)
-	struct sockaddr_in *dest;
+int do_krb_login(struct sockaddr_in *dest)
 {
 	int rc;
 	char instance[INST_SZ], version[VERSION_SIZE];
@@ -745,8 +828,7 @@ do_krb_login(dest)
 
 }
 
-void
-usage()
+void usage(void)
 {
 	syslog(LOG_ERR, "usage: rlogind [-aln] [-k | -v]");
 }
@@ -759,9 +841,7 @@ usage()
  * assume that the host is local, as it will be
  * interpreted as such.
  */
-int
-local_domain(h)
-	char *h;
+int local_domain(char *h)
 {
 	char localhost[MAXHOSTNAMELEN];
 	char *p1, *p2;
@@ -775,9 +855,7 @@ local_domain(h)
 	return (0);
 }
 
-char *
-topdomain(h)
-	char *h;
+char *topdomain(char *h)
 {
 	register char *p;
 	char *maybe = NULL;
