@@ -33,9 +33,7 @@
 
 #include "bsd_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/appl/bsd/rsh.c,v 1.8 1995/10/26 11:49:00 bg Exp $");
-
-#include <sys/ioctl.h>
+RCSID("$Id: rsh.c,v 1.11 1996/02/11 16:06:35 assar Exp $");
 
 CREDENTIALS cred;
 Key_schedule schedule;
@@ -214,22 +212,22 @@ static int get_shell_port(int kerberos, int encryption)
 {
   struct servent *sp;
   char *service="shell";
-  int port=514;
+  int port=htons(514);
 
   if(kerberos && encryption){
     service="ekshell";
-    port=2106;
+    port=htons(2106);
   }
   
   if(kerberos && !encryption){
     service="kshell";
-    port=544;
+    port=htons(544);
   }
   
   sp=getservbyname(service, "tcp");
   if(sp == NULL){
     fprintf(stderr, "rsh: can't get entry for %s service, using default port %d.\n",
-	    service, port);
+	    service, ntohs(port));
   }else{
     port=sp->s_port;
   }
