@@ -42,7 +42,7 @@
 
 #include "bsd_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/appl/bsd/rshd.c,v 1.13 1995/10/24 14:03:10 bg Exp $");
+RCSID("$Id: rshd.c,v 1.17 1996/02/11 09:05:03 d91-jda Exp $");
 
 #include <sys/ioctl.h>
 
@@ -52,6 +52,7 @@ extern int __check_rhosts_file;
 static int	keepalive = 1;
 static int	check_all;
 static int	log_success;	/* If TRUE, log all successful accesses */
+static int	new_pag = 1;	/* Put process in new PAG by default */
 static int	sent_null;
 
 void	 doit __P((struct sockaddr_in *));
@@ -63,7 +64,7 @@ void	 usage __P((void));
 
 #define	VERSION_SIZE	9
 #define SECURE_MESSAGE  "This rsh session is using DES encryption for all transmissions.\r\n"
-#define	OPTIONS		"alnkvxL"
+#define	OPTIONS		"alnkvxLP"
 char	authbuf[sizeof(AUTH_DAT)];
 char	tickbuf[sizeof(KTEXT_ST)];
 int	doencrypt, use_kerberos, vacuous;
@@ -106,6 +107,9 @@ main(argc, argv)
 		case 'L':
 			log_success = 1;
 			break;
+		case 'P':
+			new_pag = 0;
+			break;
 		case '?':
 		default:
 			usage();
@@ -584,6 +588,11 @@ fail:
 	if (setlogin(pwd->pw_name) < 0)
 		syslog(LOG_ERR, "setlogin() failed: %m");
 #endif
+
+#ifdef HAVE_SETPCRED
+	if (setpcred (pwd->pw_name, NULL) == -1)
+		syslog(LOG_ERR, "setpcred() failure: %m");
+#endif /* HAVE_SETPCRED */
 	(void) setgid((gid_t)pwd->pw_gid);
 	initgroups(pwd->pw_name, pwd->pw_gid);
 	(void) setuid((uid_t)pwd->pw_uid);
@@ -614,7 +623,7 @@ fail:
 		    syslog(LOG_INFO|LOG_AUTH, "%s@%s as %s: cmd='%.80s'",
 			remuser, hostname, locuser, cmdbuf);
 	}
-	if (k_hasafs())
+	if (new_pag && k_hasafs())
 		k_setpag();	/* Put users process in an new pag */
 	execl(pwd->pw_shell, cp, "-c", cmdbuf, 0);
 	perror(pwd->pw_shell);
@@ -628,22 +637,12 @@ fail:
  */
 
 void
-#if __STDC__
 error(const char *fmt, ...)
-#else
-error(fmt, va_alist)
-	char *fmt;
-        va_dcl
-#endif
 {
 	va_list ap;
 	int len;
 	char *bp, buf[BUFSIZ];
-#if __STDC__
 	va_start(ap, fmt);
-#else
-	va_start(ap);
-#endif
 	bp = buf;
 	if (sent_null == 0) {
 		*bp++ = 1;
