@@ -33,28 +33,10 @@
 
 #include "bsd_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/appl/bsd/su.c,v 1.22 1995/10/24 14:03:11 bg Exp $");
+RCSID("$Id: su.c,v 1.29 1996/02/07 17:29:57 bg Exp $");
 
-#ifndef HAVE_SETENV
-static int
-setenv(const char *var, const char *val, int rewrite)
-{
-  char *t;
-
-  if (!rewrite && getenv(var) != 0)
-    return 0;
-  
-  if ((t = malloc(strlen(var) + strlen(val) + 2)) == 0)
-    return -1;
-
-  strcpy(t, var);
-  strcat(t, "=");
-  strcat(t, val);
-  if (putenv(t) == 0)
-    return 0;
-  else
-    return -1;
-}
+#ifdef SYSV_SHADOW
+#include "sysv_shadow.h"
 #endif
 
 #ifndef HAVE_SETPRIORITY
@@ -74,9 +56,10 @@ getpriority(int which, int who)
 #endif
 
 /* Handle '-' option after all the getopt options */
-#define	ARGSTR	"Kflm"
+#define	ARGSTR	"Kflmi:"
 
 int use_kerberos = 1;
+char *root_inst = "root";
 
 static int chshell();
 static char *ontty();
@@ -114,10 +97,13 @@ main(argc, argv)
 			asme = 1;
 			asthem = 0;
 			break;
+		case 'i':
+		        root_inst = optarg;
+			break;
 		case '?':
 		default:
-			(void)fprintf(stderr, "usage: ksu [%s] [-] [login]\n",
-			    ARGSTR);
+			(void)fprintf(stderr,
+				      "usage: ksu [-Kflm] [-i root-instance] [-] [login]\n");
 			exit(1);
 		}
 	/* Don't handle '-' option with getopt */
@@ -169,6 +155,12 @@ main(argc, argv)
 	  fprintf(stderr, "ksu: won't use /etc/passwd authentication\n");
 	  exit(1);
 #endif
+	  /* getpwnam() is not reentrant and kerberos might use it! */
+	  pwd = getpwnam(user);
+	  if (pwd == NULL){
+		  fprintf(stderr, "ksu: unknown login %s\n", user);
+		  exit(1);
+	  }
 	  /* only allow those in group zero to su to root. */
 	  if (pwd->pw_uid == 0 && (gr = getgrgid((gid_t)0)))
 	    for (g = gr->gr_mem;; ++g) {
@@ -294,7 +286,7 @@ main(argc, argv)
 		if (k_setpag() != 0)
 			perror("ksu: setpag");
 		code = k_afsklog(0, 0);
-		if (code != KSUCCESS)
+		if (code != KSUCCESS && code != KDC_PR_UNKNOWN)
 		    fprintf(stderr, "ksu: afsklog: %s\n", krb_err_txt[code]);
 	}
 
@@ -307,6 +299,9 @@ main(argc, argv)
 	exit(1);
 }
 
+/* this does *not* belong here */
+char *getusershell(void);
+
 static int
 chshell(sh)
 	char *sh;
@@ -350,7 +345,7 @@ kerberos(username, user, uid)
 		(void)fprintf(stderr, "ksu: not in %s's ACL.\n", user);
 		return (1);
 	}
-	sprintf(krbtkfile, "%s_%s_to_%s", TKT_ROOT, username, user);
+	sprintf(krbtkfile, "%s_%s_to_%s_%d", TKT_ROOT, username, user, getpid());
 
 	setenv("KRBTKFILE", krbtkfile, 1);
 	krb_set_tkt_string(krbtkfile);
@@ -358,7 +353,8 @@ kerberos(username, user, uid)
 	 * Set real as well as effective ID to 0 for the moment,
 	 * to make the kerberos library do the right thing.
 	 */
-#if 1
+#if 0
+#if 0
 	if (k_setreuid(uid, 0) < 0) {
 		perror("ksu: setuid");
 		return (1);
@@ -368,6 +364,7 @@ kerberos(username, user, uid)
 		perror("ksu: setuid");
 		return (1);
 	}
+#endif
 #endif
 	/*
 	 * Little trick here -- if we are su'ing to root,
@@ -382,18 +379,18 @@ kerberos(username, user, uid)
 	printf("User's %s%s%s@%s ",
 	       (uid == 0 ? username : user),
 	       (uid == 0 ? "." : ""),
-	       (uid == 0 ? "root" : ""),
+	       (uid == 0 ? root_inst : ""),
 	       lrealm);
 	fflush(stdout);
 	kerno = krb_get_pw_in_tkt((uid == 0 ? username : user),
-		(uid == 0 ? "root" : ""), lrealm,
+		(uid == 0 ? root_inst : ""), lrealm,
 	    	"krbtgt", lrealm, DEFAULT_TKT_LIFE, 0);
 
 	if (kerno != KSUCCESS) {
 		if (kerno == KDC_PR_UNKNOWN) {
 			fprintf(stderr, "principal unknown: %s.%s@%s\n",
 				(uid == 0 ? username : user),
-				(uid == 0 ? "root" : ""), lrealm);
+				(uid == 0 ? root_inst : ""), lrealm);
 			return (1);
 		}
 		(void)fprintf(stderr, "ksu: unable to su: %s\n",
@@ -471,8 +468,7 @@ kerberos(username, user, uid)
 }
 
 int
-koktologin(name, realm, toname)
-	char *name, *realm, *toname;
+koktologin(char *name, char *realm, char *toname)
 {
 	register AUTH_DAT *kdata;
 	AUTH_DAT kdata_st;
@@ -481,7 +477,7 @@ koktologin(name, realm, toname)
 	bzero((caddr_t) kdata, sizeof(*kdata));
 	(void)strcpy(kdata->pname, name);
 	(void)strcpy(kdata->pinst,
-	    ((strcmp(toname, "root") == 0) ? "root" : ""));
+	    ((strcmp(toname, "root") == 0) ? root_inst : ""));
 	(void)strcpy(kdata->prealm, realm);
 	return (kuserok(kdata, toname));
 }
