@@ -2,11 +2,11 @@
 
 #include "bsd_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/appl/bsd/sysv_shadow.c,v 1.3 1995/10/24 14:03:13 bg Exp $");
+RCSID("$Id: sysv_shadow.c,v 1.5 1996/02/11 00:35:21 assar Exp $");
 
 #ifdef SYSV_SHADOW
 
-#include <shadow.h>
+#include <sysv_shadow.h>
 
 /* sysv_expire - check account and password expiration times */
 
