@@ -2,14 +2,14 @@
 
 #include "bsd_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/appl/bsd/utmpx_login.c,v 1.4 1995/10/24 14:03:15 bg Exp $");
+RCSID("$Id: utmpx_login.c,v 1.7 1996/02/08 16:23:07 bg Exp $");
 
 /* utmpx_login - update utmp and wtmp after login */
 
 int
 utmpx_login(char *line, char *user, char *host)
 {
-#ifndef SYSV_UTMP
+#ifndef HAVE_UTMPX_H
     return 0;
 #else
     struct utmpx *ut;
@@ -34,7 +34,7 @@ utmpx_login(char *line, char *user, char *host)
 	    if (ut->ut_syslen > sizeof(ut->ut_host))
 		ut->ut_syslen = sizeof(ut->ut_host);
 	    ut->ut_type = USER_PROCESS;
-	    gettimeofday(&(ut->ut_tv));
+	    gettimeofday(&(ut->ut_tv), 0);
 	    pututxline(ut);
 	    updwtmpx(WTMPX_FILE, ut);
 	    ret = 0;
@@ -43,5 +43,5 @@ utmpx_login(char *line, char *user, char *host)
     }
     endutxent();
     return (ret);
-#endif /* SYSV_UTMP */
+#endif /* HAVE_UTMPX_H */
 }
