@@ -8,7 +8,7 @@
 
 #include "config.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/appl/kauth/kauth.c,v 1.32 1995/10/27 10:55:31 bg Exp $");
+RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/appl/kauth/kauth.c,v 1.40 1996/02/09 23:29:12 assar Exp $");
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -64,23 +64,26 @@ doexec(argc, argv)
   case 0:
     /* in child */
     execvp(argv[0], argv);
-    perror(prog);
+    fprintf(stderr, "%s: Can't exec program ``%s'' %s\n",
+	    prog, argv[0], k_strerror(errno));
     exit(1);
     break;
   default:
     /* in parent */
     do {
       ret = wait(&status);
-    } while (ret > 0 && !WIFEXITED(status) || ret < 0 && errno == EINTR);
+    } while ((ret > 0 && !WIFEXITED(status)) || (ret < 0 && errno == EINTR));
     if (ret < 0)
       perror("wait");
     dest_tkt();
+    if (k_hasafs())
+      k_unlog();	 
     break;
   }
 }
 
-static void
-renew()
+static RETSIGTYPE
+renew(int sig)
 {
   int code;
 
@@ -92,13 +95,48 @@ renew()
     fprintf(stderr, "%s: %s\n", prog, krb_err_txt[code]);
   else if (k_hasafs())
     {
-      if ((code = k_afsklog(0, realm)) != 0) {
+      if ((code = k_afsklog(0, realm)) != 0 && code != KDC_PR_UNKNOWN) {
         fprintf(stderr, "%s: %s\n", prog, krb_err_txt[code]);
       }
     }
 
   alarm(krb_life_to_time(0, lifetime)/2 - 60);
+#ifdef VOID_RETSIGTYPE
   return;
+#else
+  return (RETSIGTYPE)0;
+#endif
+}
+
+static int
+zrefresh()
+{
+  switch (fork()) {
+  case -1:
+      fprintf(stderr, "Warning %s: Failed to fork zrefresh\n", prog);
+      return -1;
+  case 0:
+      /* Child */
+      execlp("zrefresh", "zrefresh", 0);
+      execl("/usr/athena/bin/zrefresh", "zrefresh", 0);
+#if 0
+      fprintf(stderr, "Warning %s: Failed to exec zrefresh\n", prog);
+#endif
+      exit(1);
+  default:
+      /* Parent */
+      break;
+  }
+  return 0;
+}
+
+static
+void
+upcase(char *t)
+{
+    for (; *t; t++)
+	if ('a' <= *t && *t <= 'z')
+	    *t = *t - 'a' + 'A';
 }
 
 int
@@ -125,11 +163,8 @@ main(argc, argv)
       argv[1][0] != '-' &&
       kname_parse(name, inst, realm, argv[1]) == 0)
     {
-      char *t;
       argc--, argv++;
-      for (t = realm; *t; t++)
-	if ('a' <= *t && *t <= 'z')
-	  *t = *t - 'a' + 'A';
+      upcase(realm);
     }
 
   while ((c = getopt(argc, argv, "f:hl:n:c:")) != EOF)
@@ -154,6 +189,7 @@ main(argc, argv)
 	  fprintf(stderr, "%s\n", krb_err_txt[code]);
 	  usage();
 	}
+      upcase(realm);
       break;
     case 'c':
       cell = optarg;
@@ -233,54 +269,14 @@ main(argc, argv)
     {
       if (more_args)
         k_setpag();
-      if ((code = k_afsklog(cell, realm)) != 0)
+      if ((code = k_afsklog(cell, realm)) != 0 && code != KDC_PR_UNKNOWN)
         fprintf(stderr, "%s: %s\n", prog, krb_err_txt[code]);
     }
 
   if (more_args)
     doexec(more_args, &argv[optind]);
-#ifdef ZEPHYR
   else
-    zrenew();
-#endif
+    zrefresh();
 
   exit(0);
 }
-
-#ifdef ZEPHYR
-#include <zephyr/zephyr.h>
-
-static int
-zrenew()
-{
-  Code_t code;
-  int wgport;
-
-  char *display = getenv("DISPLAY");
-  if (display != 0 && *display)
-    {
-      char *t = strchr(display, ':');
-      if (t != 0)
-	if (   strcmp(t, ":0.0") == 0
-	    || strcmp(t, ":0.1") == 0)
-	  t[2] = '\0';		/* Chop of .0 */
-    }
-
-  if ((wgport = ZGetWGPort()) != -1)
-    if ((code = ZInitialize() == ZERR_NONE))
-      if ((code = ZSubscribeTo(0, 0, (u_short)wgport)) == ZERR_NONE)
-	{
-	  char buf[100], *t;
-	  t = ZGetVariable("exposure");
-
-	  strncpy(buf, t, sizeof(buf));
-	  for (t = buf; *t; t++)
-	    if (islower(*t))
-	      *t = toupper(*t);
-
-	  if ((code = ZSetLocation(buf)) == ZERR_NONE)
-	    return 0;
-	}
-  return -1;
-}
-#endif
