@@ -7,7 +7,7 @@
 #ifndef lint
 static char copyright[] = "Copyright (c) 1990 Regents of the University of California.\nAll rights reserved.\n";
 static char SccsId[] = "@(#)@(#)pop_updt.c	2.3  2.3 3/20/91";
-#endif not lint
+#endif /* not lint */
 
 #include <errno.h>
 #include <stdio.h>
@@ -52,7 +52,7 @@ POP     *   p;
         pop_log(p,POP_DEBUG,"Performing maildrop update...");
         pop_log(p,POP_DEBUG,"Checking to see if all messages were deleted");
     }
-#endif DEBUG
+#endif /* DEBUG */
 
     if (p->msgs_deleted == p->msg_count) {
         /* Truncate before close, to avoid race condition,  DO NOT UNLINK!
@@ -65,7 +65,7 @@ POP     *   p;
 #ifdef DEBUG
     if (p->debug) 
         pop_log(p,POP_DEBUG,"Opening mail drop \"%s\"",p->drop_name);
-#endif DEBUG
+#endif /* DEBUG */
 
     /*  Open the user's real maildrop */
     if ((mfd = open(p->drop_name,O_RDWR|O_CREAT,0600)) == -1 ||
@@ -110,7 +110,7 @@ POP     *   p;
     if (p->debug) 
         pop_log(p,POP_DEBUG,"Creating new maildrop \"%s\" from \"%s\"",
                 p->drop_name,p->temp_drop);
-#endif DEBUG
+#endif /* DEBUG */
 
     for (msg_num = 0; msg_num < p->msg_count; ++msg_num) {
 
@@ -124,7 +124,7 @@ POP     *   p;
             if(p->debug)
                 pop_log(p,POP_DEBUG,
                     "Message %d flagged for deletion.",mp->number);
-#endif DEBUG
+#endif /* DEBUG */
             continue;
         }
 
@@ -133,7 +133,7 @@ POP     *   p;
 #ifdef DEBUG
         if(p->debug)
             pop_log(p,POP_DEBUG,"Copying message %d.",mp->number);
-#endif DEBUG
+#endif /* DEBUG */
         for(status_written = doing_body = 0 ;
             fgets(buffer,MAXMSGLINELEN,p->drop);) {
 
