@@ -1,7 +1,7 @@
 /*
  *	$Source: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/appl/telnet/libtelnet/kerberos5.c,v $
- *	$Author: d91-jda $
- *	$Id: kerberos5.c,v 1.2 1995/07/19 23:44:54 d91-jda Exp $
+ *	$Author: bg $
+ *	$Id: kerberos5.c,v 1.4 1995/12/27 13:24:51 bg Exp $
  */
 
 #if !defined(lint) && !defined(SABER)
@@ -9,7 +9,7 @@ static
 #ifdef __STDC__
 const
 #endif
-char rcsid_kerberos5_c[] = "$Id: kerberos5.c,v 1.2 1995/07/19 23:44:54 d91-jda Exp $";
+char rcsid_kerberos5_c[] = "$Id: kerberos5.c,v 1.4 1995/12/27 13:24:51 bg Exp $";
 #endif /* lint */
 
 /*-
@@ -45,10 +45,6 @@ char rcsid_kerberos5_c[] = "$Id: kerberos5.c,v 1.2 1995/07/19 23:44:54 d91-jda E
  * SUCH DAMAGE.
  */
 
-#ifndef lint
-static char sccsid[] = "@(#)kerberos5.c	8.3 (Berkeley) 5/30/95";
-#endif /* not lint */
-
 /*
  * Copyright (C) 1990 by the Massachusetts Institute of Technology
  *
@@ -71,6 +67,8 @@ static char sccsid[] = "@(#)kerberos5.c	8.3 (Berkeley) 5/30/95";
 
 #include <config.h>
 
+RCSID("$Id: kerberos5.c,v 1.4 1995/12/27 13:24:51 bg Exp $");
+
 #ifdef	KRB5
 #include <arpa/telnet.h>
 #include <stdio.h>
@@ -131,7 +129,7 @@ static	krb5_authenticator authenticator;
 
 #define Voidptr krb5_pointer
 
-Block	session_key;
+des_cblock	session_key;
 
 	static int
 Data(ap, type, d, c)
@@ -430,12 +428,12 @@ kerberos5_is(ap, data, cnt)
 		    authdat->authenticator->subkey->keytype == KEYTYPE_DES) {
 		    memmove((Voidptr )session_key,
 			   (Voidptr )authdat->authenticator->subkey->contents,
-			   sizeof(Block));
+			   sizeof(des_cblock));
 		} else if (authdat->ticket->enc_part2->session->keytype ==
 			   KEYTYPE_DES) {
 		    memmove((Voidptr )session_key,
 			(Voidptr )authdat->ticket->enc_part2->session->contents,
-			sizeof(Block));
+			sizeof(des_cblock));
 		} else
 		    break;
 
@@ -518,7 +516,7 @@ kerberos5_reply(ap, data, cnt)
 
 		    tmpkey.keytype = KEYTYPE_DES;
 		    tmpkey.contents = session_key;
-		    tmpkey.length = sizeof(Block);
+		    tmpkey.length = sizeof(des_cblock);
 
 		    if (r = krb5_rd_rep(&inbuf, &tmpkey, &reply)) {
 			printf("[ Mutual authentication failed: %s ]\n",
