@@ -31,16 +31,14 @@
  * SUCH DAMAGE.
  */
 
-#ifndef lint
-static char sccsid[] = "@(#)sys_term.c	8.4 (Berkeley) 5/30/95";
-#endif /* not lint */
-
 #include <config.h>
 
+RCSID("$Id: sys_term.c,v 1.33 1996/02/11 23:21:10 d91-jda Exp $");
+
 #include "telnetd.h"
 #include "pathnames.h"
 
-#if	defined(AUTHENTICATION)
+#ifdef AUTHENTICATION
 #include <libtelnet/auth.h>
 #endif
 
@@ -85,14 +83,21 @@ extern struct sysv sysv;
 #endif	/* NEWINIT */
 
 #ifdef	STREAMSPTY
+
+#ifdef HAVE_SAC_H
 #include <sac.h>
+#endif
+
 #include <sys/stropts.h>
 #endif
 
 #define SCPYN(a, b)	(void) strncpy(a, b, sizeof(a))
 #define SCMPN(a, b)	strncmp(a, b, sizeof(a))
 
-#ifdef	STREAMS
+#ifdef	HAVE_SYS_STREAM_H
+#ifdef  HAVE_SYS_UIO_H
+#include <sys/uio.h>
+#endif
 #include <sys/stream.h>
 #endif
 #ifdef __hpux
@@ -101,7 +106,7 @@ extern struct sysv sysv;
 #include <sys/proc.h>
 #undef SE
 #endif
-#if !(defined(__sgi) || defined(__linux))
+#if !(defined(__sgi) || defined(__linux) || defined(_AIX)) && defined(HAVE_SYS_TTY)
 #include <sys/tty.h>
 #endif
 #ifdef	t_erase
@@ -125,22 +130,6 @@ extern struct sysv sysv;
 # define EXTPROC 0400
 #endif
 
-#ifndef	USE_TERMIO
-struct termbuf {
-	struct sgttyb sg;
-	struct tchars tc;
-	struct ltchars ltc;
-	int state;
-	int lflags;
-} termbuf, termbuf2;
-# define	cfsetospeed(tp, val)	(tp)->sg.sg_ospeed = (val)
-# define	cfsetispeed(tp, val)	(tp)->sg.sg_ispeed = (val)
-# define	cfgetospeed(tp)		(tp)->sg.sg_ospeed
-# define	cfgetispeed(tp)		(tp)->sg.sg_ispeed
-#else	/* USE_TERMIO */
-# ifdef	SYSV_TERMIO
-#	define termios termio
-# endif
 # ifndef	TCSANOW
 #  ifdef TCSETS
 #   define	TCSANOW		TCSETS
@@ -175,7 +164,6 @@ struct termios termbuf, termbuf2;	/* pty control structure */
 # ifdef  STREAMSPTY
 int ttyfd = -1;
 # endif
-#endif	/* USE_TERMIO */
 
 char *new_login = LOGIN_PATH;
 
@@ -190,23 +178,14 @@ char *new_login = LOGIN_PATH;
  * set_termbuf() writes the structure into the kernel.
  */
 
-	void
-init_termbuf()
+void
+init_termbuf(void)
 {
-#ifndef	USE_TERMIO
-	(void) ioctl(pty, TIOCGETP, (char *)&termbuf.sg);
-	(void) ioctl(pty, TIOCGETC, (char *)&termbuf.tc);
-	(void) ioctl(pty, TIOCGLTC, (char *)&termbuf.ltc);
-# ifdef	TIOCGSTATE
-	(void) ioctl(pty, TIOCGSTATE, (char *)&termbuf.state);
-# endif
-#else
 # ifdef  STREAMSPTY
 	(void) tcgetattr(ttyfd, &termbuf);
 # else
-	(void) tcgetattr(pty, &termbuf);
+	(void) tcgetattr(ourpty, &termbuf);
 # endif
-#endif
 	termbuf2 = termbuf;
 }
 
@@ -224,34 +203,20 @@ copy_termbuf(cp, len)
 #endif	/* defined(LINEMODE) && defined(TIOCPKT_IOCTL) */
 
 	void
-set_termbuf()
+set_termbuf(void)
 {
 	/*
 	 * Only make the necessary changes.
 	 */
-#ifndef	USE_TERMIO
-	if (memcmp((char *)&termbuf.sg, (char *)&termbuf2.sg,
-							sizeof(termbuf.sg)))
-		(void) ioctl(pty, TIOCSETN, (char *)&termbuf.sg);
-	if (memcmp((char *)&termbuf.tc, (char *)&termbuf2.tc,
-							sizeof(termbuf.tc)))
-		(void) ioctl(pty, TIOCSETC, (char *)&termbuf.tc);
-	if (memcmp((char *)&termbuf.ltc, (char *)&termbuf2.ltc,
-							sizeof(termbuf.ltc)))
-		(void) ioctl(pty, TIOCSLTC, (char *)&termbuf.ltc);
-	if (termbuf.lflags != termbuf2.lflags)
-		(void) ioctl(pty, TIOCLSET, (char *)&termbuf.lflags);
-#else	/* USE_TERMIO */
 	if (memcmp((char *)&termbuf, (char *)&termbuf2, sizeof(termbuf)))
 # ifdef  STREAMSPTY
 		(void) tcsetattr(ttyfd, TCSANOW, &termbuf);
 # else
-		(void) tcsetattr(pty, TCSANOW, &termbuf);
+		(void) tcsetattr(ourpty, TCSANOW, &termbuf);
 # endif
 # if	defined(CRAY2) && defined(UNICOS5)
 	needtermstat = 1;
 # endif
-#endif	/* USE_TERMIO */
 }
 
 
@@ -266,87 +231,9 @@ set_termbuf()
  * It returns the SLC_ level of support for this function.
  */
 
-#ifndef	USE_TERMIO
-	int
-spcset(func, valp, valpp)
-	int func;
-	cc_t *valp;
-	cc_t **valpp;
-{
-	switch(func) {
-	case SLC_EOF:
-		*valp = termbuf.tc.t_eofc;
-		*valpp = (cc_t *)&termbuf.tc.t_eofc;
-		return(SLC_VARIABLE);
-	case SLC_EC:
-		*valp = termbuf.sg.sg_erase;
-		*valpp = (cc_t *)&termbuf.sg.sg_erase;
-		return(SLC_VARIABLE);
-	case SLC_EL:
-		*valp = termbuf.sg.sg_kill;
-		*valpp = (cc_t *)&termbuf.sg.sg_kill;
-		return(SLC_VARIABLE);
-	case SLC_IP:
-		*valp = termbuf.tc.t_intrc;
-		*valpp = (cc_t *)&termbuf.tc.t_intrc;
-		return(SLC_VARIABLE|SLC_FLUSHIN|SLC_FLUSHOUT);
-	case SLC_ABORT:
-		*valp = termbuf.tc.t_quitc;
-		*valpp = (cc_t *)&termbuf.tc.t_quitc;
-		return(SLC_VARIABLE|SLC_FLUSHIN|SLC_FLUSHOUT);
-	case SLC_XON:
-		*valp = termbuf.tc.t_startc;
-		*valpp = (cc_t *)&termbuf.tc.t_startc;
-		return(SLC_VARIABLE);
-	case SLC_XOFF:
-		*valp = termbuf.tc.t_stopc;
-		*valpp = (cc_t *)&termbuf.tc.t_stopc;
-		return(SLC_VARIABLE);
-	case SLC_AO:
-		*valp = termbuf.ltc.t_flushc;
-		*valpp = (cc_t *)&termbuf.ltc.t_flushc;
-		return(SLC_VARIABLE);
-	case SLC_SUSP:
-		*valp = termbuf.ltc.t_suspc;
-		*valpp = (cc_t *)&termbuf.ltc.t_suspc;
-		return(SLC_VARIABLE);
-	case SLC_EW:
-		*valp = termbuf.ltc.t_werasc;
-		*valpp = (cc_t *)&termbuf.ltc.t_werasc;
-		return(SLC_VARIABLE);
-	case SLC_RP:
-		*valp = termbuf.ltc.t_rprntc;
-		*valpp = (cc_t *)&termbuf.ltc.t_rprntc;
-		return(SLC_VARIABLE);
-	case SLC_LNEXT:
-		*valp = termbuf.ltc.t_lnextc;
-		*valpp = (cc_t *)&termbuf.ltc.t_lnextc;
-		return(SLC_VARIABLE);
-	case SLC_FORW1:
-		*valp = termbuf.tc.t_brkc;
-		*valpp = (cc_t *)&termbuf.ltc.t_lnextc;
-		return(SLC_VARIABLE);
-	case SLC_BRK:
-	case SLC_SYNCH:
-	case SLC_AYT:
-	case SLC_EOR:
-		*valp = (cc_t)0;
-		*valpp = (cc_t *)0;
-		return(SLC_DEFAULT);
-	default:
-		*valp = (cc_t)0;
-		*valpp = (cc_t *)0;
-		return(SLC_NOSUPPORT);
-	}
-}
-
-#else	/* USE_TERMIO */
 
 	int
-spcset(func, valp, valpp)
-	int func;
-	cc_t *valp;
-	cc_t **valpp;
+spcset(int func, cc_t *valp, cc_t **valpp)
 {
 
 #define	setval(a, b)	*valp = termbuf.c_cc[a]; \
@@ -436,7 +323,6 @@ spcset(func, valp, valpp)
 		return(SLC_NOSUPPORT);
 	}
 }
-#endif	/* USE_TERMIO */
 
 #ifdef CRAY
 /*
@@ -467,40 +353,69 @@ getnpty()
  *
  * Returns the file descriptor of the opened pty.
  */
-#ifndef	__GNUC__
-char *line = "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0";
-#else
+
 static char Xline[] = "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0";
 char *line = Xline;
-#endif
+
 #ifdef	CRAY
 char *myline = "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0";
 #endif	/* CRAY */
 
-	int
-getpty(ptynum)
-int *ptynum;
+#ifndef HAVE_PTSNAME
+static char *ptsname(int fd)
 {
-	register int p;
-#ifdef	STREAMSPTY
-	int t;
-	char *ptsname();
-
-	p = open("/dev/ptmx", 2);
-	if (p > 0) {
-		grantpt(p);
-		unlockpt(p);
-		strcpy(line, ptsname(p));
-		return(p);
-	}
-
-#else	/* ! STREAMSPTY */
-#ifndef CRAY
-	register char *cp, *p1, *p2;
-	register int i;
-#if defined(sun) && defined(TIOCGPGRP) && BSD < 199207
-	int dummy;
+#ifdef HAVE_TTYNAME
+  return ttyname(fd);
+#else
+  return NULL;
 #endif
+}
+#endif
+
+int getpty(int *ptynum)
+{
+#ifdef __osf__
+        int master;
+	int slave;
+	if(openpty(&master, &slave, line, 0, 0) == 0){
+	  close(slave);
+	  return master;
+	}
+	return -1;
+#else
+
+        int p;
+	char *cp, *p1, *p2;
+	int i;
+	int dummy;
+#if 0 /* && defined(HAVE_OPENPTY) */
+        int master;
+	int slave;
+	if(openpty(&master, &slave, line, 0, 0) == 0){
+	  close(slave);
+	  return master;
+	}
+#else
+#ifdef	STREAMSPTY
+	char *clone[] = { "/dev/ptc", "/dev/ptmx", "/dev/ptm", 
+			  "/dev/ptym/clone", 0 };
+
+	char **q;
+	for(q=clone; *q; q++){
+	  p=open(*q, O_RDWR);
+	  if(p >= 0){
+#ifdef HAVE_GRANTPT
+	    grantpt(p);
+#endif
+#ifdef HAVE_UNLOCKPT
+	    unlockpt(p);
+#endif
+	    strcpy(line, ptsname(p));
+	    return p;
+	  }
+	}
+#endif /* STREAMSPTY */
+#ifndef CRAY
 
 #ifndef	__hpux
 	(void) sprintf(line, "/dev/ptyXX");
@@ -512,6 +427,7 @@ int *ptynum;
 	p2 = &line[14];
 #endif
 
+	
 	for (cp = "pqrstuvwxyzPQRST"; *cp; cp++) {
 		struct stat stb;
 
@@ -526,7 +442,7 @@ int *ptynum;
 			break;
 		for (i = 0; i < 16; i++) {
 			*p2 = "0123456789abcdef"[i];
-			p = open(line, 2);
+			p = open(line, O_RDWR);
 			if (p > 0) {
 #ifndef	__hpux
 				line[5] = 't';
@@ -537,14 +453,14 @@ int *ptynum;
 #endif
 				chown(line, 0, 0);
 				chmod(line, 0600);
-#if defined(sun) && defined(TIOCGPGRP) && BSD < 199207
+#if SunOS == 4
 				if (ioctl(p, TIOCGPGRP, &dummy) == 0
 				    || errno != EIO) {
 					chmod(line, 0666);
 					close(p);
 					line[5] = 'p';
 				} else
-#endif /* defined(sun) && defined(TIOCGPGRP) && BSD < 199207 */
+#endif /* SunOS == 4 */
 					return(p);
 			}
 		}
@@ -587,9 +503,10 @@ int *ptynum;
 	}
 #endif	/* CRAY */
 #endif	/* STREAMSPTY */
+#endif /* OPENPTY */
 	return(-1);
+#endif
 }
-#endif	/* convex */
 
 #ifdef	LINEMODE
 /*
@@ -621,11 +538,7 @@ static int linestate;
 tty_linemode()
 {
 #ifndef convex
-#ifndef	USE_TERMIO
-	return(termbuf.state & TS_EXTPROC);
-#else
 	return(termbuf.c_lflag & EXTPROC);
-#endif
 #else
 	return(linestate);
 #endif
@@ -641,7 +554,7 @@ tty_setlinemode(on)
 # else
 	linestate = on;
 # endif
-	(void) ioctl(pty, TIOCEXT, (char *)&on);
+	(void) ioctl(ourpty, TIOCEXT, (char *)&on);
 # ifndef convex
 	init_termbuf();
 # endif
@@ -657,112 +570,62 @@ tty_setlinemode(on)
 #endif	/* LINEMODE */
 
 	int
-tty_isecho()
+tty_isecho(void)
 {
-#ifndef USE_TERMIO
-	return (termbuf.sg.sg_flags & ECHO);
-#else
 	return (termbuf.c_lflag & ECHO);
-#endif
 }
 
 	int
-tty_flowmode()
+tty_flowmode(void)
 {
-#ifndef USE_TERMIO
-	return(((termbuf.tc.t_startc) > 0 && (termbuf.tc.t_stopc) > 0) ? 1 : 0);
-#else
 	return((termbuf.c_iflag & IXON) ? 1 : 0);
-#endif
 }
 
 	int
-tty_restartany()
+tty_restartany(void)
 {
-#ifndef USE_TERMIO
-# ifdef	DECCTQ
-	return((termbuf.lflags & DECCTQ) ? 0 : 1);
-# else
-	return(-1);
-# endif
-#else
 	return((termbuf.c_iflag & IXANY) ? 1 : 0);
-#endif
 }
 
 	void
-tty_setecho(on)
-	int on;
+tty_setecho(int on)
 {
-#ifndef	USE_TERMIO
-	if (on)
-		termbuf.sg.sg_flags |= ECHO|CRMOD;
-	else
-		termbuf.sg.sg_flags &= ~(ECHO|CRMOD);
-#else
 	if (on)
 		termbuf.c_lflag |= ECHO;
 	else
 		termbuf.c_lflag &= ~ECHO;
-#endif
 }
 
 	int
-tty_israw()
+tty_israw(void)
 {
-#ifndef USE_TERMIO
-	return(termbuf.sg.sg_flags & RAW);
-#else
 	return(!(termbuf.c_lflag & ICANON));
-#endif
 }
 
 #if	defined (AUTHENTICATION) && defined(NO_LOGIN_F) && defined(LOGIN_R)
 	int
 tty_setraw(on)
 {
-#  ifndef USE_TERMIO
-	if (on)
-		termbuf.sg.sg_flags |= RAW;
-	else
-		termbuf.sg.sg_flags &= ~RAW;
-#  else
 	if (on)
 		termbuf.c_lflag &= ~ICANON;
 	else
 		termbuf.c_lflag |= ICANON;
-#  endif
 }
 #endif
 
 	void
-tty_binaryin(on)
-	int on;
+tty_binaryin(int on)
 {
-#ifndef	USE_TERMIO
-	if (on)
-		termbuf.lflags |= LPASS8;
-	else
-		termbuf.lflags &= ~LPASS8;
-#else
 	if (on) {
 		termbuf.c_iflag &= ~ISTRIP;
 	} else {
 		termbuf.c_iflag |= ISTRIP;
 	}
-#endif
 }
 
 	void
-tty_binaryout(on)
-	int on;
+tty_binaryout(int on)
 {
-#ifndef	USE_TERMIO
-	if (on)
-		termbuf.lflags |= LLITOUT;
-	else
-		termbuf.lflags &= ~LLITOUT;
-#else
 	if (on) {
 		termbuf.c_cflag &= ~(CSIZE|PARENB);
 		termbuf.c_cflag |= CS8;
@@ -772,108 +635,68 @@ tty_binaryout(on)
 		termbuf.c_cflag |= CS7|PARENB;
 		termbuf.c_oflag |= OPOST;
 	}
-#endif
 }
 
 	int
-tty_isbinaryin()
+tty_isbinaryin(void)
 {
-#ifndef	USE_TERMIO
-	return(termbuf.lflags & LPASS8);
-#else
 	return(!(termbuf.c_iflag & ISTRIP));
-#endif
 }
 
 	int
-tty_isbinaryout()
+tty_isbinaryout(void)
 {
-#ifndef	USE_TERMIO
-	return(termbuf.lflags & LLITOUT);
-#else
 	return(!(termbuf.c_oflag&OPOST));
-#endif
 }
 
 #ifdef	LINEMODE
 	int
 tty_isediting()
 {
-#ifndef USE_TERMIO
-	return(!(termbuf.sg.sg_flags & (CBREAK|RAW)));
-#else
 	return(termbuf.c_lflag & ICANON);
-#endif
 }
 
 	int
 tty_istrapsig()
 {
-#ifndef USE_TERMIO
-	return(!(termbuf.sg.sg_flags&RAW));
-#else
 	return(termbuf.c_lflag & ISIG);
-#endif
 }
 
 	void
 tty_setedit(on)
 	int on;
 {
-#ifndef USE_TERMIO
-	if (on)
-		termbuf.sg.sg_flags &= ~CBREAK;
-	else
-		termbuf.sg.sg_flags |= CBREAK;
-#else
 	if (on)
 		termbuf.c_lflag |= ICANON;
 	else
 		termbuf.c_lflag &= ~ICANON;
-#endif
 }
 
 	void
 tty_setsig(on)
 	int on;
 {
-#ifndef	USE_TERMIO
-	if (on)
-		;
-#else
 	if (on)
 		termbuf.c_lflag |= ISIG;
 	else
 		termbuf.c_lflag &= ~ISIG;
-#endif
 }
 #endif	/* LINEMODE */
 
 	int
-tty_issofttab()
+tty_issofttab(void)
 {
-#ifndef	USE_TERMIO
-	return (termbuf.sg.sg_flags & XTABS);
-#else
 # ifdef	OXTABS
 	return (termbuf.c_oflag & OXTABS);
 # endif
 # ifdef	TABDLY
 	return ((termbuf.c_oflag & TABDLY) == TAB3);
 # endif
-#endif
 }
 
 	void
-tty_setsofttab(on)
-	int on;
+tty_setsofttab(int on)
 {
-#ifndef	USE_TERMIO
-	if (on)
-		termbuf.sg.sg_flags |= XTABS;
-	else
-		termbuf.sg.sg_flags &= ~XTABS;
-#else
 	if (on) {
 # ifdef	OXTABS
 		termbuf.c_oflag |= OXTABS;
@@ -891,15 +714,11 @@ tty_setsofttab(on)
 		termbuf.c_oflag |= TAB0;
 # endif
 	}
-#endif
 }
 
 	int
-tty_islitecho()
+tty_islitecho(void)
 {
-#ifndef	USE_TERMIO
-	return (!(termbuf.lflags & LCTLECH));
-#else
 # ifdef	ECHOCTL
 	return (!(termbuf.c_lflag & ECHOCTL));
 # endif
@@ -909,19 +728,11 @@ tty_islitecho()
 # if	!defined(ECHOCTL) && !defined(TCTLECH)
 	return (0);	/* assumes ctl chars are echoed '^x' */
 # endif
-#endif
 }
 
 	void
-tty_setlitecho(on)
-	int on;
+tty_setlitecho(int on)
 {
-#ifndef	USE_TERMIO
-	if (on)
-		termbuf.lflags &= ~LCTLECH;
-	else
-		termbuf.lflags |= LCTLECH;
-#else
 # ifdef	ECHOCTL
 	if (on)
 		termbuf.c_lflag &= ~ECHOCTL;
@@ -934,17 +745,12 @@ tty_setlitecho(on)
 	else
 		termbuf.c_lflag |= TCTLECH;
 # endif
-#endif
 }
 
 	int
-tty_iscrnl()
+tty_iscrnl(void)
 {
-#ifndef	USE_TERMIO
-	return (termbuf.sg.sg_flags & CRMOD);
-#else
 	return (termbuf.c_iflag & ICRNL);
-#endif
 }
 
 /*
@@ -998,8 +804,7 @@ struct termspeeds {
 #endif	/* DECODE_BUAD */
 
 	void
-tty_tspeed(val)
-	int val;
+tty_tspeed(int val)
 {
 #ifdef	DECODE_BAUD
 	register struct termspeeds *tp;
@@ -1015,8 +820,7 @@ tty_tspeed(val)
 }
 
 	void
-tty_rspeed(val)
-	int val;
+tty_rspeed(int val)
 {
 #ifdef	DECODE_BAUD
 	register struct termspeeds *tp;
@@ -1047,10 +851,9 @@ extern char wtmpf[];
 # else	/* NEWINIT */
 int	gotalarm;
 
-	/* ARGSUSED */
-	void
-nologinproc(sig)
-	int sig;
+/* ARGSUSED */
+void
+nologinproc(int sig)
 {
 	gotalarm++;
 }
@@ -1066,6 +869,62 @@ extern void utmp_sig_notify P((int));
 # endif /* PARENT_DOES_UTMP */
 #endif
 
+#ifdef STREAMSPTY
+
+/* I_FIND seems to live a life of its own */
+static int my_find(int fd, char *module)
+{
+  static int flag;
+  static struct str_list sl;
+  int n;
+  int i;
+  
+  if(!flag){
+    n = ioctl(fd, I_LIST, 0);
+    if(n < 0){
+      perror("ioctl(fd, I_LIST, 0)");
+      return -1;
+    }
+    sl.sl_modlist=(struct str_mlist*)malloc(n * sizeof(struct str_mlist));
+    sl.sl_nmods = n;
+    n = ioctl(fd, I_LIST, &sl);
+    if(n < 0){
+      perror("ioctl(fd, I_LIST, n)");
+      return -1;
+    }
+    flag = 1;
+  }
+  
+  for(i=0; i<sl.sl_nmods; i++)
+    if(!strcmp(sl.sl_modlist[i].l_name, module))
+      return 1;
+  return 0;
+}
+
+static void maybe_push_modules(int fd, char **modules)
+{
+  char **p;
+  int err;
+
+  for(p=modules; *p; p++){
+    err = my_find(fd, *p);
+    if(err == 1)
+      break;
+    if(err < 0 && errno != EINVAL)
+      fatalperror(net, "my_find()");
+    /* module not pushed or does not exist */
+  }
+  /* p points to null or to an already pushed module, now push all
+     modules before this one */
+  
+  for(p--; p >= modules; p--){
+    err = ioctl(fd, I_PUSH, *p);
+    if(err < 0 && errno != EINVAL)
+      fatalperror(net, "I_PUSH");
+  }
+}
+#endif
+
 /*
  * getptyslave()
  *
@@ -1073,8 +932,7 @@ extern void utmp_sig_notify P((int));
  * that is necessary.  The return value is a file descriptor
  * for the slave side.
  */
-	int
-getptyslave()
+int getptyslave(void)
 {
 	register int t = -1;
 
@@ -1104,6 +962,11 @@ getptyslave()
 	 * Make sure that we don't have a controlling tty, and
 	 * that we are the session (process group) leader.
 	 */
+
+#ifdef HAVE_SETSID
+	if(setsid()<0)
+	  fatalperror(net, "setsid()");
+#else
 # ifdef	TIOCNOTTY
 	t = open(_PATH_TTY, O_RDWR);
 	if (t >= 0) {
@@ -1111,7 +974,7 @@ getptyslave()
 		(void) close(t);
 	}
 # endif
-
+#endif
 
 # ifdef PARENT_DOES_UTMP
 	/*
@@ -1125,23 +988,34 @@ getptyslave()
 		fatalperror(net, line);
 
 #ifdef  STREAMSPTY
-#ifdef	USE_TERMIO
 	ttyfd = t;
-#endif
-	if (ioctl(t, I_PUSH, "ptem") < 0)
-		fatal(net, "I_PUSH ptem");
-	if (ioctl(t, I_PUSH, "ldterm") < 0)
-		fatal(net, "I_PUSH ldterm");
+	  
+
 	/*
 	 * Not all systems have (or need) modules ttcompat and pckt so
 	 * don't flag it as a fatal error if they don't exist.
 	 */
-	if (ioctl(t, I_PUSH, "ttcompat") < 0 && errno != EINVAL)
-		fatal(net, "I_PUSH ttcompat");
-	if (ioctl(pty, I_PUSH, "pckt") < 0 && errno != EINVAL)
-		fatal(net, "I_PUSH pckt");
-#endif
 
+	{
+	  /* these are the streams modules that we want pushed. note
+	     that they are in reverse order, ptem will be pushed
+	     first. maybe_push_modules() will try to push all modules
+	     before the first one that isn't already pushed. i.e if
+	     ldterm is pushed, only ttcompat will be attempted.
+
+	     all this is because we don't know which modules are
+	     available, and we don't know which modules are already
+	     pushed (via autopush, for instance).
+
+	     */
+	     
+	  char *ttymodules[] = { "ttcompat", "ldterm", "ptem", NULL };
+	  char *ptymodules[] = { "pckt", NULL };
+
+	  maybe_push_modules(t, ttymodules);
+	  maybe_push_modules(ourpty, ptymodules);
+	}
+#endif
 	/*
 	 * set up the tty modes as we like them to be.
 	 */
@@ -1158,9 +1032,6 @@ getptyslave()
 	/*
 	 * Settings for sgtty based systems
 	 */
-# ifndef	USE_TERMIO
-	termbuf.sg.sg_flags |= CRMOD|ANYP|ECHO|XTABS;
-# endif	/* USE_TERMIO */
 
 	/*
 	 * Settings for UNICOS (and HPUX)
@@ -1177,7 +1048,7 @@ getptyslave()
 	 * systems, other than 4.4BSD.  In 4.4BSD the
 	 * kernel does the initial terminal setup.
 	 */
-# if defined(USE_TERMIO) && !(defined(CRAY) || defined(__hpux)) && (BSD <= 43)
+# if !(defined(CRAY) || defined(__hpux)) && (BSD <= 43)
 #  ifndef	OXTABS
 #   define OXTABS	0
 #  endif
@@ -1185,7 +1056,7 @@ getptyslave()
 	termbuf.c_oflag |= ONLCR|OXTABS;
 	termbuf.c_iflag |= ICRNL;
 	termbuf.c_iflag &= ~IXOFF;
-# endif /* defined(USE_TERMIO) && !defined(CRAY) && (BSD <= 43) */
+# endif
 	tty_rspeed((def_rspeed > 0) ? def_rspeed : 9600);
 	tty_tspeed((def_tspeed > 0) ? def_tspeed : 9600);
 # ifdef	LINEMODE
@@ -1208,9 +1079,9 @@ getptyslave()
 	 * protocol for /bin/login, if the authentication works.
 	 */
 #else
-	if (pty > 2) {
-		(void) close(pty);
-		pty = -1;
+	if (ourpty > 2) {
+		(void) close(ourpty);
+		ourpty = -1;
 	}
 #endif
 }
@@ -1223,9 +1094,8 @@ getptyslave()
  * Open the specified slave side of the pty,
  * making sure that we have a clean tty.
  */
-	int
-cleanopen(line)
-	char *line;
+
+int cleanopen(char *line)
 {
 	register int t;
 #ifdef	UNICOS7x
@@ -1323,13 +1193,12 @@ cleanopen(line)
 }
 #endif	/* !defined(CRAY) || !defined(NEWINIT) */
 
-#if BSD <= 43
+#if !defined(BSD4_4)
 
-	int
-login_tty(t)
-	int t;
+int login_tty(int t)
 {
-#ifndef _AIX
+#if 0 /* setsid done in other place */
+#if defined(HAVE_SETSID) && !defined(_AIX)
 	if (setsid() < 0) {
 #ifdef ultrix
 		/*
@@ -1341,6 +1210,7 @@ login_tty(t)
 #endif
 			fatalperror(net, "setsid()");
 	}
+#endif /* HAVE_SETSID */
 #endif
 # if defined(TIOCSCTTY) && !defined(__hpux)
 	if (ioctl(t, TIOCSCTTY, (char *)0) < 0)
@@ -1362,11 +1232,12 @@ login_tty(t)
 	 * setsid() call above may have set our pgrp, so clear
 	 * it out before opening the tty...
 	 */
-#  if defined(SOLARIS) || defined(__hpux) || defined(__sgi)
-	(void) setpgrp();
-#  else
-	(void) setpgrp(0, 0);
-#  endif
+#if defined HAVE_SETPGID
+	(void) setpgid(0, 0);
+#else
+	(void) setpgrp(0, 0); /* if setpgid isn't available, setpgrp
+				 probably takes arguments */
+#endif
 	close(open(line, O_RDWR));
 # endif
 	if (t != 0)
@@ -1394,10 +1265,7 @@ char *gen_id = "fe";
 
 /* ARGSUSED */
 	void
-startslave(host, autologin, autoname)
-	char *host;
-	int autologin;
-	char *autoname;
+startslave(char *host, int autologin, char *autoname)
 {
 	register int i;
 	char name[256];
@@ -1421,11 +1289,11 @@ startslave(host, autologin, autoname)
       {
 	char *tbuf =
 	  "\r\n*** Connection not encrypted! "
-	  "Communication may be eavesdropped.***\r\n";
+	  "Communication may be eavesdropped. ***\r\n";
 #ifdef ENCRYPTION
 	if (encrypt_output == 0 || decrypt_input == 0)
 #endif
-	  writenet(tbuf, strlen(tbuf));
+	  writenet((unsigned char*)tbuf, strlen(tbuf));
       }
 #ifndef	NEWINIT
 # ifdef	PARENT_DOES_UTMP
@@ -1472,7 +1340,7 @@ startslave(host, autologin, autoname)
 		utmp_sig_notify(pid);
 # endif	/* PARENT_DOES_UTMP */
 	} else {
-	  getptyslave(autologin);
+	  getptyslave();
 	  start_login(host, autologin, autoname);
 	  /*NOTREACHED*/
 	}
@@ -1515,7 +1383,7 @@ startslave(host, autologin, autoname)
 	for (i = 0; ; i++) {
 		char tbuf[128];
 		alarm(15);
-		n = read(pty, ptyip, BUFSIZ);
+		n = read(ourpty, ptyip, BUFSIZ);
 		if (i == 3 || n >= 0 || !gotalarm)
 			break;
 		gotalarm = 0;
@@ -1536,9 +1404,9 @@ char	*envinit[3];
 extern char **environ;
 
 	void
-init_env()
+init_env(void)
 {
-	extern char *getenv();
+	extern char *getenv(const char *);
 	char **envp;
 
 	envp = envinit;
@@ -1591,24 +1459,17 @@ int addarg(struct arg_val*, char*);
  * function will turn us into the login process.
  */
 
-void start_login(host, autologin, name)
-	char *host;
-	int autologin;
-	char *name;
+void start_login(char *host, int autologin, char *name)
 {
 	register char *cp;
 	struct arg_val argv;
-	extern char *getenv();
+	extern char *getenv(const char *);
 #ifdef	HAVE_UTMPX_H
 	char id_buf[3];
 	int ptynum;
 	register int pid = getpid();
 	struct utmpx utmpx;
 #endif
-#ifdef SOLARIS
-	char *term;
-	char termbuf[64];
-#endif
 
 #ifdef	HAVE_UTMPX_H
 	/*
@@ -1663,20 +1524,6 @@ void start_login(host, autologin, name)
 	{
 		addarg(&argv, "-h");
 		addarg(&argv, host);
-#ifdef	SOLARIS_LOGIN
-		/*
-		 * SVR4 version of -h takes TERM= as second arg, or -
-		 */
-		term = getenv("TERM");
-		if (term == NULL || term[0] == 0) {
-			term = "-";
-		} else {
-			strcpy(termbuf, "TERM=");
-			strncat(termbuf, term, sizeof(termbuf) - 6);
-			term = termbuf;
-		}
-		addarg(&argv, term);
-#endif
 	}
 #endif
 #if	!defined(NO_LOGIN_P)
@@ -1732,7 +1579,7 @@ void start_login(host, autologin, name)
 		 *	local-user\0remote-user\0term/speed\0
 		 */
 
-		if (pty > 2) {
+		if (ourpty > 2) {
 			register char *cp;
 			char speed[128];
 			int isecho, israw, xpty, len;
@@ -1754,7 +1601,7 @@ void start_login(host, autologin, name)
 			addarg(&argv, "-r");
 			addarg(&argv, LOGIN_HOST);
 
-			xpty = pty;
+			xpty = ourpty;
 # ifndef  STREAMSPTY
 			pty = 0;
 # else
@@ -1781,7 +1628,6 @@ void start_login(host, autologin, name)
 				tty_setecho(isecho);
 				tty_setraw(israw);
 				set_termbuf();
-#ifndef SOLARIS
 				if (!israw) {
 					/*
 					 * Write a newline to ensure
@@ -1790,9 +1636,8 @@ void start_login(host, autologin, name)
 					 */
 					write(xpty, "\n", 1);
 				}
-#endif
 			}
-			pty = xpty;
+			ourpty = xpty;
 		}
 #  else /* LOGIN_R */
 		addarg(&argv, name);
@@ -1822,19 +1667,9 @@ void start_login(host, autologin, name)
 		 */
 		unsetenv("USER");
 	}
-#ifdef	SOLARIS_LOGIN
-	else {
-		char **p;
-
-		addarg(&argv, "");	/* no login name */
-		for (p = environ; *p; p++) {
-			addarg(&argv, *p);
-		}
-	}
-#endif	/* SOLARIS */
 #if	defined(AUTHENTICATION) && defined(NO_LOGIN_F) && defined(LOGIN_R)
-	if (pty > 2)
-		close(pty);
+	if (ourpty > 2)
+		close(ourpty);
 #endif
 	closelog();
 	/*
@@ -1885,8 +1720,7 @@ int addarg(struct arg_val *argv, char *val)
  */
 	/* ARGSUSED */
 	void
-cleanup(sig)
-	int sig;
+cleanup(int sig)
 {
 #ifndef	PARENT_DOES_UTMP
 # if (BSD > 43) || defined(convex)
@@ -1967,7 +1801,7 @@ cleanup(sig)
 	setutent();	/* just to make sure */
 #  endif /* CRAY */
 	rmut(line);
-	close(pty);
+	close(ourpty);
 	(void) shutdown(net, 2);
 #  ifdef CRAY
 	if (t == 0)
