@@ -7,10 +7,10 @@
  * Original sunview version written by Dan Heller 1985 (not included here).
  */
 #ifdef HAVE_CONFIG_H
-#include "config.h"
-#include "protos.h"
+#include <config.h>
+#include <protos.h>
 #define KERBEROS
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/appl/xnlock/xnlock.c,v 1.12 1995/10/27 10:56:33 bg Exp $");
+RCSID("$Id: xnlock.c,v 1.18 1996/02/11 12:28:36 bg Exp $");
 #endif
 
 #include <stdio.h>
@@ -23,6 +23,7 @@ RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/appl
 #include <X11/Xos.h>
 #include <ctype.h>
 #include <pwd.h>
+#include <xnlock.h>
 
 #ifdef KERBEROS
 #include <krb.h>
@@ -33,6 +34,7 @@ char name[100];
 char realm[REALM_SZ + 1];
 #endif
 
+char *SPACE_STRING = "                                                      ";
 char STRING[] = "****************";
 #define MAX_PASSWD_LENGTH (sizeof(STRING))
 
@@ -53,6 +55,8 @@ char STRING[] = "****************";
 #define FRONT	020
 #define X_INCR 3
 #define Y_INCR 2
+#define CTRL 1
+#define NOCTRL 1
 
 XtAppContext	app;
 Display        *dpy;
@@ -243,6 +247,25 @@ ScreenSaver(int save)
 static void talk(int force_erase);
 static unsigned long look(void);
 
+static int
+zrefresh()
+{
+  switch (fork()) {
+  case -1:
+      fprintf(stderr, "Warning %s: Failed to fork zrefresh\n", ProgName);
+      return -1;
+  case 0:
+      /* Child */
+      execlp("zrefresh", "zrefresh", 0);
+      execl("/usr/athena/bin/zrefresh", "zrefresh", 0);
+      fprintf(stderr, "Warning %s: Failed to exec zrefresh\n", ProgName);
+      return -1;
+  default:
+      /* Parent */
+      break;
+  }
+  return 0;
+}
 
 static void
 leave(void)
@@ -254,6 +277,7 @@ leave(void)
     XUngrabKeyboard(dpy, CurrentTime);
 #endif
     ScreenSaver(0);
+    zrefresh();
     exit(0);
 }
 
@@ -284,7 +308,7 @@ walk(int dir)
 	/* note that maybe neither UP nor DOWN is set! */
 	if (dir & UP && y > Y_INCR)
 	    y -= Y_INCR;
-	else if (dir & DOWN && y < Height - 64)
+	else if (dir & DOWN && y < (int)Height - 64)
 	    y += Y_INCR;
     }
     /* Explicit up/down movement only (no left/right) */
@@ -360,28 +384,28 @@ move(XtPointer _p, XtIntervalId *_id)
 		    if (x - X_INCR*length >= 5)
 			dir = LEFT;
 		case 1:
-		    if (x + X_INCR*length <= Width - 70)
+		    if (x + X_INCR*length <= (int)Width - 70)
 			dir = RIGHT;
 		case 2:
 		    if (y - (Y_INCR*length) >= 5)
 			dir = UP, interval = 40;
 		case 3:
-		    if (y + Y_INCR*length <= Height - 70)
+		    if (y + Y_INCR*length <= (int)Height - 70)
 			dir = DOWN, interval = 20;
 		case 4:
 		    if (x - X_INCR*length >= 5 && y - (Y_INCR*length) >= 5)
 			dir = (LEFT|UP);
 		case 5:
-		    if (x + X_INCR * length <= Width - 70 &&
+		    if (x + X_INCR * length <= (int)Width - 70 &&
 			y-Y_INCR * length >= 5)
 			dir = (RIGHT|UP);
 		case 6:
 		    if (x - X_INCR * length >= 5 &&
-			y + Y_INCR * length <= Height - 70)
+			y + Y_INCR * length <= (int)Height - 70)
 			dir = (LEFT|DOWN);
 		case 7:
-		    if (x + X_INCR*length <= Width - 70 &&
-			y + Y_INCR*length <= Height - 70)
+		    if (x + X_INCR*length <= (int)Width - 70 &&
+			y + Y_INCR*length <= (int)Height - 70)
 			dir = (RIGHT|DOWN);
 	    }
 	} while (!dir);
@@ -474,7 +498,7 @@ Visibility(Widget w, XtPointer client_data, XEvent *event, Boolean *_b)
 static void
 countdown(XtPointer _t, XtIntervalId *_d)
 {
-    int *timeout = _t;
+    int *timeout = (int *)_t;
     char buf[16];
     time_t seconds;
 
@@ -491,14 +515,15 @@ countdown(XtPointer _t, XtIntervalId *_d)
     seconds = time(0) - locked_at;
     if (seconds >= 3600)
       (void) sprintf(buf, "Locked for %d:%02d:%02d    ",
-                   seconds/3600, seconds/60%60, seconds%60);
+                   (int)seconds/3600, (int)seconds/60%60, (int)seconds%60);
     else
       (void) sprintf(buf, "Locked for %2d:%02d    ",
-                   seconds/60, seconds%60);
+		     (int)seconds/60, (int)seconds%60);
       
     XDrawImageString(dpy, XtWindow(widget), gc,
 	time_x, time_y, buf, strlen(buf));
     XtAppAddTimeOut(app, 1000L, countdown, timeout);
+    return;
 }
 
 static void
@@ -507,6 +532,7 @@ GetPasswd(Widget w, XEvent *_event, String *_s, Cardinal *_n)
     XKeyEvent *event = (XKeyEvent *)_event;
     static char passwd[MAX_PASSWD_LENGTH];
     static int cnt;
+    static int is_ctrl = NOCTRL;
     char c;
     KeySym keysym;
 
@@ -523,11 +549,22 @@ GetPasswd(Widget w, XEvent *_event, String *_s, Cardinal *_n)
 	post_prompt_box(XtWindow(w));
 	cnt = 0;
 	time_left = 30;
-	countdown(&time_left, 0);
+	countdown((XtPointer)&time_left, 0);
 	return;
     }
+    if (event->type == KeyRelease) {
+      keysym = XLookupKeysym(event, 0);
+      if (keysym == XK_Control_L || keysym == XK_Control_R) {
+	is_ctrl = NOCTRL;
+      }
+    }
     if (event->type != KeyPress)
 	return;
+    keysym = XLookupKeysym(event, 0);
+    if (keysym == XK_Control_L || keysym == XK_Control_R) {
+      is_ctrl = CTRL;
+      return;
+    }
     if (!XLookupString(event, &c, 1, &keysym, 0))
 	return;
     if (keysym == XK_Return || keysym == XK_Linefeed) {
@@ -547,7 +584,14 @@ GetPasswd(Widget w, XEvent *_event, String *_s, Cardinal *_n)
 	    && geteuid() != 0
 	    && (time(0) - locked_at) > ALLOW_LOGOUT
 	    && strncmp(passwd, LOGOUT_PASSWD, sizeof(LOGOUT_PASSWD)) == 0)
-	    kill(-1, SIGHUP);
+	    {
+		signal(SIGHUP, SIG_IGN);
+		kill(-1, SIGHUP);
+		sleep(5);
+		/* If the X-server shut down then so will we, else
+		 * continue */
+		signal(SIGHUP, SIG_DFL);
+	    }
 #ifdef KERBEROS
 	/*
 	 * Try to verify as user with kerberos.
@@ -565,7 +609,8 @@ GetPasswd(Widget w, XEvent *_event, String *_s, Cardinal *_n)
 			int code;
 			if (k_hasafs())
 			    {
-				if ((code = k_afsklog(NULL, realm)) != KSUCCESS)
+				if ((code = k_afsklog(NULL, realm)) != KSUCCESS
+				    && code != KDC_PR_UNKNOWN)
 				    fprintf(stderr,
 					    "%s: Warning %s\n",
 					    ProgName,
@@ -591,8 +636,17 @@ GetPasswd(Widget w, XEvent *_event, String *_s, Cardinal *_n)
     if (keysym == XK_BackSpace || keysym == XK_Delete || keysym == XK_Left) {
 	if (cnt)
 	    passwd[cnt--] = ' ';
+    } else if (keysym == XK_u && is_ctrl == 1) {
+      while (cnt) {
+	passwd[cnt--] = ' ';
+	XDrawImageString(dpy, XtWindow(w), gc,
+		    prompt_x, prompt_y, STRING, cnt);
+	XDrawImageString(dpy, XtWindow(w), gc,
+			 prompt_x + XTextWidth(font, STRING, cnt),
+			 prompt_y, SPACE_STRING, MAX_PASSWD_LENGTH - cnt + 1);
+      }
     } else if (isprint(c))
-	if (cnt >= MAX_PASSWD_LENGTH)
+	if ((cnt + 1) >= MAX_PASSWD_LENGTH)
 	    XBell(dpy, 50);
 	else
 	    passwd[cnt++] = c;
@@ -602,7 +656,7 @@ GetPasswd(Widget w, XEvent *_event, String *_s, Cardinal *_n)
 	prompt_x, prompt_y, STRING, cnt);
     XDrawImageString(dpy, XtWindow(w), gc,
 	prompt_x + XTextWidth(font, STRING, cnt),
-	prompt_y, "           ", 11-cnt);
+	prompt_y, SPACE_STRING, MAX_PASSWD_LENGTH - cnt + 1);
 }
 
 #include "nose.0.left"
@@ -621,7 +675,7 @@ init_images(void)
 	&left0, &left1, &right0, &right1,
 	&left_front, &right_front, &front, &down 
     };
-    static char *bits[] = {
+    static unsigned char *bits[] = {
 	nose_0_left_bits, nose_1_left_bits, nose_0_right_bits,
 	nose_1_right_bits, nose_left_front_bits, nose_right_front_bits,
 	nose_front_bits, nose_down_bits
@@ -660,7 +714,9 @@ talk(int force_erase)
 	}
 	talking = 0;
 	if (!force_erase)
-	    timeout_id = XtAppAddTimeOut(app, 40L, move, NULL);
+	    timeout_id = XtAppAddTimeOut(app, 40L,
+					 (XtTimerCallbackProc)move,
+					 NULL);
 	return;
     }
     talking = 1;
@@ -681,7 +737,7 @@ talk(int force_erase)
 	/* give us a nice 5 pixel margin */
 	if (X < 5)
 	    X = 5;
-	else if (X + w + 15 > Width + 5)
+	else if (X + w + 15 > (int)Width + 5)
 	    X = Width - w - 5;
 	if (Y < 5)
 	    Y = y + 64 + 5 + font_height(font);
@@ -719,7 +775,7 @@ talk(int force_erase)
 	s_rect.x = 5;
     else
 	if ((s_rect.x = x+32-(s_rect.width+15)/2)
-					 + s_rect.width+15 > Width-5)
+					 + s_rect.width+15 > (int)Width-5)
 	    s_rect.x = Width - 15 - s_rect.width;
     if (y - s_rect.height - 10 < 5)
 	s_rect.y = y + 64 + 5;
@@ -783,6 +839,10 @@ main (int argc, char **argv)
     XGCValues gcvalues;
     char **list;
 
+    srand(getpid());
+    for (i = 0; i < (sizeof(STRING)-2); i++)
+	STRING[i] = ((unsigned long)rand() % ('~' - ' ')) + ' ';
+
     locked_at = time(0);
 
     if ((ProgName = rindex(*argv, '/')) != 0)
