@@ -36,7 +36,7 @@ or implied warranty.
 
 #include "kadm_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/kadmin/kadmin.c,v 1.8 1995/10/27 10:56:39 bg Exp $");
+RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/kadmin/kadmin.c,v 1.11 1996/02/04 09:23:50 assar Exp $");
 
 #define BAD_PW 1
 #define GOOD_PW 0
@@ -117,7 +117,7 @@ static void
 get_expdate(Kadm_vals *vals)
 {
     char buff[BUFSIZ];
-    time_t when;
+    time_t when=0;
     struct tm edate;
 
     bzero(&edate, sizeof(edate));
@@ -451,6 +451,100 @@ change_password(int argc, char **argv)
     return;
 }
 
+static
+int
+getkey(unsigned char *k)
+{
+    int i, c;
+    for (i = 0; i < 8; i++)
+	{
+	    c = getchar();
+	    if (c == EOF)
+		return 0;
+	    else if (c == '\\')
+		{
+		    int oct = -1;
+		    scanf("%03o", &oct);
+		    if (oct < 0 || oct > 255)
+			return 0;
+		    k[i] = oct;
+		}
+	    else if (!isalpha(c))
+		return 0;
+	    else
+		k[i] = c;
+	}
+    c = getchar();
+    if (c != '\n')
+	return 0;
+    return 1;			/* Success */
+}
+
+static
+void
+printkey(unsigned char *tkey)
+{
+    int j;
+    for(j = 0; j < 8; j++)
+	if(tkey[j] != '\\' && isalpha(tkey[j]) != 0)
+	    printf("%c", tkey[j]);
+	else
+	    printf("\\%03o",(unsigned char)tkey[j]);
+    printf("\n");
+}
+
+void 
+change_key(int argc, char **argv)
+{
+    Kadm_vals old, new;
+    unsigned char newkey[8];
+    int status;
+
+    if (argc != 2) {
+	printf("Usage: change_key principal-name\n");
+	return;
+    }
+
+    if (setvals(&old, argv[1]) != KADM_SUCCESS)
+	return;
+
+    new = old;
+
+    SET_FIELD(KADM_DESKEY,new.fields);
+
+    if (princ_exists(old.name, old.instance, krbrlm) != PE_NO) {
+	/* get the admin's password */
+        if (get_admin_password() != GOOD_PW)
+	    return;
+
+	/* get the new password */
+	(void) printf("New DES key for %s: ", argv[1]);
+	
+	if (getkey(newkey)) {
+	    bcopy((char *) newkey,(char *)&new.key_low,4);
+	    bcopy((char *)(((int32_t *) newkey) + 1), (char *)&new.key_high,4);
+	    printf("Entered key for %s: ", argv[1]);
+	    printkey(newkey);
+	    bzero(newkey, sizeof(newkey));
+
+	    status = kadm_mod(&old, &new);
+	    if (status == KADM_SUCCESS) {
+		printf("Key changed for %s.\n", argv[1]);
+	    } else {
+		printf("kadmin: %s\nwhile changing key for %s",
+		       error_message(status), argv[1]);
+	    }
+	} else
+	    printf("Error reading key; key unchanged\n");
+	bzero((char *)&new, sizeof(new));
+	if (!multiple)
+	    clean_up();
+    }
+    else 
+	printf("kadmin: Principal does not exist.\n");
+    return;
+}
+
 /*ARGSUSED*/
 void 
 change_admin_password(int argc, char **argv)
@@ -683,6 +777,12 @@ help(int argc, char **argv)
 	printf("appropriate comparison). Note: No minimum password");
 	printf(" length restrictions apply, but\n");
 	printf("longer passwords are more secure.\n");
+    } else if (!strcmp(argv[1], "change_key") ||
+	       !strcmp(argv[1], "ckey")) {
+	printf("Usage: change_key principal-name.\n");
+	printf("\n");
+	printf("Descripton:\n");
+	printf("\tAs change_password but input password as binary key\n");
     } else if (!strcmp(argv[1], "change_admin_password") ||
 	       !strcmp(argv[1], "cap")) {
 	printf("Usage: change_admin_password.\n");
