@@ -30,7 +30,7 @@ or implied warranty.
 
 #include "kadm_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/kadmin/kpasswd.c,v 1.4 1995/10/27 10:56:40 bg Exp $");
+RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/kadmin/kpasswd.c,v 1.8 1996/02/07 19:38:28 bg Exp $");
 
 #ifdef NOENCRYPTION
 #define read_long_pw_string placebo_read_pw_string
@@ -71,19 +71,29 @@ char *FascistCheck(char *pw, char *dictpath)
 static char *
 check_pw (char *pword)
 {
-     if (strlen(pword) == 0)
-	  return "Null passwords are not allowed";
-     return FascistCheck (pword, DICTPATH);
+    if (strlen(pword) == 0)
+	return "Null passwords are not allowed - Please enter a longer password.";
+    
+    if (strlen(pword) < MIN_KPW_LEN)
+	return "Password is to short - Please enter a longer password.";
+    
+    /* Don't allow all lower case passwords regardless of length */
+    {
+	char *t;
+	for (t = pword; *t && islower(*t); t++)
+	    ;
+	if (*t == 0)
+	    return "Please don't use an all-lower case password.\n\tUnusual capitalization, delimiter characters or digits are suggested.";
+    }
+
+    return FascistCheck (pword, DICTPATH);
 }
 
 static void
-get_pw_new_key(des_cblock (*new_key), char *name, char *inst,
-	       char *realm,
-	       int print_realm)
-                  		/* True if realm was give on cmdline */
+get_pw_new_pwd(char *pword, int pwlen, char *name, char *inst,
+	       char *realm, int print_realm)
 {
     char ppromp[40+ANAME_SZ+INST_SZ+REALM_SZ]; /* for the password prompt */
-    char pword[MAX_KPW_LEN];	               /* storage for the password */
     char npromp[40+ANAME_SZ+INST_SZ+REALM_SZ]; /* for the password prompt */
     
     char local_realm[REALM_SZ];
@@ -102,7 +112,7 @@ get_pw_new_key(des_cblock (*new_key), char *name, char *inst,
     (void) sprintf(ppromp,"Old password for %s%s%s%s%s:",
 		   name, *inst ? "." : "", inst,
 		   print_realm ? "@" : "", print_realm ? realm : "");
-    if (read_long_pw_string(pword, sizeof(pword)-1, ppromp, 0)) {
+    if (read_long_pw_string(pword, pwlen-1, ppromp, 0)) {
 	fprintf(stderr, "Error reading old password.\n");
 	exit(1);
     }
@@ -118,25 +128,30 @@ get_pw_new_key(des_cblock (*new_key), char *name, char *inst,
 	    exit(1);
 	}
     }
-    bzero(pword, sizeof(pword));
+    bzero(pword, pwlen);
     do {
+	char verify[MAX_KPW_LEN];
 	(void) sprintf(npromp,"New Password for %s%s%s%s%s:",
 		       name, *inst ? "." : "", inst,
 		       print_realm ? "@" : "", print_realm ? realm : "");
-	if (read_long_pw_string(pword, sizeof(pword)-1, npromp, 1))
+	if (read_long_pw_string(pword, pwlen-1, npromp, 0))
 	    go_home("Error reading new password, password unchanged.\n",0);
 	expl = check_pw (pword);
-	if (expl)
-	    printf("Bad password: %s\n", expl);
+	if (expl) {
+	    printf("\n\t%s\n\n", expl);
+	    continue;
+	}
+	/* Now we got an ok password, verify it. */
+	(void) sprintf(npromp,"Verifying New Password for %s%s%s%s%s:",
+		       name, *inst ? "." : "", inst,
+		       print_realm ? "@" : "", print_realm ? realm : "");
+	if (read_long_pw_string(verify, MAX_KPW_LEN-1, npromp, 0))
+	    go_home("Error reading new password, password unchanged.\n",0);
+	if (strcmp(pword, verify) != 0) {
+	    printf("Verify failure - try again\n");
+	    expl = "";		/* continue */
+	}
     } while (expl);
-    
-#ifdef NOENCRYPTION
-    bzero((char *) new_key, sizeof(des_cblock));
-    new_key[0] = (unsigned char) 1;
-#else
-   (void) des_string_to_key(pword, new_key);
-#endif
-    bzero(pword, sizeof(pword));
 }
 
 int
@@ -152,7 +167,7 @@ main(int argc, char **argv)
     int use_default = 1;	/* True if we should use default name */
     struct passwd *pw;
     int status;			/* return code */
-    des_cblock new_key;
+    char pword[MAX_KPW_LEN];
     int c;
     char tktstring[MaxPathLen];
     
@@ -240,22 +255,34 @@ main(int argc, char **argv)
     (void) sprintf(tktstring, "/tmp/tkt_cpw_%d",(int)getpid());
     krb_set_tkt_string(tktstring);
     
-    get_pw_new_key(&new_key, name, inst, realm, realm_given);
+    get_pw_new_pwd(pword, sizeof(pword), name, inst, realm, realm_given);
     
-    if ((status = kadm_init_link("changepw", KRB_MASTER, realm)) 
-	!= KADM_SUCCESS) 
-	com_err(argv[0], status, "while initializing");
-    else if ((status = kadm_change_pw(new_key)) != KADM_SUCCESS)
+    status = kadm_init_link("changepw", KRB_MASTER, realm);
+    if (status != KADM_SUCCESS) 
+      com_err(argv[0], status, "while initializing");
+    else {
+      des_cblock newkey;
+      char *pw_msg; /* message from server */
+
+      des_string_to_key(pword, &newkey);
+      status = kadm_change_pw_plain((unsigned char*)&newkey, pword, &pw_msg);
+      bzero(newkey, sizeof(newkey));
+      
+      if (status == KADM_INSECURE_PW)
+	fprintf(stderr, "%s: Insecure password: %s\n", argv[0], pw_msg);
+      else if (status != KADM_SUCCESS)
 	com_err(argv[0], status, " attempting to change password.");
+    }
+    bzero(pword, sizeof(pword));
 
     if (status != KADM_SUCCESS)
-	fprintf(stderr,"Password NOT changed.\n");
+      fprintf(stderr,"Password NOT changed.\n");
     else
-	printf("Password changed.\n");
+      printf("Password changed.\n");
 
     (void) dest_tkt();
     if (status)
-	exit(2);
+      exit(2);
     else 
-	exit(0);
+      exit(0);
 }
