@@ -27,37 +27,6 @@ or implied warranty.
 
   */
 
-/*
- *
- * $Log: ksrvutil.c,v $
- * Revision 1.10  1995/10/23 11:56:20  d91-jda
- * *** empty log message ***
- *
- * Revision 1.9  1995/09/12  12:53:12  bg
- * Remove instance part from name when using the form foo.bar.
- * Also copy instance part to sinst.
- *
- * Revision 1.8  1995/08/18 18:44:14  bg
- * Version numbers are never 0.
- *
- * Revision 1.7  1995/08/18 18:38:35  bg
- * If there is a . in name don't ask for instance.
- *
- * Revision 1.6  1995/08/10 13:59:11  bg
- * KRB_REALM macro removed.
- *
- * Revision 1.5  1995/07/11 17:19:27  bg
- * Use new random generator.
- *
- * Revision 1.4  1995/07/11 17:15:41  bg
- * Hack for new option get.
- *
- * Revision 1.3  1995/06/07 13:14:06  bg
- * Fix copyright.
- *
- *
- */
-
 /*
  * list and update contents of srvtab files
  */
@@ -67,14 +36,6 @@ or implied warranty.
  * list and update the contents of srvtab files
  */
 
-#ifndef FALSE
-#define FALSE 0
-#endif
-
-#ifndef TRUE
-#define TRUE 1
-#endif
-
 #include "kadm_locl.h"
 #include "ksrvutil.h"
 
@@ -98,6 +59,8 @@ char u_name[ANAME_SZ];
 char u_inst[INST_SZ];
 char u_realm[REALM_SZ];
 
+int destroyp = FALSE;		/* Should the ticket file be destroyed? */
+
 static void get_default_principal(void)
 {
   struct passwd *pw;
@@ -219,12 +182,13 @@ leave(char *str, int x)
 {
     if (str)
 	(void) fprintf(stderr, "%s\n", str);
-    (void) dest_tkt();
+    if (destroyp)
+	 (void) dest_tkt();
     exit(x);
 }
 
 void
-safe_read_stdin(char *prompt, char *buf, int size)
+safe_read_stdin(char *prompt, char *buf, size_t size)
 {
     (void) printf(prompt);
     (void) fflush(stdout);
@@ -240,7 +204,7 @@ safe_read_stdin(char *prompt, char *buf, int size)
   
 
 void
-safe_write(char *filename, int fd, char *buf, int len)
+safe_write(char *filename, int fd, void *buf, size_t len)
 {
     if (write(fd, buf, len) != len) {
 	(void) fprintf(stderr, "%s: Failure writing to %s: %s\n", progname,
@@ -355,6 +319,7 @@ static void
 usage(void)
 {
     (void) fprintf(stderr, "Usage: ksrvutil [-f keyfile] [-i] [-k] ");
+    (void) fprintf(stderr, "[-p principal] [-r realm] ");
     (void) fprintf(stderr, "{list | change | add | get}\n");
     (void) fprintf(stderr, "   -i causes the program to ask for ");
     (void) fprintf(stderr, "confirmation before changing keys.\n");
@@ -381,6 +346,7 @@ main(int argc, char **argv)
     int work_keyfile_fd = -1;	/* Initialize so that */
     int backup_keyfile_fd = -1;	/* compiler doesn't complain */
     char local_realm[REALM_SZ];	/* local kerberos realm */
+    int c;
     int i;
     int interactive = FALSE;
     int list = FALSE;
@@ -405,61 +371,76 @@ main(int argc, char **argv)
     
     get_default_principal();
 
-    (void) sprintf(change_tkt, "/tmp/tkt_ksrvutil.%d", (int)getpid());
-    krb_set_tkt_string(change_tkt);
-
     progname=argv[0];
 
     /* This is used only as a default for adding keys */
     if (krb_get_lrealm(local_realm, 1) != KSUCCESS)
 	(void) strcpy(local_realm, "NO.DEFAULT.REALM");
     
-    for (i = 1; i < argc; i++) {
-	if (strcmp(argv[i], "-i") == 0) 
-	    interactive++;
-	else if (strcmp(argv[i], "-k") == 0) 
-	    key++;
-	else if (strcmp(argv[i], "list") == 0) {
-	    if (arg_entered)
-		usage();
-	    else {
-		arg_entered++;
-		list++;
-	    }
-	}
-	else if (strcmp(argv[i], "change") == 0) {
-	    if (arg_entered)
-		usage();
-	    else {
-		arg_entered++;
-		change++;
-	    }
-	}
-	else if (strcmp(argv[i], "add") == 0) {
-	    if (arg_entered)
-		usage();
-	    else {
-		arg_entered++;
-		add++;
-	    }
-	}
-	else if (strcmp(argv[i], "get") == 0) {
-	    if (arg_entered)
-		usage();
-	    else {
-		arg_entered++;
-		get++;
-	    }
-	}
-	else if (strcmp(argv[i], "-f") == 0) {
-	    if (++i == argc)
-		usage();
-	    else
-		(void) strcpy(keyfile, argv[i]);
-	}
-	else
-	    usage();
+    while((c = getopt(argc, argv, "ikf:p:r:")) != EOF) {
+	 switch (c) {
+	      case 'i':
+	      interactive++;
+	      break;
+	      case 'k':
+	      key++;
+	      break;
+	      case 'f':
+	      (void) strcpy(keyfile, optarg);
+	      break;
+	      case 'p':
+	      if((status = kname_parse (u_name, u_inst, u_realm, optarg)) !=
+		 KSUCCESS) {
+		   fprintf(stderr, "%s: principal %s: %s\n", progname,
+			   optarg,
+			   krb_err_txt[status]);
+		   return 1;
+	      }
+	      break;
+	      case 'r':
+	      (void) strcpy(u_realm, optarg);
+	      break;
+	      case '?':
+	      usage();
+	 }
     }
+    if (optind >= argc)
+	 usage();
+    if (strcmp(argv[optind], "list") == 0) {
+	 if (arg_entered)
+	      usage();
+	 else {
+	      arg_entered++;
+	      list++;
+	 }
+    }
+    else if (strcmp(argv[optind], "change") == 0) {
+	 if (arg_entered)
+	      usage();
+	 else {
+	      arg_entered++;
+	      change++;
+	 }
+    }
+    else if (strcmp(argv[optind], "add") == 0) {
+	 if (arg_entered)
+	      usage();
+	 else {
+	      arg_entered++;
+	      add++;
+	 }
+    }
+    else if (strcmp(argv[optind], "get") == 0) {
+	 if (arg_entered)
+	      usage();
+	 else {
+	      arg_entered++;
+	      get++;
+	 }
+    }
+    else
+	 usage();
+    ++optind;
     
     if (!arg_entered)
 	usage();
@@ -499,7 +480,7 @@ main(int argc, char **argv)
 	    exit(1);
 	}
     }
-    else if (add || get) {
+    else if (add) {
 	if ((work_keyfile_fd =
 	     open(work_keyfile, O_APPEND | O_WRONLY, SRVTAB_MODE)) < 0) {
 	    (void) fprintf(stderr, "%s: Unable to open %s for append: %s\n",
@@ -507,6 +488,15 @@ main(int argc, char **argv)
 	    exit(1);
 	}
     }
+    else if (get) {
+	if ((work_keyfile_fd =
+	     open(work_keyfile, O_RDWR | O_CREAT, SRVTAB_MODE)) < 0) {
+	     (void) fprintf(stderr, "%s: Unable to open %s for writing: %s\n",
+			    progname, work_keyfile,
+			    k_strerror(errno));
+	     exit(1);
+	}
+    }
     
     if (change || list) {
 	while ((getst(backup_keyfile_fd, sname, SNAME_SZ) > 0) &&
@@ -535,6 +525,11 @@ main(int argc, char **argv)
 		(void) printf("\n");
 	    }
 	    else if (change) {
+		(void) sprintf(change_tkt, "/tmp/tkt_ksrvutil.%d",
+			       (int)getpid());
+		krb_set_tkt_string(change_tkt);
+		destroyp = TRUE;
+
 		(void) printf("\nPrincipal: ");
 		print_name(sname, sinst, srealm);
 		(void) printf("; version %d\n", key_vno);
@@ -648,7 +643,8 @@ main(int argc, char **argv)
 	} while (yn("Would you like to add another key?"));
     }
     else if (get) {
-        ksrvutil_get();
+        ksrvutil_get(work_keyfile_fd, work_keyfile,
+		     argc - optind, argv + optind);
     }
 
     if (change || list) 
