@@ -13,21 +13,19 @@
 
 #include "kuser_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/kuser/kdestroy.c,v 1.2 1995/10/27 10:56:43 bg Exp $");
+RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/kuser/kdestroy.c,v 1.3 1996/02/07 10:48:19 d91-jda Exp $");
 
 static char *pname;
 
 static void
-usage()
+usage(void)
 {
     fprintf(stderr, "Usage: %s [-f] [-q]\n", pname);
     exit(1);
 }
 
 int
-main(argc, argv)
-    int argc;
-    char   *argv[];
+main(int argc, char **argv)
 {
     int     fflag=0, qflag=0, k_errno;
     register char *cp;
