@@ -10,13 +10,12 @@
 
 #include "kuser_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/kuser/klist.c,v 1.2 1995/10/27 10:56:45 bg Exp $");
+RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/kuser/klist.c,v 1.5 1996/02/07 11:04:40 d91-jda Exp $");
 
 char   *whoami;			/* What was I invoked as?? */
 
 static char *
-short_date(dp)
-    time_t *dp;
+short_date(time_t *dp)
 {
     register char *cp;
 
@@ -27,9 +26,7 @@ short_date(dp)
 }
 
 static void
-display_tktfile(file, tgt_test, long_form)
-char *file;
-int tgt_test, long_form;
+display_tktfile(char *file, int tgt_test, int long_form)
 {
     char    pname[ANAME_SZ];
     char    pinst[INST_SZ];
@@ -128,9 +125,12 @@ int tgt_test, long_form;
 	        (void) strcpy(buf2, ">>> Expired <<< ");
 	    printf("%s  %s  ", buf1, buf2);
 	}
-	printf("%s%s%s%s%s\n",
+	printf("%s%s%s%s%s",
 	       c.service, (c.instance[0] ? "." : ""), c.instance,
-	       (c.realm[0] ? "@" : ""), c.realm);
+	       (c.realm[0] ? "@" : ""), c.realm); 
+	if(long_form)
+	  printf(" (%d)", c.kvno);
+	printf("\n");
     }
     if (tgt_test)
 	exit(1);			/* no tgt found */
@@ -153,10 +153,7 @@ int tgt_test, long_form;
  */
 
 static int
-ok_getst(fd, s, n)
-    int fd;
-    register char *s;
-    int n;
+ok_getst(int fd, char *s, int n)
 {
     register count = n;
     int err;
@@ -170,8 +167,7 @@ ok_getst(fd, s, n)
 }
 
 static void
-display_srvtab(file)
-char *file;
+display_srvtab(char *file)
 {
     int stab;
     char serv[SNAME_SZ];
@@ -211,18 +207,16 @@ char *file;
 }
 
 static void
-usage()
+usage(void)
 {
     fprintf(stderr,
-        "Usage: %s [ -s | -t ] [ -file filename ] [ -srvtab ]\n", whoami);
+        "Usage: %s [ -s | -t ] [ -f filename ] [ -srvtab ]\n", whoami);
     exit(1);
 }
 
 /* ARGSUSED */
 int
-main(argc, argv)
-    int     argc;
-    char  **argv;
+main(int argc, char **argv)
 {
     int     long_form = 1;
     int     tgt_test = 0;
@@ -245,7 +239,7 @@ main(argc, argv)
 	if (!strcmp(*argv, "-l")) {	/* now default */
 	    continue;
 	}
-	if (!strcmp(*argv, "-file")) {
+	if (!strncmp(*argv, "-f", 2)) {
 	    if (*(++argv)) {
 		tkt_file = *argv;
 		continue;
