@@ -34,7 +34,7 @@ or implied warranty.
 
 #include "kadm_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/kadm/kadm_cli_wrap.c,v 1.4 1995/10/24 14:03:18 bg Exp $");
+RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/kadm/kadm_cli_wrap.c,v 1.9 1996/02/07 10:48:31 d91-jda Exp $");
 
 #ifndef NULL
 #define NULL 0
@@ -75,6 +75,7 @@ kadm_cli_disconn(void)
  * data about the connection between the server and client, the services 
  * used, the locations and other fun things 
  */
+
 int
 kadm_init_link(char *n, char *i, char *r)
 {
@@ -95,14 +96,16 @@ kadm_init_link(char *n, char *i, char *r)
 	if ((hop = gethostbyname(adm_hostname)) == NULL)
 		return KADM_UNK_HOST;  /* couldnt find the admin servers
 				        * address */
-	if ((sep = getservbyname(KADM_SNAME, "tcp")) == NULL)
-		return KADM_NO_SERV;   /* couldnt find the admin service */
 	bzero((char *) &client_parm.admin_addr,
 	      sizeof(client_parm.admin_addr));
+	if ((sep = getservbyname(KADM_SNAME, "tcp")) == NULL){
+	  fprintf(stderr, "Warning: %s/tcp service not found.\n", KADM_SNAME);
+	  client_parm.admin_addr.sin_port = htons(751);
+	}else
+	  client_parm.admin_addr.sin_port = sep->s_port;
 	client_parm.admin_addr.sin_family = hop->h_addrtype;
 	bcopy((char *) hop->h_addr, (char *) &client_parm.admin_addr.sin_addr,
 	      hop->h_length);
-	client_parm.admin_addr.sin_port = sep->s_port;
 
 	return KADM_SUCCESS;
 }				       /* procedure kadm_init_link */
@@ -337,23 +340,23 @@ kadm_cli_send(u_char *st_dat, int st_siz, u_char **ret_dat, int *ret_siz)
 	return retdat;
 }
 
-/*
- * kadm_change_pw
- * recieves    : key 
+
+
+/* 
+ * kadm_change_pw_plain
+ *
+ * see kadm_change_pw
  *
- * Replaces the password (i.e. des key) of the caller with that specified in
- * key. Returns no actual data from the master server, since this is called
- * by a user 
  */
-int
-kadm_change_pw(unsigned char *newkey)
-                  		       /* The DES form of the users key */
+int kadm_change_pw_plain(unsigned char *newkey, char *password, char **pw_msg)
 {
 	int stsize, retc;	       /* stream size and return code */
 	u_char *send_st;	       /* send stream */
+	u_int32_t keytmp;
 	u_char *ret_st;
 	int ret_sz;
-	u_int32_t keytmp;
+	int status;
+	static char msg[128];
 
 	if ((retc = kadm_cli_conn()) != KADM_SUCCESS)
 	    return(retc);
@@ -373,15 +376,41 @@ kadm_change_pw(unsigned char *newkey)
 	keytmp=ntohl(keytmp);
 	stsize += vts_long(keytmp, &send_st, stsize);
 	
+	if(password && *password)
+	  stsize += vts_string(password, &send_st, stsize);
+
 	retc = kadm_cli_send(send_st, stsize, &ret_st, &ret_sz);
 	free((char *)send_st);
-	if (retc == KADM_SUCCESS) {
-	    free((char *)ret_st);
+	
+	if(retc != KADM_SUCCESS){
+	  status = stv_string(ret_st, msg, 0, sizeof(msg), ret_sz);
+	  if(status<0)
+	    msg[0]=0;
+	  *pw_msg=msg;
 	}
+
+	if (ret_st)
+	    free((char *)ret_st);
+	
 	kadm_cli_disconn();
 	return(retc);
 }
 
+/*
+ * kadm_change_pw
+ * recieves    : key 
+ *
+ * Replaces the password (i.e. des key) of the caller with that specified in
+ * key. Returns no actual data from the master server, since this is called
+ * by a user 
+ */
+
+int kadm_change_pw(unsigned char *newkey)
+{
+  char *pw_msg;
+  return kadm_change_pw_plain(newkey, "", &pw_msg);
+}
+
 /*
  * kadm_add
  * 	receives    : vals
