@@ -26,7 +26,7 @@ or implied warranty.
 
 #include "kdb_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/kdb/krb_dbm.c,v 1.7 1995/10/24 15:42:19 bg Exp $");
+RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/kdb/krb_dbm.c,v 1.9 1996/02/07 10:48:36 d91-jda Exp $");
 
 #ifdef NDBM
 #include <ndbm.h>
@@ -170,25 +170,12 @@ encode_princ_contents(contents, principal)
     contents->dptr = (char *) principal;
 }
 
-static void
-decode_princ_contents __P((datum *contents, Principal *principal));
-
-static void
-decode_princ_contents(contents, principal)
-     datum *contents;
-     Principal *principal;
+static void decode_princ_contents (datum *contents, Principal *principal)
 {
     bcopy(contents->dptr, (char *) principal, sizeof(*principal));
 }
 
-static void
-encode_princ_key __P((datum *key, char *name, char *instance));
-
-static void
-encode_princ_key(key, name, instance)
-     datum *key;
-     char *name;
-     char *instance;
+static void encode_princ_key (datum *key, char *name, char *instance)
 {
     static char keystring[ANAME_SZ + INST_SZ];
 
@@ -570,16 +557,8 @@ kerb_db_rename(from, to)
  * found , and whether there were more than requested. 
  */
 
-int
-kerb_db_get_principal __P((char *name, char *inst, Principal *principal, unsigned int max, int *more));
-
-int
-kerb_db_get_principal(name, inst, principal, max, more)
-     char *name;		/* could have wild card */
-     char *inst;		/* could have wild card */
-     Principal *principal;	/* max number of name structs to return */
-     unsigned int max;		/* where there more than 'max' tuples? */
-     int *more;
+int kerb_db_get_principal (char *name, char *inst, Principal *principal, 
+			   unsigned int max, int *more)
 {
     int     found = 0, code;
     int     wildp, wildi;
@@ -830,12 +809,7 @@ kerb_db_get_dba(dba_name, dba_inst, dba, max, more)
 }
 
 int
-kerb_db_iterate __P((int (*func) (/* ??? */), char *arg));
-                  
-int
-kerb_db_iterate (func, arg)
-     int (*func)();
-     char *arg;			/* void *, really */
+kerb_db_iterate (k_iter_proc_t func, void *arg)
 {
     datum key, contents;
     Principal *principal;
