@@ -34,22 +34,24 @@ or implied warranty.
 
 #include "kdb_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/kdb/krb_kdb_utils.c,v 1.6 1995/10/24 14:03:27 bg Exp $");
-
 #include <kdc.h>
 
+RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/kdb/krb_kdb_utils.c,v 1.7 1996/02/10 12:58:47 assar Exp $");
+
 long 
 kdb_get_master_key(int prompt, des_cblock (*master_key), struct des_ks_struct *master_key_sched)
 {
   int kfile;
 
-  if (prompt)  {
+  if (prompt == KDB_GET_PROMPT || prompt == KDB_GET_TWICE) {
 #ifdef NOENCRYPTION
       placebo_read_password(master_key,
-			    "\nEnter Kerberos master key: ", 0);
+			    "\nEnter Kerberos master key: ",
+			    prompt == KDB_GET_TWICE);
 #else
       des_read_password(master_key,
-			     "\nEnter Kerberos master key: ", 0);
+			     "\nEnter Kerberos master key: ",
+			prompt == KDB_GET_TWICE);
 #endif
       printf ("\n");
   }
