@@ -26,7 +26,7 @@ or implied warranty.
 
 #include "krb_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/krb/create_ticket.c,v 1.2 1995/10/24 10:14:14 bg Exp $");
+RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/krb/create_ticket.c,v 1.3 1995/11/09 00:40:00 assar Exp $");
 
 /*
  * Create ticket takes as arguments information that should be in a
@@ -78,7 +78,7 @@ RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/
  */
 
 int
-krb_create_ticket(KTEXT tkt, unsigned char flags, char *pname, char *pinstance, char *prealm, int32_t paddress, char *session, int16_t life, int32_t time_sec, char *sname, char *sinstance, des_cblock *key)
+krb_create_ticket(KTEXT tkt, unsigned char flags, char *pname, char *pinstance, char *prealm, int32_t paddress, void *session, int16_t life, int32_t time_sec, char *sname, char *sinstance, des_cblock *key)
                                 /* Gets filled in by the ticket */
                                 /* Various Kerberos flags */
                                 /* Principal's name */
