@@ -26,7 +26,7 @@ or implied warranty.
 
 #include "krb_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/krb/get_ad_tkt.c,v 1.4 1995/10/24 10:14:18 bg Exp $");
+RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/krb/get_ad_tkt.c,v 1.5 1995/12/07 10:55:24 assar Exp $");
 
 static int swap_bytes;
 
@@ -59,7 +59,7 @@ pkt_clen(KTEXT pkt)
     clen = (int) temp;
 
     if (krb_debug)
-	printf("Clen is %d\n",clen);
+	fprintf(stderr, "Clen is %d\n",clen);
     return(clen);
 }
 
@@ -219,7 +219,7 @@ get_ad_tkt(char *service, char *sinstance, char *realm, int lifetime)
     /* Attempt to decrypt it */
 
     des_key_sched(&cr.session,key_s);
-    if (krb_debug)  printf("About to do decryption ...");
+    if (krb_debug)  fprintf(stderr, "About to do decryption ...\n");
     des_pcbc_encrypt((des_cblock *)cip->dat,(des_cblock *)cip->dat,
                  (long) cip->length,key_s,&cr.session,0);
 #endif /* !NOENCRYPTION */
