@@ -26,7 +26,7 @@ or implied warranty.
 
 #include "krb_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/krb/in_tkt.c,v 1.3 1995/10/24 10:14:28 bg Exp $");
+RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/krb/in_tkt.c,v 1.5 1995/12/07 10:56:05 assar Exp $");
 
 #include <sys/file.h>
 #include <sys/types.h>
@@ -88,14 +88,14 @@ in_tkt(char *pname, char *pinst)
        This isn't a security problem, since the ticket file, if it already
        exists, has the right uid (== ruid) and mode. */
     if (me != metoo) {
-	if (setreuid(metoo, me) < 0) {
+	if (k_setreuid(metoo, me) < 0) {
 	    /* can't switch??? barf! */
 	    if (krb_debug)
 		perror("in_tkt: setreuid");
 	    return(KFAILURE);
 	} else
 	    if (krb_debug)
-		printf("swapped UID's %d and %d\n",(int)metoo,(int)me);
+	      fprintf(stderr, "swapped UID's %d and %d\n",(int)metoo,(int)me);
     }
     if ((tktfile = creat(file,0600)) < 0) {
 	if (krb_debug)
@@ -103,14 +103,14 @@ in_tkt(char *pname, char *pinst)
         return(KFAILURE);
     }
     if (me != metoo) {
-	if (setreuid(me, metoo) < 0) {
+	if (k_setreuid(me, metoo) < 0) {
 	    /* can't switch??? barf! */
 	    if (krb_debug)
 		perror("in_tkt: setreuid2");
 	    return(KFAILURE);
 	} else
 	    if (krb_debug)
-		printf("swapped UID's %d and %d\n",(int)me,(int)metoo);
+	      fprintf(stderr, "swapped UID's %d and %d\n",(int)me,(int)metoo);
     }
     if (lstat(file,&buf) < 0) {
 	if (krb_debug)
