@@ -8,13 +8,15 @@
  * It could probably do better than this, but currently this seem to
  * work good enough.  
  *
- * $Id: k_setreuid.c,v 1.2 1995/10/24 10:14:31 bg Exp $
+ * $Id: k_setreuid.c,v 1.3 1996/02/07 10:48:40 d91-jda Exp $
  *
  */
 
 #include "config.h"
+#include <unistd.h>
+#include <stdlib.h>
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/krb/k_setreuid.c,v 1.2 1995/10/24 10:14:31 bg Exp $");
+RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/krb/k_setreuid.c,v 1.3 1996/02/07 10:48:40 d91-jda Exp $");
 
 int k_setreuid(int r, int e)
 {
