@@ -1,11 +1,11 @@
 #include "krb_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/krb/k_strerror.c,v 1.2 1995/10/24 10:14:32 bg Exp $");
+RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/krb/k_strerror.c,v 1.3 1995/11/12 20:22:13 d91-jda Exp $");
 
 const char *
 k_strerror(int errnum)
 {
-#if HAVE_STRERROR
+#ifdef HAVE_STRERROR
   return strerror(errnum);
 #else
   extern int sys_nerr;
