@@ -26,7 +26,7 @@ or implied warranty.
 
 #include "krb_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/krb/krb_get_in_tkt.c,v 1.5 1995/10/24 10:14:38 bg Exp $");
+RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/krb/krb_get_in_tkt.c,v 1.7 1996/02/07 10:48:43 d91-jda Exp $");
 
 #include <sys/time.h>
 
@@ -63,13 +63,8 @@ pkt_clen(KTEXT pkt)
  * using the key returned by key_proc.
  */
 
-static int decrypt_tkt(user, instance, realm, arg, key_proc, cipp)
-  char *user;
-  char *instance;
-  char *realm;
-  char *arg;
-  int (*key_proc)();
-  KTEXT *cipp;
+static int decrypt_tkt(char *user, char *instance, char *realm, void *arg, 
+		       key_proc_t key_proc, KTEXT *cipp)
 {
     KTEXT cip = *cipp;
     des_cblock key;		/* Key for decrypting cipher */
@@ -83,7 +78,7 @@ static int decrypt_tkt(user, instance, realm, arg, key_proc, cipp)
     
     {
 	register int rc;
-	rc = (*key_proc)(user,instance,realm,arg,key);
+	rc = key_proc(user,instance,realm,arg,&key);
 	if (rc)
 	    return(rc);
     }
@@ -142,17 +137,9 @@ static int decrypt_tkt(user, instance, realm, arg, key_proc, cipp)
  */
 
 int
-krb_get_in_tkt(user, instance, realm, service, sinstance, life,
-	       key_proc, decrypt_proc, arg)
-    char *user;
-    char *instance;
-    char *realm;
-    char *service;
-    char *sinstance;
-    int life;
-    int (*key_proc)();
-    int (*decrypt_proc)();
-    char *arg;
+krb_get_in_tkt(char *user, char *instance, char *realm, 
+	       char *service, char *sinstance, int life, key_proc_t key_proc, 
+	       decrypt_proc_t decrypt_proc, void *arg)
 {
     KTEXT_ST pkt_st;
     KTEXT pkt = &pkt_st;	/* Packet to KDC */
@@ -316,6 +303,7 @@ krb_get_in_tkt(user, instance, realm, service, sinstance, life,
 					   code */
     }
 
+#if 0
     /* initialize ticket cache */
     if (in_tkt(user,instance) != KSUCCESS)
 	return(INTK_ERR);
@@ -325,5 +313,23 @@ krb_get_in_tkt(user, instance, realm, service, sinstance, life,
 				  lifetime, kvno, tkt, t_local.tv_sec)))
 	return(kerror);
 
-    return(INTK_OK);
+#endif
+
+
+    /* initialize ticket cache */
+
+    if (tf_create(TKT_FILE) != KSUCCESS)
+	return(INTK_ERR);
+
+    if (tf_put_pname(user) != KSUCCESS ||
+	tf_put_pinst(instance) != KSUCCESS) {
+	tf_close();
+	return(INTK_ERR);
+    }
+
+    kerror = tf_save_cred(s_name, s_instance, rlm, ses, lifetime, kvno,
+			  tkt, t_local.tv_sec);
+    tf_close();
+
+    return(kerror);
 }
