@@ -26,7 +26,7 @@ or implied warranty.
 
 #include "krb_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/krb/log.c,v 1.3 1995/10/24 10:14:41 bg Exp $");
+RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/krb/log.c,v 1.6 1996/02/10 20:01:16 assar Exp $");
 
 #include <sys/time.h>
 
@@ -57,7 +57,7 @@ static is_open;
  */
 
 /*VARARGS1 */
-void log(const char *format, ...)
+void k_log(const char *format, ...)
 {
     FILE *logfile;
     time_t now;
@@ -71,8 +71,8 @@ void log(const char *format, ...)
     (void) time(&now);
     tm = localtime(&now);
 
-    fprintf(logfile,"%2d-%s-%02d %02d:%02d:%02d ",tm->tm_mday,
-            month_sname(tm->tm_mon + 1),tm->tm_year,
+    fprintf(logfile,"%2d-%s-%04d %02d:%02d:%02d ",tm->tm_mday,
+            month_sname(tm->tm_mon + 1),tm->tm_year + 1900,
             tm->tm_hour, tm->tm_min, tm->tm_sec);
     vfprintf(logfile, format, ap);
     fprintf(logfile,"\n");
@@ -115,8 +115,8 @@ new_log(time_t t, char *string)
     if (t) {
         tm = localtime(&t);
 
-        fprintf(logfile,"\n%2d-%s-%02d %02d:%02d:%02d  %s",tm->tm_mday,
-                month_sname(tm->tm_mon + 1),tm->tm_year,
+        fprintf(logfile,"\n%2d-%s-%04d %02d:%02d:%02d  %s",tm->tm_mday,
+                month_sname(tm->tm_mon + 1),tm->tm_year + 1900,
                 tm->tm_hour, tm->tm_min, tm->tm_sec, string);
     }
     else {
