@@ -39,7 +39,7 @@ or implied warranty.
 
 #include "krb_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/krb/mk_priv.c,v 1.3 1995/10/24 10:14:43 bg Exp $");
+RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/krb/mk_priv.c,v 1.4 1995/12/07 11:00:56 assar Exp $");
 
 #include <sys/types.h>
 #include <netinet/in.h>
@@ -184,7 +184,7 @@ krb_mk_priv(u_char *in, u_char *out, u_int32_t length, struct des_ks_struct *sch
      */
     cksum =  des_quad_cksum(q,NULL,p-q,0,key);
     if (krb_debug)
-        printf("\ncksum = %u",cksum);
+      fprintf(stderr, "\ncksum = %u",cksum);
     /* stuff checksum */
     bcopy((char *) &cksum,(char *) p,sizeof(cksum));
     p += sizeof(cksum);
