@@ -38,7 +38,7 @@ or implied warranty.
 
 #include "krb_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/krb/mk_safe.c,v 1.3 1995/10/24 10:14:47 bg Exp $");
+RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/krb/mk_safe.c,v 1.4 1995/12/07 11:01:32 assar Exp $");
 
 /* system include files */
 #include <sys/types.h>
@@ -165,7 +165,7 @@ krb_mk_safe(u_char *in, u_char *out, u_int32_t length, des_cblock (*key), struct
     cksum = des_quad_cksum((des_cblock *)q,big_cksum,p-q,2,key);
 #endif
     if (krb_debug)
-        printf("\ncksum = %u",(u_int) cksum);
+      fprintf(stderr, "\ncksum = %u",(u_int) cksum);
 
     /* stuff checksum */
     bcopy((char *)big_cksum,(char *)p,sizeof(big_cksum));
