@@ -1,6 +1,6 @@
 /*
  * $Source: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/krb/stime.c,v $
- * $Author: d91-jda $
+ * $Author: assar $
  *
  * Copyright 1985, 1986, 1987, 1988 by the Massachusetts Institute
  * of Technology.
@@ -18,7 +18,7 @@
  * since the beginning of time (midnight 1 Jan 1970 GMT), return
  * a string containing the local time in the form:
  *
- * "25-Jan-88 10:17:56"
+ * "25-Jan-1988 10:17:56"
  */
 
 char *stime(long int *t)
@@ -29,8 +29,8 @@ char *stime(long int *t)
     char *month_sname(int n);
 
     tm = localtime(t);
-    (void) sprintf(st,"%2d-%s-%02d %02d:%02d:%02d",tm->tm_mday,
-                   month_sname(tm->tm_mon + 1),tm->tm_year,
+    (void) sprintf(st,"%2d-%s-%04d %02d:%02d:%02d",tm->tm_mday,
+                   month_sname(tm->tm_mon + 1),tm->tm_year + 1900,
                    tm->tm_hour, tm->tm_min, tm->tm_sec);
     return st;
 }
