@@ -26,7 +26,7 @@ or implied warranty.
 
 #include "krb_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/krb/tf_util.c,v 1.2 1995/10/24 10:15:00 bg Exp $");
+RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/krb/tf_util.c,v 1.4 1996/01/08 16:31:12 d91-jda Exp $");
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -42,6 +42,7 @@ RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/
 #define TF_LCK_RETRY ((unsigned)2)	/* seconds to sleep before
 					 * retry if ticket file is
 					 * locked */
+#define	TF_LCK_RETRY_COUNT	(50)	/* number of retries	*/
 
 #ifdef TKT_SHMEM
 static char *krb_shm_addr = 0;
@@ -137,6 +138,8 @@ tf_init(char *tf_name, int rw)
     int     wflag;
     uid_t   me, getuid(void);
     struct stat stat_buf;
+    int		i_retry;
+
 #ifdef TKT_SHMEM
     char shmidname[MaxPathLen]; 
     FILE *sfp;
@@ -161,10 +164,24 @@ tf_init(char *tf_name, int rw)
 	default:
 	    return TKT_FIL_ACC;
 	}
+    /* The old code tried to guess when the calling program was
+     * running set-uid, this is now removed - the kerberos library
+     * does not (or shouldn't) know anything about user-ids.
+
+     * All library functions now assume that the right userids are set
+     * upon entry, therefore there is no need to test permissions like
+     * before. If the file is openable, just open it.
+     */
+
+    if(!S_ISREG(stat_buf.st_mode))
+      return TKT_FIL_ACC;
+
+#if 0
     me = getuid();
     if ((stat_buf.st_uid != me && me != 0) ||
 	((stat_buf.st_mode & S_IFMT) != S_IFREG))
 	return TKT_FIL_ACC;
+#endif
 #ifdef TKT_SHMEM
     (void) strcpy(shmidname, tf_name);
     (void) strcat(shmidname, ".shm");
@@ -225,19 +242,23 @@ tf_init(char *tf_name, int rw)
 #endif /* TKT_SHMEM */
     
     if (wflag) {
-	fd = open(tf_name, O_RDWR, 0600);
-	if (fd < 0) {
-	    return TKT_FIL_ACC;
-	}
+      fd = open(tf_name, O_RDWR, 0600);
+      if (fd < 0) {
+	return TKT_FIL_ACC;
+      }
+      for (i_retry = 0; i_retry < TF_LCK_RETRY_COUNT; i_retry++) {
 	if (k_flock(fd, K_LOCK_EX | K_LOCK_NB) < 0) {
-	    sleep(TF_LCK_RETRY);
-	    if (k_flock(fd, K_LOCK_EX | K_LOCK_NB) < 0) {
-		(void) close(fd);
-		fd = -1;
-		return TKT_FIL_LCK;
-	    }
+	  if (krb_debug)
+	    fprintf(stderr, "tf_init: retry %d of write lock of `%s'.\n",
+		    i_retry, tf_name);
+	  sleep (TF_LCK_RETRY);
+	} else {
+	  return KSUCCESS;		/* all done */
 	}
-	return KSUCCESS;
+      }
+      (void) close (fd);
+      fd = -1;
+      return TKT_FIL_LCK;
     }
     /*
      * Otherwise "wflag" is not set and the ticket file should be opened
@@ -248,9 +269,62 @@ tf_init(char *tf_name, int rw)
     if (fd < 0) {
 	return TKT_FIL_ACC;
     }
-    if (k_flock(fd, K_LOCK_SH | K_LOCK_NB) < 0) {
-	sleep(TF_LCK_RETRY);
+
+    for (i_retry = 0; i_retry < TF_LCK_RETRY_COUNT; i_retry++) {
 	if (k_flock(fd, K_LOCK_SH | K_LOCK_NB) < 0) {
+	  if (krb_debug)
+	    fprintf(stderr, "tf_init: retry %d of read lock of `%s'.\n",
+		    i_retry, tf_name);
+	  sleep (TF_LCK_RETRY);
+	} else {
+	  return KSUCCESS;		/* all done */
+	}
+      }
+    /* failure */
+    (void) close(fd);
+    fd = -1;
+    return TKT_FIL_LCK;
+}
+
+/*
+ * tf_create() should be called when creating a new ticket file.
+ * The only argument is the name of the ticket file.
+ * After calling this, it should be possible to use other tf_* functions.
+ *
+ * New algoritm for creating ticket file:
+ * 1. try to erase contents of existing file.
+ * 2. try to remove old file.
+ * 3. try to open with O_CREAT and O_EXCL
+ * 4. if this fails, someone has created a file in between 1 and 2 and
+ *    we should fail.  Otherwise, all is wonderful.
+ */
+
+int
+tf_create(char *tf_name)
+{
+    struct stat statbuf;
+    char garbage[BUFSIZ];
+
+    fd = open(tf_name, O_RDWR, 0);
+    if (fd) {
+	if (fstat (fd, &statbuf) == 0) {
+	    int i;
+
+	    for (i = 0; i < statbuf.st_size; i += sizeof(garbage))
+		write (fd, garbage, sizeof(garbage));
+	}
+	close (fd);
+    }
+
+    if (unlink (tf_name) && errno != ENOENT)
+	return TKT_FIL_ACC;
+
+    fd = open(tf_name, O_RDWR | O_CREAT | O_EXCL, 0600);
+    if (fd < 0)
+	return TKT_FIL_ACC;
+    if (k_flock(fd, K_LOCK_EX | K_LOCK_NB) < 0) {
+	sleep(TF_LCK_RETRY);
+	if (k_flock(fd, K_LOCK_EX | K_LOCK_NB) < 0) {
 	    (void) close(fd);
 	    fd = -1;
 	    return TKT_FIL_LCK;
@@ -277,7 +351,31 @@ tf_get_pname(char *p)
 	return TKT_FIL_INI;
     }
     if (tf_gets(p, ANAME_SZ) < 2)	/* can't be just a null */
+      {
+	if (krb_debug) fprintf (stderr, "tf_get_pname: pname < 2.\n");
 	return TKT_FIL_FMT;
+      }
+    return KSUCCESS;
+}
+
+/*
+ * tf_put_pname() sets the principal's name in the ticket file. Call
+ * after tf_create().
+ */
+
+int
+tf_put_pname(char *p)
+{
+    unsigned count;
+
+    if (fd < 0) {
+	if (krb_debug)
+	    fprintf(stderr, "tf_put_pname called before tf_create.\n");
+	return TKT_FIL_INI;
+    }
+    count = strlen(p)+1;
+    if (write(fd,p,count) != count)
+        return(KFAILURE);
     return KSUCCESS;
 }
 
@@ -300,7 +398,31 @@ tf_get_pinst(char *inst)
 	return TKT_FIL_INI;
     }
     if (tf_gets(inst, INST_SZ) < 1)
+      {
+	if (krb_debug) fprintf (stderr, "tf_get_pinst: inst_sz < 1.\n");
 	return TKT_FIL_FMT;
+      }
+    return KSUCCESS;
+}
+
+/*
+ * tf_put_pinst writes the principal's instance to the ticket file.
+ * Call after tf_create.
+ */
+
+int
+tf_put_pinst(char *inst)
+{
+    unsigned count;
+
+    if (fd < 0) {
+	if (krb_debug)
+	    fprintf(stderr, "tf_put_pinst called before tf_create.\n");
+	return TKT_FIL_INI;
+    }
+    count = strlen(inst)+1;
+    if (write(fd,inst,count) != count)
+        return(KFAILURE);
     return KSUCCESS;
 }
 
@@ -329,27 +451,32 @@ tf_get_cred(CREDENTIALS *c)
     if ((k_errno = tf_gets(c->service, SNAME_SZ)) < 2)
 	switch (k_errno) {
 	case TOO_BIG:
+	  if (krb_debug) fprintf (stderr, "tf_get_cred: too big service cred.\n");
 	case 1:		/* can't be just a null */
-	    tf_close();
-	    return TKT_FIL_FMT;
+	  tf_close();
+	  if (krb_debug) fprintf (stderr, "tf_get_cred: null service cred.\n");
+	  return TKT_FIL_FMT;
 	case 0:
-	    return EOF;
+	  return EOF;
 	}
     if ((k_errno = tf_gets(c->instance, INST_SZ)) < 1)
 	switch (k_errno) {
 	case TOO_BIG:
-	    return TKT_FIL_FMT;
+	  if (krb_debug) fprintf (stderr, "tf_get_cred: too big instance cred.\n");
+	  return TKT_FIL_FMT;
 	case 0:
-	    return EOF;
+	  return EOF;
 	}
     if ((k_errno = tf_gets(c->realm, REALM_SZ)) < 2)
 	switch (k_errno) {
 	case TOO_BIG:
+	  if (krb_debug) fprintf (stderr, "tf_get_cred: too big realm cred.\n");
 	case 1:		/* can't be just a null */
-	    tf_close();
-	    return TKT_FIL_FMT;
+	  tf_close();
+	  if (krb_debug) fprintf (stderr, "tf_get_cred: null realm cred.\n");
+	  return TKT_FIL_FMT;
 	case 0:
-	    return EOF;
+	  return EOF;
 	}
     if (
 	tf_read((char *) (c->session), DES_KEY_SZ) < 1 ||
@@ -363,6 +490,7 @@ tf_get_cred(CREDENTIALS *c)
 	tf_read((char *) &(c->issue_date), sizeof(c->issue_date)) < 1
 	) {
 	tf_close();
+	if (krb_debug) fprintf (stderr, "tf_get_cred: failed tf_read.\n");
 	return TKT_FIL_FMT;
     }
 #ifdef TKT_SHMEM
@@ -392,7 +520,7 @@ tf_close(void)
 	} else {
 	    krb_shm_addr = 0;
 	}
-#endif TKT_SHMEM
+#endif /* TKT_SHMEM */
 	(void) k_flock(fd, K_LOCK_UN);
 	(void) close(fd);
 	fd = -1;		/* see declaration of fd above */
