@@ -9,7 +9,7 @@
 #include "config.h"
 #include "protos.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/server/kerberos.c,v 1.10 1995/10/27 10:56:46 bg Exp $");
+RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/server/kerberos.c,v 1.16 1996/02/09 23:13:45 assar Exp $");
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -29,6 +29,10 @@ RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/serv
 #endif /* !HAVE_SYS_TIME_H */
 #endif /* !TIME_WITH_SYS_TIME */
 
+#ifdef HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif
+
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
@@ -53,8 +57,7 @@ RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/serv
 
 #include <kdc.h>
 
-static struct sockaddr_in sina = {AF_INET};
-int     f;
+#define MAX2(a, b) ((a > b) ? a : b)
 
 /* XXX several files in libkdb know about this */
 char *progname;
@@ -119,7 +122,7 @@ usage(void)
  */
 
 static void
-kerb_err_reply(struct sockaddr_in *client, KTEXT pkt, long int err, char *string)
+kerb_err_reply(int f, struct sockaddr_in *client, KTEXT pkt, long int err, char *string)
 {
     static KTEXT_ST e_pkt_st;
     KTEXT   e_pkt = &e_pkt_st;
@@ -129,7 +132,8 @@ kerb_err_reply(struct sockaddr_in *client, KTEXT pkt, long int err, char *string
     strcat(e_msg, string);
     cr_err_reply(e_pkt, req_name_ptr, req_inst_ptr, req_realm_ptr,
 		 req_time_ws, err, e_msg);
-    sendto(f, e_pkt->dat, e_pkt->length, 0, client, S_AD_SZ);
+    sendto(f, e_pkt->dat, e_pkt->length, 0, (struct sockaddr *)client,
+	   S_AD_SZ);
 }
 
 static void
@@ -166,7 +170,7 @@ strtime(u_int32_t *t)
     char *month_sname(int n);
 
     tm = localtime((time_t *)t);
-    (void) sprintf(st,"%2d-%s-%02d %02d:%02d:%02d",tm->tm_mday,
+    (void) sprintf(st,"%2d-%s-%04d %02d:%02d:%02d",tm->tm_mday,
                    month_sname(tm->tm_mon + 1),tm->tm_year,
                    tm->tm_hour, tm->tm_min, tm->tm_sec);
     return st;
@@ -246,7 +250,7 @@ set_tgtkey(char *r)
     if (!strcmp(lastrealm, r))
 	return (KSUCCESS);
 
-    log("Getting key for %s", r);
+    k_log("Getting key for %s", r);
 
     n = kerb_get_principal("krbtgt", r, p, 1, &more);
     if (n == 0)
@@ -262,7 +266,7 @@ set_tgtkey(char *r)
 }
 
 static void
-kerberos(struct sockaddr_in *client, KTEXT pkt)
+kerberos(int f, struct sockaddr_in *client, KTEXT pkt)
 {
     static KTEXT_ST rpkt_st;
     KTEXT   rpkt = &rpkt_st;
@@ -304,7 +308,7 @@ kerberos(struct sockaddr_in *client, KTEXT pkt)
 	"KRB prot version mismatch: KRB =%d request = %d",
 		  KRB_PROT_VERSION, req_version, 0);
 	/* send an error reply */
-	kerb_err_reply(client, pkt, KERB_ERR_PKT_VER, lt);
+	kerb_err_reply(f, client, pkt, KERB_ERR_PKT_VER, lt);
 	return;
     }
     msg_byte_order = req_msg_type & 1;
@@ -353,7 +357,7 @@ kerberos(struct sockaddr_in *client, KTEXT pkt)
 
 	    if ((i = check_princ(req_name_ptr, req_inst_ptr, 0,
 		&a_name_data))) {
-		kerb_err_reply(client, pkt, i, lt);
+		kerb_err_reply(f, client, pkt, i, lt);
 		return;
 	    }
 	    tk->length = 0;	/* init */
@@ -364,7 +368,7 @@ kerberos(struct sockaddr_in *client, KTEXT pkt)
 	    /* this does all the checking */
 	    if ((i = check_princ(service, instance, 0,
 		&s_name_data))) {
-		kerb_err_reply(client, pkt, i, lt);
+		kerb_err_reply(f, client, pkt, i, lt);
 		return;
 	    }
 	    /* Bound requested lifetime with service and user */
@@ -415,7 +419,8 @@ kerberos(struct sockaddr_in *client, KTEXT pkt)
 	    rpkt = create_auth_reply(req_name_ptr, req_inst_ptr,
 		req_realm_ptr, req_time_ws, 0, a_name_data.exp_date,
 		a_name_data.key_version, ciph);
-	    sendto(f, rpkt->dat, rpkt->length, 0, client, S_AD_SZ);
+	    sendto(f, rpkt->dat, rpkt->length, 0, (struct sockaddr *)client,
+		   S_AD_SZ);
 	    bzero(&a_name_data, sizeof(a_name_data));
 	    bzero(&s_name_data, sizeof(s_name_data));
 	    break;
@@ -433,18 +438,18 @@ kerberos(struct sockaddr_in *client, KTEXT pkt)
 	    tk->length = 0;
 	    k_flags = 0;	/* various kerberos flags */
 
-	    auth->length = 4 + strlen(pkt->dat + 3);
+	    auth->length = 4 + strlen((char *)pkt->dat + 3);
 	    auth->length += (int) *(pkt->dat + auth->length) +
 		(int) *(pkt->dat + auth->length + 1) + 2;
 
 	    bcopy(pkt->dat, auth->dat, auth->length);
 
-	    strncpy(tktrlm, auth->dat + 3, REALM_SZ);
+	    strncpy(tktrlm, (char *)auth->dat + 3, REALM_SZ);
 	    if (set_tgtkey(tktrlm)) {
 		lt = klog(L_ERR_UNK,
 		    "FAILED realm %s unknown. Host: %s ",
 			  tktrlm, inet_ntoa(client_host));
-		kerb_err_reply(client, pkt, kerno, lt);
+		kerb_err_reply(f, client, pkt, kerno, lt);
 		return;
 	    }
 	    kerno = krb_rd_req(auth, "ktbtgt", tktrlm, client_host.s_addr,
@@ -453,7 +458,7 @@ kerberos(struct sockaddr_in *client, KTEXT pkt)
 	    if (kerno) {
 		klog(L_ERR_UNK, "FAILED krb_rd_req from %s: %s",
 		     inet_ntoa(client_host), krb_err_txt[kerno]);
-		kerb_err_reply(client, pkt, kerno, "krb_rd_req failed");
+		kerb_err_reply(f, client, pkt, kerno, "krb_rd_req failed");
 		return;
 	    }
 	    ptr = (char *) pkt->dat + auth->length;
@@ -471,19 +476,19 @@ kerberos(struct sockaddr_in *client, KTEXT pkt)
 		 service, instance, 0);
 
 	    if (strcmp(ad->prealm, tktrlm)) {
-		kerb_err_reply(client, pkt, KERB_ERR_PRINCIPAL_UNKNOWN,
+		kerb_err_reply(f, client, pkt, KERB_ERR_PRINCIPAL_UNKNOWN,
 		     "Can't hop realms");
 		return;
 	    }
 	    if (!strcmp(service, "changepw")) {
-		kerb_err_reply(client, pkt, KERB_ERR_PRINCIPAL_UNKNOWN,
+		kerb_err_reply(f, client, pkt, KERB_ERR_PRINCIPAL_UNKNOWN,
 		     "Can't authorize password changed based on TGT");
 		return;
 	    }
 	    kerno = check_princ(service, instance, req_life,
 		&s_name_data);
 	    if (kerno) {
-		kerb_err_reply(client, pkt, kerno, lt);
+		kerb_err_reply(f, client, pkt, kerno, lt);
 		return;
 	    }
 	    /* Bound requested lifetime with service and user */
@@ -524,7 +529,8 @@ kerberos(struct sockaddr_in *client, KTEXT pkt)
 	    rpkt = create_auth_reply(ad->pname, ad->pinst,
 				     ad->prealm, time_ws,
 				     0, 0, 0, ciph);
-	    sendto(f, rpkt->dat, rpkt->length, 0, client, S_AD_SZ);
+	    sendto(f, rpkt->dat, rpkt->length, 0, (struct sockaddr *)client,
+		   S_AD_SZ);
 	    bzero(&s_name_data, sizeof(s_name_data));
 	    break;
 	}
@@ -603,19 +609,89 @@ static void check_db_age(void)
     }
 }
 
-int
-main(int argc, char **argv)
+static int
+k_getportbyname (const char *service, const char *proto, int default_port)
+{
+    struct servent *sp;
+
+    if ((sp = getservbyname (service, proto)) == NULL) {
+	 fprintf (stderr, "%s: %s/%s unknown service, "
+		  "using default port %d\n", progname, service, proto,
+		  ntohs(default_port));
+	 return default_port;
+    } else
+	 return sp->s_port;
+}
+
+static int
+mksocket(struct in_addr addr, const char *service, int port)
+{
+    struct sockaddr_in sina;
+    int     on = 1;
+    int sock;
+
+    if ((sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
+	fprintf(stderr, "%s: Can't open socket\n", progname);
+	exit(1);
+    }
+    if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on)) < 0)
+	fprintf(stderr, "%s: setsockopt (SO_REUSEADDR)\n", progname);
+    bzero(&sina, sizeof(sina));
+    sina.sin_family = AF_INET;
+    sina.sin_port   = port;
+    sina.sin_addr   = addr;
+    if (bind(sock, (struct sockaddr *)&sina, sizeof(sina)) < 0) {
+	fprintf(stderr, "%s: Can't bind socket '%s/udp' (%d)\n", progname, service, ntohs(sina.sin_port));
+	exit(1);
+    }
+    return sock;
+}
+
+static void
+process (int sock)
 {
     struct sockaddr_in from;
-    register int n;
-    int     on = 1;
-    int     child;
-    struct servent *sp;
     int     fromlen;
+    int n;
     static KTEXT_ST pkt_st;
     KTEXT   pkt = &pkt_st;
     int     kerror;
+
+    fromlen = S_AD_SZ;
+    n = recvfrom(sock, pkt->dat, MAX_PKT_LEN, 0, (struct sockaddr *)&from,
+		 &fromlen);
+    if (n > 0) {
+	 pkt->length = n;
+	 pkt->mbz = 0; /* force zeros to catch runaway strings */
+	 /* see what is left in the input queue */
+	 ioctl(sock, FIONREAD, &q_bytes);
+	 gettimeofday(&kerb_time, NULL);
+	 q_n++;
+	 max_q_n = max(max_q_n, q_n);
+	 n_packets++;
+	 klog(L_NET_INFO,
+	      "q_byt %d, q_n %d, rd_byt %d, mx_q_b %d, mx_q_n %d, n_pkt %d",
+	      q_bytes, q_n, n, max_q_bytes, max_q_n, n_packets, 0);
+	 max_q_bytes = max(max_q_bytes, q_bytes);
+	 if (!q_bytes)
+	      q_n = 0;	/* reset consecutive packets */
+	 kerberos(sock, &from, pkt);
+    } else
+	 klog(L_NET_ERR,
+	      "%s: bad recvfrom n = %d errno = %d", progname, n, errno, 0);
+}
+
+int
+main(int argc, char **argv)
+{
+    int     child;
     int c;
+    int *fds;
+    int nfds;
+    int maxfd = 0;
+    int i;
+    int n;
+    int     kerror;
     extern char *optarg;
     extern int optind;
 
@@ -714,22 +790,48 @@ main(int argc, char **argv)
 	exit(1);
     }
 
-    if ((sp = getservbyname("kerberos", "udp")) == 0) {
-	fprintf(stderr, "%s: udp/kerberos unknown service\n", progname);
-	exit(1);
-    }
-    sina.sin_port = sp->s_port;
+    /*
+     * Yes this looks backwards but it has to be this way to enable a
+     * smooth migration to the new port 88.
+     */
+    {
+      int p1, p2;
+      struct in_addr *a;
+      int j;
 
-    if ((f = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
-	fprintf(stderr, "%s: Can't open socket\n", progname);
-	exit(1);
-    }
-    if (setsockopt(f, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on)) < 0)
-	fprintf(stderr, "%s: setsockopt (SO_REUSEADDR)\n", progname);
+      p1 = k_getportbyname ("kerberos", "udp", htons(750));
+      p2 = k_getportbyname ("kerberos-sec", "udp", htons(88));
 
-    if (bind(f, &sina, sizeof(sina)) < 0) {
-	fprintf(stderr, "%s: Can't bind socket\n", progname);
-	exit(1);
+      if (p1 == p2)
+	{
+	  fprintf(stderr, "Either define kerberos/udp as 88\n");
+	  fprintf(stderr, "      and kerberos-sec/udp as 750\n");
+	  fprintf(stderr, "or the other way around!");
+	  exit(1);
+	}
+
+      nfds = k_get_all_addrs (&a);
+      if (nfds < 0) {
+	   struct in_addr any;
+
+	   any.s_addr = INADDR_ANY;
+
+	   fprintf (stderr, "Could not get local addresses, "
+		    "binding to INADDR_ANY\n");
+	   nfds = 1;
+	   a = malloc(sizeof(*a) * nfds);
+	   memcpy(a, &any, sizeof(struct in_addr));
+      }
+      nfds *= 2;
+      fds = malloc(sizeof(*fds) * nfds);
+      j = 0;
+      for (i = 0; i < nfds/2; i++) {
+	   fds[j++] = mksocket (a[i], "kerberos", p1);
+	   maxfd = MAX2(maxfd, fds[j-1]);
+	   fds[j++] = mksocket (a[i], "kerberos-sec", p2);
+	   maxfd = MAX2(maxfd, fds[j-1]);
+      }
+      free (a);
     }
     /* do all the database and cache inits */
     if ((n = kerb_init())) {
@@ -788,26 +890,16 @@ main(int argc, char **argv)
     }
     /* receive loop */
     for (;;) {
-	fromlen = S_AD_SZ;
-	n = recvfrom(f, pkt->dat, MAX_PKT_LEN, 0, &from, &fromlen);
-	if (n > 0) {
-	    pkt->length = n;
-	    pkt->mbz = 0; /* force zeros to catch runaway strings */
-	    /* see what is left in the input queue */
-	    ioctl(f, FIONREAD, &q_bytes);
-	    gettimeofday(&kerb_time, NULL);
-	    q_n++;
-	    max_q_n = max(max_q_n, q_n);
-	    n_packets++;
-	    klog(L_NET_INFO,
-	 "q_byt %d, q_n %d, rd_byt %d, mx_q_b %d, mx_q_n %d, n_pkt %d",
-		 q_bytes, q_n, n, max_q_bytes, max_q_n, n_packets, 0);
-	    max_q_bytes = max(max_q_bytes, q_bytes);
-	    if (!q_bytes)
-		q_n = 0;	/* reset consecutive packets */
-	    kerberos(&from, pkt);
-	} else
-	    klog(L_NET_ERR,
-	    "%s: bad recvfrom n = %d errno = %d", progname, n, errno, 0);
-    }
+        int sock;
+	int ret;
+        fd_set readfds;
+
+	FD_ZERO(&readfds);
+	for (i = 0; i < nfds; ++i)
+	     FD_SET(fds[i], &readfds);
+	ret = select(maxfd + 1, &readfds, 0, 0, 0);
+	for (i = 0; i < nfds; ++i)
+	     if (FD_ISSET(fds[i], &readfds))
+		  process (fds[i]);
+   }
 }
