@@ -24,7 +24,7 @@ provided "as is" without express or implied warranty.
 
 #include "slav_locl.h"
 
-RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/slave/kpropd.c,v 1.9 1995/10/24 16:57:36 bg Exp $");
+RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/slave/kpropd.c,v 1.11 1995/12/07 13:06:00 assar Exp $");
 
 #include "kprop.h"
 
@@ -86,7 +86,7 @@ recv_auth (int in, int out, int private, struct sockaddr_in *remote, struct sock
     }
     length = ntohl (length);
     if (length > sizeof buf) {
-      sprintf (errmsg, "kpropd: read length %ld, bigger than buf %d",
+      sprintf (errmsg, "kpropd: read length %d, bigger than buf %d",
 	       length, (int)(sizeof(buf)));
       klog (L_KRB_PERR, errmsg);
       SlowDeath();
@@ -219,7 +219,7 @@ main(int argc, char **argv)
     klog(L_KRB_PERR, errmsg);
     SlowDeath();
   }
-  if (bind(s, &sin, sizeof sin) < 0) {
+  if (bind(s, (struct sockaddr *)&sin, sizeof sin) < 0) {
     sprintf(errmsg, "kpropd: bind: %s", k_strerror(errno));
     klog(L_KRB_PERR, errmsg);
     SlowDeath();
