@@ -1,5 +1,5 @@
 /*
- * $Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/util/et/error_message.c,v 1.1.1.1 1995/10/23 11:20:37 d91-jda Exp $
+ * $Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/util/et/error_message.c,v 1.3 1996/02/06 21:37:55 bg Exp $
  * $Source: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/util/et/error_message.c,v $
  * $Locker:  $
  *
@@ -9,21 +9,46 @@
  * For copyright info, see "mit-sipb-copyright.h".
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif /* HAVE_CONFIG_H */
+
 #include <stdio.h>
 #include <string.h>
 #include "error_table.h"
 #include "mit-sipb-copyright.h"
 #include "internal.h"
 
+#if 0
 static const char rcsid[] =
-    "$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/util/et/error_message.c,v 1.1.1.1 1995/10/23 11:20:37 d91-jda Exp $";
+    "$Id: error_message.c,v 1.3 1996/02/06 21:37:55 bg Exp $";
 static const char copyright[] =
     "Copyright 1986, 1987, 1988 by the Student Information Processing Board\nand the department of Information Systems\nof the Massachusetts Institute of Technology";
+#endif
 
 static char buffer[25];
 
 struct et_list * _et_list = (struct et_list *) NULL;
 
+#ifndef HAVE_STRERROR
+extern char const *const sys_errlist[];
+extern const int sys_nerr;
+
+static const char *
+strerror(int err)
+{
+    if (0 <= err && err < sys_nerr)
+	return sys_errlist[err];
+    else {
+	static char buf[64];
+
+	sprintf(buf, "Error %d", err);
+	return buf;
+    }
+}
+	
+#endif /* HAVE_STRERROR */
+
 const char * error_message (code)
 long	code;
 {
@@ -36,10 +61,7 @@ long	code;
     offset = code & ((1<<ERRCODE_RANGE)-1);
     table_num = code - offset;
     if (!table_num) {
-	if (offset < sys_nerr)
-	    return(sys_errlist[offset]);
-	else
-	    goto oops;
+	return strerror(offset);
     }
     for (et = _et_list; et; et = et->next) {
 	if (et->table->base == table_num) {
