@@ -12,7 +12,7 @@
 static const char copyright[] =
     "Copyright 1987,1988 by Student Information Processing Board, Massachusetts Institute of Technology";
 static const char rcsid_et_name_c[] =
-    "$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/util/et/et_name.c,v 1.1.1.1 1995/10/23 11:20:38 d91-jda Exp $";
+    "$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/util/et/et_name.c,v 1.2 1996/02/07 10:48:49 d91-jda Exp $";
 #endif
 
 static const char char_set[] =
@@ -20,8 +20,7 @@ static const char char_set[] =
 
 static char buf[6];
 
-const char * error_table_name(num)
-    int num;
+const char * error_table_name(int num)
 {
     int ch;
     int i;
