@@ -1,5 +1,5 @@
 /*
- * $Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/util/et/init_et.c,v 1.1.1.1 1995/10/23 11:20:38 d91-jda Exp $
+ * $Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/util/et/init_et.c,v 1.3 1996/02/07 10:48:50 d91-jda Exp $
  * $Source: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/util/et/init_et.c,v $
  * $Locker:  $
  *
@@ -9,20 +9,16 @@
  * For copyright info, see mit-sipb-copyright.h.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif /* HAVE_CONFIG_H */
 #include <stdio.h>
+#include <stdlib.h>
+#include <errno.h>
 #include "error_table.h"
 #include "mit-sipb-copyright.h"
 
-#ifndef __STDC__
-#define const
-#endif
-
-#ifndef	lint
-static const char rcsid_init_et_c[] =
-    "$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/util/et/init_et.c,v 1.1.1.1 1995/10/23 11:20:38 d91-jda Exp $";
-#endif
-
-extern char *malloc(), *realloc();
+RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/util/et/init_et.c,v 1.3 1996/02/07 10:48:50 d91-jda Exp $");
 
 struct foobar {
     struct et_list etl;
@@ -31,10 +27,7 @@ struct foobar {
 
 extern struct et_list * _et_list;
 
-int init_error_table(msgs, base, count)
-    const char * const * msgs;
-    int base;
-    int count;
+int init_error_table(const char **msgs, int base, int count)
 {
     struct foobar * new_et;
 
