@@ -1,7 +1,7 @@
 /*
  * Listener loop for subsystem library libss.a.
  *
- *	$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/util/ss/listen.c,v 1.2 1995/10/21 11:45:30 assar Exp $
+ *	$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/util/ss/listen.c,v 1.5 1996/02/11 16:06:53 assar Exp $
  *	$Locker:  $
  * 
  * Copyright 1987, 1988 by MIT Student Information Processing Board
@@ -9,37 +9,35 @@
  * For copyright information, see copyright.h.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif /* HAVE_CONFIG_H */
 #include "copyright.h"
 #include "ss_internal.h"
 #include <stdio.h>
-#ifndef NPOSIX
 #include <string.h>
-#else
-#include <strings.h>
-#define memcpy(a, b, c) bcopy(b, a, c)
-#endif
 #include <setjmp.h>
 #include <signal.h>
+#ifdef HAVE_SIGACTION_H
+#include <sigaction.h>
+#endif /* HAVE_SIGACTION_H */
 #include <sys/param.h>
+#include <sys/ioctl.h>
 #ifdef BSD
 #include <sgtty.h>
 #endif
 
 #ifndef	lint
 static char const rcs_id[] =
-    "$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/util/ss/listen.c,v 1.2 1995/10/21 11:45:30 assar Exp $";
-#endif
-
-#ifndef NPOSIX
-#define sigtype void
-#else
-#define sigtype int
+    "$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/util/ss/listen.c,v 1.5 1996/02/11 16:06:53 assar Exp $";
 #endif
 
 static ss_data *current_info;
 static jmp_buf listen_jmpb;
 
-static sigtype print_prompt()
+int ss_execute_line (int sci_idx, char *line_ptr);
+
+static RETSIGTYPE print_prompt(int sig)
 {
 #ifdef BSD
     /* put input into a reasonable mode */
@@ -55,14 +53,13 @@ static sigtype print_prompt()
     (void) fflush(stdout);
 }
 
-static sigtype listen_int_handler()
+static RETSIGTYPE listen_int_handler(int sig)
 {
     putc('\n', stdout);
     longjmp(listen_jmpb, 1);
 }
 
-int ss_listen (sci_idx)
-    int sci_idx;
+int ss_listen (int sci_idx)
 {
     register char *cp;
     register ss_data *info;
@@ -77,21 +74,21 @@ int ss_listen (sci_idx)
     struct sigaction isig, csig, nsig, osig;
     sigset_t nmask, omask;
 #else
-    register sigtype (*sig_cont)();
-    sigtype (*sig_int)(), (*old_sig_cont)();
+    register RETSIGTYPE (*sig_cont)();
+    RETSIGTYPE (*sig_int)(), (*old_sig_cont)();
     int mask;
 #endif
 
     current_info = info = ss_info(sci_idx);
     info->abort = 0;
 #ifndef NPOSIX
-    csig.sa_handler = (sigtype (*)())0;
+    csig.sa_handler = (RETSIGTYPE (*)())0;
     
     sigemptyset(&nmask);
     sigaddset(&nmask, SIGINT);
     sigprocmask(SIG_BLOCK, &nmask, &omask);
 #else
-    sig_cont = (sigtype (*)())0;
+    sig_cont = (RETSIGTYPE (*)())0;
     mask = sigblock(sigmask(SIGINT));
 #endif
 
@@ -114,25 +111,19 @@ int ss_listen (sci_idx)
     (void) sigsetmask(mask);
 #endif
     while(!info->abort) {
-	print_prompt();
+	print_prompt(0);
 	*end = '\0';
 #ifndef NPOSIX
 	nsig.sa_handler = listen_int_handler;	/* fgets is not signal-safe */
 	osig = csig;
 	sigaction(SIGCONT, &nsig, &csig);
-	if ((sigtype (*)())csig.sa_handler==(sigtype (*)())listen_int_handler)
+	if ((RETSIGTYPE (*)())csig.sa_handler==(RETSIGTYPE (*)())listen_int_handler)
 	    csig = osig;
 #else
 	old_sig_cont = sig_cont;
 	sig_cont = signal(SIGCONT, print_prompt);
-#ifdef mips
-	/* The mips compiler breaks on determining the types, we help */
-	if ( (sigtype *) sig_cont == (sigtype *) print_prompt)
-	    sig_cont = old_sig_cont;
-#else
 	if (sig_cont == print_prompt)
 	    sig_cont = old_sig_cont;
-#endif
 #endif
 	if (fgets(input, BUFSIZ, stdin) != input) {
 	    code = SS_ET_EOF;
@@ -180,19 +171,14 @@ egress:
     return code;
 }
 
-void ss_abort_subsystem(sci_idx, code)
-    int sci_idx;
+void ss_abort_subsystem(int sci_idx, int code)
 {
     ss_info(sci_idx)->abort = 1;
     ss_info(sci_idx)->exit_status = code;
     
 }
 
-int ss_quit(argc, argv, sci_idx, infop)
-    int argc;
-    char **argv;
-    int sci_idx;
-    pointer infop;
+void ss_quit(int argc, char **argv, int sci_idx, pointer infop)
 {
     ss_abort_subsystem(sci_idx, 0);
 }
