@@ -1,7 +1,7 @@
 /*
  * prompt.c: Routines for retrieving and setting a prompt.
  *
- * $Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/util/ss/prompt.c,v 1.1.1.1 1995/10/23 11:20:47 d91-jda Exp $
+ * $Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/util/ss/prompt.c,v 1.2 1996/02/07 10:49:06 d91-jda Exp $
  * $Locker:  $
  *
  * Copyright 1987, 1988 by MIT Student Information Processing Board
@@ -14,11 +14,9 @@
 #include "ss_internal.h"
 
 static const char rcsid[] =
-    "$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/util/ss/prompt.c,v 1.1.1.1 1995/10/23 11:20:47 d91-jda Exp $";
+    "$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/util/ss/prompt.c,v 1.2 1996/02/07 10:49:06 d91-jda Exp $";
 
-ss_set_prompt(sci_idx, new_prompt)
-     int sci_idx;
-     char *new_prompt;
+void ss_set_prompt(int sci_idx, char *new_prompt)
 {
      ss_info(sci_idx)->prompt = new_prompt;
 }
