@@ -10,24 +10,14 @@
 #include <ss/mit-sipb-copyright.h>
 #include <ss/ss_err.h>
 
-extern int errno;
-
-#ifdef __STDC__
-#define __SS_CONST const
-#define __SS_PROTO (int, const char * const *, int, void *)
-#else
-#define __SS_CONST
-#define __SS_PROTO ()
-#endif
-
-typedef __SS_CONST struct _ss_request_entry {
-    __SS_CONST char * __SS_CONST *command_names; /* whatever */
-    void (* __SS_CONST function) __SS_PROTO; /* foo */
-    __SS_CONST char * __SS_CONST info_string;	/* NULL */
+typedef const struct _ss_request_entry {
+    const char * const *command_names; /* whatever */
+    void (* const function) (int, const char * const *, int, void *);
+    const char * const info_string;	/* NULL */
     int flags;			/* 0 */
 } ss_request_entry;
 
-typedef __SS_CONST struct _ss_request_table {
+typedef const struct _ss_request_table {
     int version;
     ss_request_entry *requests;
 } ss_request_table;
@@ -36,7 +26,7 @@ typedef __SS_CONST struct _ss_request_table {
 
 typedef struct _ss_rp_options {	/* DEFAULT VALUES */
     int version;		/* SS_RP_V1 */
-    void (*unknown) __SS_PROTO;	/* call for unknown command */
+    void (*unknown) (int, const char * const *, int, void *);	/* call for unknown command */
     int allow_suspend;
     int catch_int;
 } ss_rp_options;
@@ -46,18 +36,13 @@ typedef struct _ss_rp_options {	/* DEFAULT VALUES */
 #define SS_OPT_DONT_LIST	0x0001
 #define SS_OPT_DONT_SUMMARIZE	0x0002
 
-void ss_help __SS_PROTO;
+void ss_help (int, const char * const *, int, void *);
 char *ss_current_request();
 char *ss_name();
-#ifdef __STDC__
 void ss_error (int, long, char const *, ...);
 void ss_perror (int, long, char const *);
 int ss_create_invocation(char *, char *, char *, ss_request_table *, int *);
 int ss_listen(int);
-#else
-void ss_error ();
-void ss_perror ();
-#endif
 void ss_abort_subsystem();
 extern ss_request_table ss_std_requests;
 #endif /* _ss_h */
